@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -28,9 +28,10 @@
 ////////////////////////////////////////////////////////////
 // Headers
 ////////////////////////////////////////////////////////////
-#include <cstdlib>
-#include <SFML/Audio/AudioResource.hpp>
+#include <SFML/System/Resource.hpp>
 #include <SFML/System/Vector3.hpp>
+#include <SFML/Audio/AudioResource.hpp>
+#include <cstdlib>
 
 
 namespace sf
@@ -38,7 +39,7 @@ namespace sf
 class SoundBuffer;
 
 ////////////////////////////////////////////////////////////
-/// Sound defines the properties of the sound such as position,
+/// Sound defines the properties of a sound such as position,
 /// volume, pitch, etc.
 ////////////////////////////////////////////////////////////
 class SFML_API Sound : public AudioResource
@@ -71,7 +72,7 @@ public :
     /// \param Position : Position (0, 0, 0 by default)
     ///
     ////////////////////////////////////////////////////////////
-    Sound(const SoundBuffer& Buffer, bool Loop = false, float Pitch = 1.f, float Volume = 100.f, const Vector3f& Position = Vector3f(0, 0, 0));
+    explicit Sound(const SoundBuffer& Buffer, bool Loop = false, float Pitch = 1.f, float Volume = 100.f, const Vector3f& Position = Vector3f(0, 0, 0));
 
     ////////////////////////////////////////////////////////////
     /// Copy constructor
@@ -158,6 +159,16 @@ public :
     ////////////////////////////////////////////////////////////
     void SetPosition(const Vector3f& Position);
 
+    ////////////////////////////////////////////////////////////
+    /// Make the sound's position relative to the listener's
+    /// position, or absolute.
+    /// The default value is false (absolute)
+    ///
+    /// \param Relative : True to set the position relative, false to set it absolute
+    ///
+    ////////////////////////////////////////////////////////////
+    void SetRelativeToListener(bool Relative);
+
     ////////////////////////////////////////////////////////////
     /// Set the minimum distance - closer than this distance,
     /// the listener will hear the sound at its maximum volume.
@@ -178,6 +189,14 @@ public :
     ////////////////////////////////////////////////////////////
     void SetAttenuation(float Attenuation);
 
+    ////////////////////////////////////////////////////////////
+    /// Set the current playing position of the sound
+    ///
+    /// \param TimeOffset : New playing position, expressed in seconds
+    ///
+    ////////////////////////////////////////////////////////////
+    void SetPlayingOffset(float TimeOffset);
+
     ////////////////////////////////////////////////////////////
     /// Get the source buffer
     ///
@@ -218,6 +237,15 @@ public :
     ////////////////////////////////////////////////////////////
     Vector3f GetPosition() const;
 
+    ////////////////////////////////////////////////////////////
+    /// Tell if the sound's position is relative to the listener's
+    /// position, or if it's absolute
+    ///
+    /// \return True if the position is relative, false if it's absolute
+    ///
+    ////////////////////////////////////////////////////////////
+    bool IsRelativeToListener() const;
+
     ////////////////////////////////////////////////////////////
     /// Get the minimum distance
     ///
@@ -260,6 +288,15 @@ public :
     ////////////////////////////////////////////////////////////
     Sound& operator =(const Sound& Other);
 
+    ////////////////////////////////////////////////////////////
+    /// Reset the internal buffer
+    ///
+    /// This function is for internal use only, you don't have
+    /// to use it.
+    ///
+    ////////////////////////////////////////////////////////////
+    void ResetBuffer();
+
 private :
 
     friend class SoundStream;
@@ -267,8 +304,8 @@ private :
     ////////////////////////////////////////////////////////////
     // Member data
     ////////////////////////////////////////////////////////////
-    unsigned int       mySource; ///< OpenAL source identifier
-    const SoundBuffer* myBuffer; ///< Sound buffer bound to the source
+    unsigned int             mySource; ///< OpenAL source identifier
+    ResourcePtr<SoundBuffer> myBuffer; ///< Sound buffer bound to the source
 };
 
 } // namespace sf
