@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -36,7 +36,7 @@
 namespace sf
 {
 ////////////////////////////////////////////////////////////
-/// Specialized sfSoundRecorder which saves the captured
+/// Specialized SoundRecorder which saves the captured
 /// audio data into a sound buffer
 ////////////////////////////////////////////////////////////
 class SFML_API SoundBufferRecorder : public SoundRecorder
@@ -49,15 +49,27 @@ public :
     /// \return Constant reference to the sound buffer
     ///
     ////////////////////////////////////////////////////////////
-    const SoundBuffer& GetBuffer();
+    const SoundBuffer& GetBuffer() const;
 
 private :
 
     ////////////////////////////////////////////////////////////
-    /// /see SoundBuffer::ProcessSamples
+    /// /see SoundBuffer::OnStart
     ///
     ////////////////////////////////////////////////////////////
-    virtual bool ProcessSamples(const Int16* Samples, std::size_t SamplesCount);
+    virtual bool OnStart();
+
+    ////////////////////////////////////////////////////////////
+    /// /see SoundBuffer::OnProcessSamples
+    ///
+    ////////////////////////////////////////////////////////////
+    virtual bool OnProcessSamples(const Int16* Samples, std::size_t SamplesCount);
+
+    ////////////////////////////////////////////////////////////
+    /// /see SoundBuffer::OnStop
+    ///
+    ////////////////////////////////////////////////////////////
+    virtual void OnStop();
 
     ////////////////////////////////////////////////////////////
     // Member data
