@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -42,18 +42,6 @@ class SFML_API SoundRecorder : private Thread
 {
 public :
 
-    typedef bool (*FuncType)(const Int16*, std::size_t, void*);
-
-    ////////////////////////////////////////////////////////////
-    /// Construct the sound recorder with a callback function
-    /// for processing captured samples
-    ///
-    /// \param Callback : Callback for processing captured samples
-    /// \param UserData : Data to pass to the callback function (NULL by default)
-    ///
-    ////////////////////////////////////////////////////////////
-    SoundRecorder(FuncType Callback, void* UserData = NULL);
-
     ////////////////////////////////////////////////////////////
     /// Virtual destructor
     ///
@@ -103,6 +91,14 @@ protected :
 
 private :
 
+    ////////////////////////////////////////////////////////////
+    /// Start recording audio data
+    ///
+    /// \return False to abort recording audio data, true to start
+    ///
+    ////////////////////////////////////////////////////////////
+    virtual bool OnStart();
+
     ////////////////////////////////////////////////////////////
     /// Process a new chunk of recorded samples
     ///
@@ -112,10 +108,16 @@ private :
     /// \return False to stop recording audio data, true to continue
     ///
     ////////////////////////////////////////////////////////////
-    virtual bool ProcessSamples(const Int16* Samples, std::size_t SamplesCount);
+    virtual bool OnProcessSamples(const Int16* Samples, std::size_t SamplesCount) = 0;
 
     ////////////////////////////////////////////////////////////
-    /// /see sfThread::Run
+    /// Stop recording audio data
+    ///
+    ////////////////////////////////////////////////////////////
+    virtual void OnStop();
+
+    ////////////////////////////////////////////////////////////
+    /// /see Thread::Run
     ///
     ////////////////////////////////////////////////////////////
     virtual void Run();
@@ -135,8 +137,6 @@ private :
     ////////////////////////////////////////////////////////////
     // Member data
     ////////////////////////////////////////////////////////////
-    FuncType           myCallback;    ///< Callback for processing captured samples
-    void*              myUserData;    ///< User data to pass to the processing callback
     std::vector<Int16> mySamples;     ///< Buffer to store captured samples
     unsigned int       mySampleRate;  ///< Sample rate
     bool               myIsCapturing; ///< Capturing state
