@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -38,7 +38,7 @@ namespace sf
 ////////////////////////////////////////////////////////////
 /// SoundStream is a streamed sound, ie samples are acquired
 /// while the sound is playing. Use it for big sounds that would
-/// require hundreds of MB in memory (see sfMusic),
+/// require hundreds of MB in memory (see Music),
 /// or for streaming sound from the network
 ////////////////////////////////////////////////////////////
 class SFML_API SoundStream : private Thread, private Sound
@@ -53,11 +53,13 @@ public :
     using Sound::SetPitch;
     using Sound::SetVolume;
     using Sound::SetPosition;
+    using Sound::SetRelativeToListener;
     using Sound::SetMinDistance;
     using Sound::SetAttenuation;
     using Sound::GetPitch;
     using Sound::GetVolume;
     using Sound::GetPosition;
+    using Sound::IsRelativeToListener;
     using Sound::GetMinDistance;
     using Sound::GetAttenuation;
 
@@ -112,6 +114,31 @@ public :
     ////////////////////////////////////////////////////////////
     Status GetStatus() const;
 
+    ////////////////////////////////////////////////////////////
+    /// Get the current playing position of the stream
+    ///
+    /// \return Current playing position, expressed in seconds
+    ///
+    ////////////////////////////////////////////////////////////
+    float GetPlayingOffset() const;
+
+    ////////////////////////////////////////////////////////////
+    /// Set the stream loop state.
+    /// This parameter is disabled by default
+    ///
+    /// \param Loop : True to play in loop, false to play once
+    ///
+    ////////////////////////////////////////////////////////////
+    void SetLoop(bool Loop);
+
+    ////////////////////////////////////////////////////////////
+    /// Tell whether or not the stream is looping
+    ///
+    /// \return True if the music is looping, false otherwise
+    ///
+    ////////////////////////////////////////////////////////////
+    bool GetLoop() const;
+
 protected :
 
     ////////////////////////////////////////////////////////////
@@ -156,21 +183,43 @@ private :
     virtual bool OnGetData(Chunk& Data) = 0;
 
     ////////////////////////////////////////////////////////////
-    /// Clean up stream internal resources
+    /// Fill a new buffer with audio data, and push it to the
+    /// playing queue
+    ///
+    /// \param Buffer : Number of the buffer to fill (in [0, BuffersCount])
+    ///
+    /// \return True if the derived class has requested to stop
     ///
     ////////////////////////////////////////////////////////////
-    void CleanUp();
+    bool FillAndPushBuffer(unsigned int BufferNum);
+
+    ////////////////////////////////////////////////////////////
+    /// Fill the buffers queue with all available buffers
+    ///
+    /// \return True if the derived class has requested to stop
+    ///
+    ////////////////////////////////////////////////////////////
+    bool FillQueue();
+
+    ////////////////////////////////////////////////////////////
+    /// Clear the queue of any remaining buffers
+    ///
+    ////////////////////////////////////////////////////////////
+    void ClearQueue();
 
     enum {BuffersCount = 3};
 
     ////////////////////////////////////////////////////////////
     // Member data
     ////////////////////////////////////////////////////////////
-    bool          myIsStreaming;           ///< Streaming state (true = playing, false = stopped)
-    unsigned int  myBuffers[BuffersCount]; ///< Sound buffers used to store temporary audio data
-    unsigned int  myChannelsCount;         ///< Number of channels (1 = mono, 2 = stereo, ...)
-    unsigned int  mySampleRate;            ///< Frequency (samples / second)
-    unsigned long myFormat;                ///< Format of the internal sound buffers
+    bool          myIsStreaming;              ///< Streaming state (true = playing, false = stopped)
+    unsigned int  myBuffers[BuffersCount];    ///< Sound buffers used to store temporary audio data
+    unsigned int  myChannelsCount;            ///< Number of channels (1 = mono, 2 = stereo, ...)
+    unsigned int  mySampleRate;               ///< Frequency (samples / second)
+    unsigned long myFormat;                   ///< Format of the internal sound buffers
+    bool          myLoop;                     ///< Loop flag (true to loop, false to play once)
+    unsigned int  mySamplesProcessed;         ///< Number of buffers processed since beginning of the stream
+    bool          myEndBuffers[BuffersCount]; ///< Each buffer is marked as "end buffer" or not, for proper duration calculation
 };
 
 } // namespace sf
