@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -35,7 +35,7 @@
 
 namespace sf
 {
-class RenderWindow;
+class RenderTarget;
 
 ////////////////////////////////////////////////////////////
 /// Enumerate the blending modes for drawable objects
@@ -280,6 +280,28 @@ public :
     ////////////////////////////////////////////////////////////
     void Rotate(float Angle);
 
+    ////////////////////////////////////////////////////////////
+    /// Transform a point from global coordinates into local coordinates
+    /// (ie it applies the inverse of object's center, translation, rotation and scale to the point)
+    ///
+    /// \param Point : Point to transform
+    ///
+    /// \return Transformed point
+    ///
+    ////////////////////////////////////////////////////////////
+    sf::Vector2f TransformToLocal(const sf::Vector2f& Point) const;
+
+    ////////////////////////////////////////////////////////////
+    /// Transform a point from local coordinates into global coordinates
+    /// (ie it applies the object's center, translation, rotation and scale to the point)
+    ///
+    /// \param Point : Point to transform
+    ///
+    /// \return Transformed point
+    ///
+    ////////////////////////////////////////////////////////////
+    sf::Vector2f TransformToGlobal(const sf::Vector2f& Point) const;
+
 protected :
 
     ////////////////////////////////////////////////////////////
@@ -290,37 +312,47 @@ protected :
     ////////////////////////////////////////////////////////////
     const Matrix3& GetMatrix() const;
 
+    ////////////////////////////////////////////////////////////
+    /// Get the inverse transform matrix of the drawable
+    ///
+    /// \return Inverse transform matrix
+    ///
+    ////////////////////////////////////////////////////////////
+    const Matrix3& GetInverseMatrix() const;
+
 private :
 
-    friend class RenderWindow;
+    friend class RenderTarget;
 
     ////////////////////////////////////////////////////////////
     /// Draw the object into the specified window
     ///
-    /// \param Window : Window into which draw the object
+    /// \param Target : Target into which render the object
     ///
     ////////////////////////////////////////////////////////////
-    void Draw(const RenderWindow& Window) const;
+    void Draw(RenderTarget& Target) const;
 
     ////////////////////////////////////////////////////////////
     /// Render the specific geometry of the object
     ///
-    /// \param Window : Window into which render the object
+    /// \param Target : Target into which render the object
     ///
     ////////////////////////////////////////////////////////////
-    virtual void Render(const RenderWindow& Window) const = 0;
+    virtual void Render(RenderTarget& Target) const = 0;
 
     ////////////////////////////////////////////////////////////
     // Member data
     ////////////////////////////////////////////////////////////
-    Vector2f        myPosition;   ///< Position of the object on screen
-    Vector2f        myScale;      ///< Scale of the object
-    Vector2f        myCenter;     ///< Origin of translation / rotation / scaling of the object
-    float           myRotation;   ///< Orientation of the object, in degrees
-    Color           myColor;      ///< Overlay color of the object
-    Blend::Mode     myBlendMode;  ///< Blending mode
-    mutable bool    myNeedUpdate; ///< Do we need to recompute the transform matrix ?
-    mutable Matrix3 myMatrix;     ///< Precomputed transform matrix gathering the translation / rotation / scale / center
+    Vector2f        myPosition;      ///< Position of the object on screen
+    Vector2f        myScale;         ///< Scale of the object
+    Vector2f        myCenter;        ///< Origin of translation / rotation / scaling of the object
+    float           myRotation;      ///< Orientation of the object, in degrees
+    Color           myColor;         ///< Overlay color of the object
+    Blend::Mode     myBlendMode;     ///< Blending mode
+    mutable bool    myNeedUpdate;    ///< Do we need to recompute the transform matrix ?
+    mutable bool    myInvNeedUpdate; ///< Do we need to recompute the inverse transform matrix ?
+    mutable Matrix3 myMatrix;        ///< Precomputed transform matrix gathering the translation / rotation / scale / center
+    mutable Matrix3 myInvMatrix;     ///< Precomputed inverse transform matrix gathering the translation / rotation / scale / center
 };
 
 } // namespace sf
