@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -28,8 +28,8 @@
 ////////////////////////////////////////////////////////////
 // Headers
 ////////////////////////////////////////////////////////////
+#include <SFML/System/Resource.hpp>
 #include <SFML/Graphics/Color.hpp>
-#include <SFML/Graphics/VideoResource.hpp>
 #include <SFML/Graphics/Rect.hpp>
 #include <string>
 #include <vector>
@@ -37,11 +37,13 @@
 
 namespace sf
 {
+class RenderWindow;
+
 ////////////////////////////////////////////////////////////
 /// Image is the low-level class for loading and
 /// manipulating images
 ////////////////////////////////////////////////////////////
-class SFML_API Image : public VideoResource
+class SFML_API Image : public Resource<Image>
 {
 public :
 
@@ -150,17 +152,30 @@ public :
     void CreateMaskFromColor(Color ColorKey, Uint8 Alpha = 0);
 
     ////////////////////////////////////////////////////////////
-    /// Resize the image - warning : this function does not scale the image,
-    /// it just adjusts its size (add padding or remove pixels)
+    /// Copy pixels from another image onto this one.
+    /// This function does a slow pixel copy and should only
+    /// be used at initialization time
     ///
-    /// \param Width :  New width
-    /// \param Height : New height
-    /// \param Col :    Color to assign to new pixels (black by default)
-    ///
-    /// \return True if resize has been successful
+    /// \param Source :     Source image to copy
+    /// \param DestX :      X coordinate of the destination position
+    /// \param DestY :      Y coordinate of the destination position
+    /// \param SourceRect : Sub-rectangle of the source image to copy (empty by default - entire image)
+    /// \param ApplyAlpha : Should the copy take in account the source transparency? (false by default)
     ///
     ////////////////////////////////////////////////////////////
-    bool Resize(unsigned int Width, unsigned int Height, Color Col = Color(0, 0, 0, 255));
+    void Copy(const Image& Source, unsigned int DestX, unsigned int DestY, const IntRect& SourceRect = IntRect(0, 0, 0, 0), bool ApplyAlpha = false);
+
+    ////////////////////////////////////////////////////////////
+    /// Create the image from the current contents of the
+    /// given window
+    ///
+    /// \param Window :     Window to capture
+    /// \param SourceRect : Sub-rectangle of the screen to copy (empty by default - entire image)
+    ///
+    /// \return True if copy was successful
+    ///
+    ////////////////////////////////////////////////////////////
+    bool CopyScreen(RenderWindow& Window, const IntRect& SourceRect = IntRect(0, 0, 0, 0));
 
     ////////////////////////////////////////////////////////////
     /// Change the color of a pixel
@@ -175,10 +190,10 @@ public :
     ////////////////////////////////////////////////////////////
     /// Get a pixel from the image
     ///
-    /// \param X :   X coordinate of pixel in the image
-    /// \param Y :   Y coordinate of pixel in the image
+    /// \param X : X coordinate of pixel in the image
+    /// \param Y : Y coordinate of pixel in the image
     ///
-    /// \return Color of pixel (x, y)
+    /// \return Color of pixel (X, Y)
     ///
     ////////////////////////////////////////////////////////////
     const Color& GetPixel(unsigned int X, unsigned int Y) const;
@@ -236,13 +251,12 @@ public :
     /// Convert a subrect expressed in pixels, into float
     /// texture coordinates
     ///
-    /// \param Rect :   Sub-rectangle of image to convert
-    /// \param Adjust : Pass true to apply the half-texel adjustment
+    /// \param Rect : Sub-rectangle of image to convert
     ///
     /// \return Texture coordinates corresponding to the sub-rectangle
     ///
     ////////////////////////////////////////////////////////////
-    FloatRect GetTexCoords(const IntRect& Rect, bool Adjust = true) const;
+    FloatRect GetTexCoords(const IntRect& Rect) const;
 
     ////////////////////////////////////////////////////////////
     /// Get a valid texture size according to hardware support
@@ -275,10 +289,16 @@ private :
     bool CreateTexture();
 
     ////////////////////////////////////////////////////////////
-    /// Update the internal texture in video memory
-    ///
+    /// Make sure the texture in video memory is updated with the
+    /// array of pixels
     ////////////////////////////////////////////////////////////
-    void Update();
+    void EnsureTextureUpdate() const;
+
+    ////////////////////////////////////////////////////////////
+    /// Make sure the array of pixels is updated with the
+    /// texture in video memory
+    ////////////////////////////////////////////////////////////
+    void EnsureArrayUpdate() const;
 
     ////////////////////////////////////////////////////////////
     /// Reset the image attributes
@@ -287,22 +307,23 @@ private :
     void Reset();
 
     ////////////////////////////////////////////////////////////
-    /// /see VideoResource::DestroyVideoResources
+    /// Destroy the OpenGL texture
     ///
     ////////////////////////////////////////////////////////////
-    virtual void DestroyVideoResources();
+    void DestroyTexture();
 
     ////////////////////////////////////////////////////////////
     // Member data
     ////////////////////////////////////////////////////////////
-    unsigned int       myWidth;         ///< Image width
-    unsigned int       myHeight;        ///< Image Height
-    unsigned int       myTextureWidth;  ///< Actual texture width (can be greater than image width because of padding)
-    unsigned int       myTextureHeight; ///< Actual texture height (can be greater than image height because of padding)
-    std::vector<Color> myPixels;        ///< Pixels of the image
-    unsigned int       myTexture;       ///< Internal texture identifier
-    bool               myIsSmooth;      ///< Status if the smooth filter
-    mutable bool       myUpdated;       ///< Tells if the internal texture needs to be updated
+    unsigned int               myWidth;             ///< Image width
+    unsigned int               myHeight;            ///< Image Height
+    unsigned int               myTextureWidth;      ///< Actual texture width (can be greater than image width because of padding)
+    unsigned int               myTextureHeight;     ///< Actual texture height (can be greater than image height because of padding)
+    unsigned int               myTexture;           ///< Internal texture identifier
+    bool                       myIsSmooth;          ///< Status of the smooth filter
+    mutable std::vector<Color> myPixels;            ///< Pixels of the image
+    mutable bool               myNeedTextureUpdate; ///< Status of synchronization between pixels in central memory and the internal texture un video memory
+    mutable bool               myNeedArrayUpdate;   ///< Status of synchronization between pixels in central memory and the internal texture un video memory
 };
 
 } // namespace sf
