@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -30,7 +30,6 @@
 ////////////////////////////////////////////////////////////
 #include <SFML/Graphics/Drawable.hpp>
 #include <SFML/Graphics/Image.hpp>
-#include <SFML/Graphics/VideoResource.hpp>
 #include <istream>
 #include <map>
 #include <string>
@@ -41,7 +40,7 @@ namespace sf
 ////////////////////////////////////////////////////////////
 /// PostFX is used to apply a post effect to a window
 ////////////////////////////////////////////////////////////
-class SFML_API PostFX : public Drawable, public VideoResource
+class SFML_API PostFX : public Drawable
 {
 public :
 
@@ -151,10 +150,10 @@ public :
 protected :
 
     ////////////////////////////////////////////////////////////
-    /// /see sfDrawable::Render
+    /// /see Drawable::Render
     ///
     ////////////////////////////////////////////////////////////
-    virtual void Render(const RenderWindow& Window) const;
+    virtual void Render(RenderTarget& Target) const;
 
 private :
 
@@ -176,18 +175,17 @@ private :
     void CreateProgram();
 
     ////////////////////////////////////////////////////////////
-    /// /see sfVideoResource::DestroyVideoResources
-    ///
+    // Types
     ////////////////////////////////////////////////////////////
-    virtual void DestroyVideoResources();
+    typedef std::map<std::string, const Image*> TextureTable;
 
     ////////////////////////////////////////////////////////////
     // Member data
     ////////////////////////////////////////////////////////////
-    unsigned int                  myShaderProgram;  ///< OpenGL identifier for the program
-    std::map<std::string, Image*> mySamplers;       ///< Sampler names and textures in the effect
-    std::string                   myFragmentShader; ///< Fragment shader source code
-    mutable Image                 myFrameBuffer;    ///< Texture containing the current frame buffer
+    unsigned int  myShaderProgram;  ///< OpenGL identifier for the program
+    TextureTable  myTextures;       ///< Texture variables in the effect
+    std::string   myFragmentShader; ///< Fragment shader source code
+    mutable Image myFrameBuffer;    ///< Texture containing the current frame buffer
 };
 
 } // namespace sf
