@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -28,10 +28,8 @@
 ////////////////////////////////////////////////////////////
 // Headers
 ////////////////////////////////////////////////////////////
-#include <SFML/Graphics/Color.hpp>
 #include <SFML/Graphics/Image.hpp>
-#include <SFML/Graphics/View.hpp>
-#include <SFML/Graphics/Rect.hpp>
+#include <SFML/Graphics/RenderTarget.hpp>
 #include <SFML/Window/Window.hpp>
 #include <string>
 
@@ -44,7 +42,7 @@ class Drawable;
 /// Simple wrapper for sf::Window that allows easy
 /// 2D rendering
 ////////////////////////////////////////////////////////////
-class SFML_API RenderWindow : public Window
+class SFML_API RenderWindow : public Window, public RenderTarget
 {
 public :
 
@@ -59,7 +57,7 @@ public :
     ///
     /// \param Mode :        Video mode to use
     /// \param Title :       Title of the window
-    /// \param WindowStyle : Window style (Resize | Close by default)
+    /// \param WindowStyle : Window style, see sf::Style (Resize | Close by default)
     /// \param Params :      Creation parameters (see default constructor for default values)
     ///
     ////////////////////////////////////////////////////////////
@@ -81,12 +79,20 @@ public :
     virtual ~RenderWindow();
 
     ////////////////////////////////////////////////////////////
-    /// Draw something on the window
+    /// Get the width of the rendering region of the window
     ///
-    /// \param Object : Object to draw
+    /// \return Width in pixels
     ///
     ////////////////////////////////////////////////////////////
-    void Draw(const Drawable& Object) const;
+    virtual unsigned int GetWidth() const;
+
+    ////////////////////////////////////////////////////////////
+    /// Get the height of the rendering region of the window
+    ///
+    /// \return Height in pixels
+    ///
+    ////////////////////////////////////////////////////////////
+    virtual unsigned int GetHeight() const;
 
     ////////////////////////////////////////////////////////////
     /// Save the content of the window to an image
@@ -96,39 +102,6 @@ public :
     ////////////////////////////////////////////////////////////
     Image Capture() const;
 
-    ////////////////////////////////////////////////////////////
-    /// Change the background color of the window.
-    /// The default color is black
-    ///
-    /// \param Col : New background color
-    ///
-    ////////////////////////////////////////////////////////////
-    void SetBackgroundColor(const Color& Col);
-
-    ////////////////////////////////////////////////////////////
-    /// Change the current active view.
-    ///
-    /// \param NewView : New view to use (pass GetDefaultView() to set the default view)
-    ///
-    ////////////////////////////////////////////////////////////
-    void SetView(const View& NewView);
-
-    ////////////////////////////////////////////////////////////
-    /// Get the current view
-    ///
-    /// \return Current view active in the window
-    ///
-    ////////////////////////////////////////////////////////////
-    const View& GetView() const;
-
-    ////////////////////////////////////////////////////////////
-    /// Get the default view of the window for read / write
-    ///
-    /// \return Default view
-    ///
-    ////////////////////////////////////////////////////////////
-    View& GetDefaultView();
-
     ////////////////////////////////////////////////////////////
     /// Convert a point in window coordinates into view coordinates
     ///
@@ -141,19 +114,6 @@ public :
     ////////////////////////////////////////////////////////////
     sf::Vector2f ConvertCoords(unsigned int WindowX, unsigned int WindowY, const View* TargetView = NULL) const;
 
-    ////////////////////////////////////////////////////////////
-    /// Tell SFML to preserve external OpenGL states, at the expense of
-    /// more CPU charge. Use this function if you don't want SFML
-    /// to mess up your own OpenGL states (if any).
-    /// Don't enable state preservation if not needed, as it will allow
-    /// SFML to do internal optimizations and improve performances.
-    /// This parameter is false by default
-    ///
-    /// \param Preserve : True to preserve OpenGL states, false to let SFML optimize
-    ///
-    ////////////////////////////////////////////////////////////
-    void PreserveOpenGLStates(bool Preserve);
-
 private :
 
     ////////////////////////////////////////////////////////////
@@ -163,25 +123,10 @@ private :
     virtual void OnCreate();
 
     ////////////////////////////////////////////////////////////
-    /// /see Window::OnDisplay
+    /// /see RenderTarget::Activate
     ///
     ////////////////////////////////////////////////////////////
-    virtual void OnDisplay();
-
-    ////////////////////////////////////////////////////////////
-    /// Set the OpenGL render states needed for the SFML rendering
-    ///
-    ////////////////////////////////////////////////////////////
-    void SetRenderStates() const;
-
-    ////////////////////////////////////////////////////////////
-    // Member data
-    ////////////////////////////////////////////////////////////
-    Color        myBackgroundColor; ///< Background color
-    View         myDefaultView;     ///< Default view
-    const View*  myCurrentView;     ///< Current active view
-    bool         myPreserveStates;  ///< Should we preserve external OpenGL states ?
-    mutable bool myIsDrawing;       ///< True when Draw is called from inside, to allow some renderstates optimizations
+    virtual bool Activate(bool Active);
 };
 
 } // namespace sf
