@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -28,6 +28,7 @@
 ////////////////////////////////////////////////////////////
 // Headers
 ////////////////////////////////////////////////////////////
+#include <SFML/System/Resource.hpp>
 #include <SFML/Graphics/Drawable.hpp>
 #include <SFML/Graphics/Rect.hpp>
 
@@ -60,7 +61,7 @@ public :
     /// \param Col :      Color of the sprite (white by default)
     ///
     ////////////////////////////////////////////////////////////
-    Sprite(const Image& Img, const Vector2f& Position = Vector2f(0, 0), const Vector2f& Scale = Vector2f(1, 1), float Rotation = 0.f, const Color& Col = Color(255, 255, 255, 255));
+    explicit Sprite(const Image& Img, const Vector2f& Position = Vector2f(0, 0), const Vector2f& Scale = Vector2f(1, 1), float Rotation = 0.f, const Color& Col = Color(255, 255, 255, 255));
 
     ////////////////////////////////////////////////////////////
     /// Change the image of the sprite
@@ -140,6 +141,7 @@ public :
 
     ////////////////////////////////////////////////////////////
     /// Get the color of a given pixel in the sprite
+    /// (point is in local coordinates)
     ///
     /// \param X : X coordinate of the pixel to get
     /// \param Y : Y coordinate of the pixel to get
@@ -155,17 +157,17 @@ protected :
     /// /see Drawable::Render
     ///
     ////////////////////////////////////////////////////////////
-    virtual void Render(const RenderWindow& Window) const;
+    virtual void Render(RenderTarget& Target) const;
 
 private :
 
     ////////////////////////////////////////////////////////////
     // Member data
     ////////////////////////////////////////////////////////////
-    const Image* myImage;      ///< Image used to draw the sprite
-    IntRect      mySubRect;    ///< Sub-rectangle of source image to assign to the sprite
-    bool         myIsFlippedX; ///< Is the sprite flipped on the X axis ?
-    bool         myIsFlippedY; ///< Is the sprite flipped on the Y axis ?
+    ResourcePtr<Image> myImage;      ///< Image used to draw the sprite
+    IntRect            mySubRect;    ///< Sub-rectangle of source image to assign to the sprite
+    bool               myIsFlippedX; ///< Is the sprite flipped on the X axis ?
+    bool               myIsFlippedY; ///< Is the sprite flipped on the Y axis ?
 };
 
 } // namespace sf
