@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -36,6 +36,8 @@
 
 namespace sf
 {
+class RenderTarget;
+
 ////////////////////////////////////////////////////////////
 /// This class defines a view (position, size, etc.) ;
 /// you can consider it as a 2D camera
@@ -154,7 +156,7 @@ public :
 
 private :
 
-    friend class RenderWindow;
+    friend class RenderTarget;
 
     ////////////////////////////////////////////////////////////
     /// Get the projection matrix of the view
