@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -77,6 +77,14 @@ public :
     ////////////////////////////////////////////////////////////
     IPAddress(Uint8 Byte0, Uint8 Byte1, Uint8 Byte2, Uint8 Byte3);
 
+    ////////////////////////////////////////////////////////////
+    /// Construct the address from a 32-bits integer
+    ///
+    /// \param Address : 4 bytes of the address packed into a 32-bits integer
+    ///
+    ////////////////////////////////////////////////////////////
+    IPAddress(Uint32 Address);
+
     ////////////////////////////////////////////////////////////
     /// Tell if the address is a valid one
     ///
@@ -93,6 +101,14 @@ public :
     ////////////////////////////////////////////////////////////
     std::string ToString() const;
 
+    ////////////////////////////////////////////////////////////
+    /// Get an integer representation of the address
+    ///
+    /// \return 32-bits integer containing the 4 bytes of the address, in system endianness
+    ///
+    ////////////////////////////////////////////////////////////
+    Uint32 ToInteger() const;
+
     ////////////////////////////////////////////////////////////
     /// Get the computer's local IP address (from the LAN point of view)
     ///
@@ -107,10 +123,12 @@ public :
     /// distant website ; as a consequence, this function may be
     /// very slow -- use it as few as possible !
     ///
+    /// \param Timeout : Maximum time to wait, in seconds (0 by default : no timeout)
+    ///
     /// \return Public IP address
     ///
     ////////////////////////////////////////////////////////////
-    static IPAddress GetPublicAddress();
+    static IPAddress GetPublicAddress(float Timeout = 0.f);
 
     ////////////////////////////////////////////////////////////
     /// Comparison operator ==
