@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -89,7 +89,7 @@ public :
     /// \return The Index-th socket
     ///
     ////////////////////////////////////////////////////////////
-    Type GetSocketReady(unsigned int Index) const;
+    Type GetSocketReady(unsigned int Index);
 
 private :
 
