@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -29,12 +29,12 @@
 // Headers
 ////////////////////////////////////////////////////////////
 #include <SFML/Network/SocketHelper.hpp>
-#include <SFML/Network/Packet.hpp>
-#include <cstddef>
+#include <vector>
 
 
 namespace sf
 {
+class Packet;
 class IPAddress;
 template <typename> class Selector;
 
@@ -66,11 +66,12 @@ public :
     ///
     /// \param Port :        Port to use for transfers (warning : ports < 1024 are reserved)
     /// \param HostAddress : IP Address of the host to connect to
+    /// \param Timeout :     Maximum time to wait, in seconds (0 by default : no timeout) (this parameter is ignored for non-blocking sockets)
     ///
     /// \return True if operation has been successful
     ///
     ////////////////////////////////////////////////////////////
-    bool Connect(unsigned short Port, const IPAddress& HostAddress);
+    Socket::Status Connect(unsigned short Port, const IPAddress& HostAddress, float Timeout = 0.f);
 
     ////////////////////////////////////////////////////////////
     /// Listen to a specified port for incoming data or connections
@@ -204,15 +205,20 @@ private :
     ////////////////////////////////////////////////////////////
     /// Create the socket
     ///
+    /// \param Descriptor : System socket descriptor to use (0 by default -- create a new socket)
+    ///
     ////////////////////////////////////////////////////////////
-    void Create();
+    void Create(SocketHelper::SocketType Descriptor = 0);
 
     ////////////////////////////////////////////////////////////
     // Member data
     ////////////////////////////////////////////////////////////
     SocketHelper::SocketType mySocket;            ///< Socket descriptor
-    sf::Packet               myPendingPacket;     ///< Current pending packet, if any (in non-blocking mode)
-    Int32                    myPendingPacketSize; ///< Size of the current pending packet, if any (in non-blocking mode)
+    Uint32                   myPendingHeader;     ///< Data of the current pending packet header, if any
+    Uint32                   myPendingHeaderSize; ///< Size of the current pending packet header, if any
+    std::vector<char>        myPendingPacket;     ///< Data of the current pending packet, if any
+    Int32                    myPendingPacketSize; ///< Size of the current pending packet, if any
+    bool                     myIsBlocking;        ///< Is the socket blocking or non-blocking ?
 };
 
 } // namespace sf
