@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -29,12 +29,12 @@
 // Headers
 ////////////////////////////////////////////////////////////
 #include <SFML/Network/SocketHelper.hpp>
-#include <SFML/Network/Packet.hpp>
-#include <cstddef>
+#include <vector>
 
 
 namespace sf
 {
+class Packet;
 class IPAddress;
 template <typename> class Selector;
 
@@ -100,11 +100,12 @@ public :
     /// \param MaxSize :      Maximum number of bytes to read
     /// \param SizeReceived : Number of bytes received
     /// \param Address :      Address of the computer which sent the data
+    /// \param Port :         Port on which the remote computer sent the data
     ///
     /// \return Status code
     ///
     ////////////////////////////////////////////////////////////
-    Socket::Status Receive(char* Data, std::size_t MaxSize, std::size_t& SizeReceived, IPAddress& Address);
+    Socket::Status Receive(char* Data, std::size_t MaxSize, std::size_t& SizeReceived, IPAddress& Address, unsigned short& Port);
 
     ////////////////////////////////////////////////////////////
     /// Send a packet of data
@@ -124,11 +125,12 @@ public :
     ///
     /// \param PacketToReceive : Packet to fill with received data
     /// \param Address :         Address of the computer which sent the packet
+    /// \param Port :            Port on which the remote computer sent the data
     ///
     /// \return Status code
     ///
     ////////////////////////////////////////////////////////////
-    Socket::Status Receive(Packet& PacketToReceive, IPAddress& Address);
+    Socket::Status Receive(Packet& PacketToReceive, IPAddress& Address, unsigned short& Port);
 
     ////////////////////////////////////////////////////////////
     /// Close the socket
@@ -203,16 +205,21 @@ private :
     ////////////////////////////////////////////////////////////
     /// Create the socket
     ///
+    /// \param Descriptor : System socket descriptor to use (0 by default -- create a new socket)
+    ///
     ////////////////////////////////////////////////////////////
-    void Create();
+    void Create(SocketHelper::SocketType Descriptor = 0);
 
     ////////////////////////////////////////////////////////////
     // Member data
     ////////////////////////////////////////////////////////////
     SocketHelper::SocketType mySocket;            ///< Socket identifier
     unsigned short           myPort;              ///< Port to which the socket is bound
-    sf::Packet               myPendingPacket;     ///< Current pending packet, if any (in non-blocking mode)
-    Int32                    myPendingPacketSize; ///< Size of the current pending packet, if any (in non-blocking mode)
+    Uint32                   myPendingHeader;     ///< Data of the current pending packet header, if any
+    Uint32                   myPendingHeaderSize; ///< Size of the current pending packet header, if any
+    std::vector<char>        myPendingPacket;     ///< Data of the current pending packet, if any
+    Int32                    myPendingPacketSize; ///< Size of the current pending packet, if any
+    bool                     myIsBlocking;        ///< Is the socket blocking or non-blocking ?
 };
 
 } // namespace sf
