@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -142,7 +142,7 @@ namespace Key
         F15,
         Pause,
 
-        Count // For internal use
+        Count // Keep last -- total number of keyboard keys
     };
 }
 
@@ -160,7 +160,7 @@ namespace Mouse
         XButton1,
         XButton2,
 
-        Count // For internal use
+        ButtonCount // Keep last -- total number of mouse buttons
     };
 }
 
@@ -180,7 +180,13 @@ namespace Joy
         AxisV,
         AxisPOV,
 
-        Count // For internal use
+        AxisCount // Keep last -- total number of joystick axis
+    };
+
+    enum
+    {
+        Count       = 4, ///< Total number of supported joysticks
+        ButtonCount = 32 ///< Total number of supported joystick buttons
     };
 }
 
@@ -208,7 +214,7 @@ public :
     ////////////////////////////////////////////////////////////
     struct TextEvent
     {
-        Uint16 Unicode;
+        Uint32 Unicode;
     };
 
     ////////////////////////////////////////////////////////////
@@ -216,8 +222,8 @@ public :
     ////////////////////////////////////////////////////////////
     struct MouseMoveEvent
     {
-        unsigned int X;
-        unsigned int Y;
+        int X;
+        int Y;
     };
 
     ////////////////////////////////////////////////////////////
@@ -226,6 +232,8 @@ public :
     struct MouseButtonEvent
     {
         Mouse::Button Button;
+        int           X;
+        int           Y;
     };
 
     ////////////////////////////////////////////////////////////
@@ -280,9 +288,13 @@ public :
         MouseButtonPressed,
         MouseButtonReleased,
         MouseMoved,
+        MouseEntered,
+        MouseLeft,
         JoyButtonPressed,
         JoyButtonReleased,
-        JoyMoved
+        JoyMoved,
+
+        Count // Keep last -- total number of event types
     };
 
     ////////////////////////////////////////////////////////////
