@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -88,7 +88,7 @@ public :
     /// \return Current mouse left position, relative to owner window
     ///
     ////////////////////////////////////////////////////////////
-    unsigned int GetMouseX() const;
+    int GetMouseX() const;
 
     ////////////////////////////////////////////////////////////
     /// Get the mouse Y position
@@ -96,7 +96,7 @@ public :
     /// \return Current mouse top position, relative to owner window
     ///
     ////////////////////////////////////////////////////////////
-    unsigned int GetMouseY() const;
+    int GetMouseY() const;
 
     ////////////////////////////////////////////////////////////
     /// Get a joystick axis position
@@ -112,20 +112,26 @@ public :
 private :
 
     ////////////////////////////////////////////////////////////
-    /// /see sfWindowListener::OnEvent
+    /// /see WindowListener::OnEvent
     ///
     ////////////////////////////////////////////////////////////
     virtual void OnEvent(const Event& EventReceived);
 
+    ////////////////////////////////////////////////////////////
+    /// Reset all the states
+    ///
+    ////////////////////////////////////////////////////////////
+    void ResetStates();
+
     ////////////////////////////////////////////////////////////
     // Member data
     ////////////////////////////////////////////////////////////
-    bool          myKeys[Key::Count];            ///< Array containing the state of all keayboard keys
-    bool          myMouseButtons[Mouse::Count];  ///< Array containing the state of all mouse buttons
-    bool          myJoystickButtons[2][16];      ///< Array containing the state of all joysticks buttons
-    unsigned int  myMouseX;                      ///< Mouse position on X
-    unsigned int  myMouseY;                      ///< Mouse position on Y
-    float         myJoystickAxis[2][Joy::Count]; ///< Joysticks position on each axis
+    bool  myKeys[Key::Count];                              ///< Array containing the state of all keyboard keys
+    bool  myMouseButtons[Mouse::ButtonCount];              ///< Array containing the state of all mouse buttons
+    int   myMouseX;                                        ///< Mouse position on X
+    int   myMouseY;                                        ///< Mouse position on Y
+    bool  myJoystickButtons[Joy::Count][Joy::ButtonCount]; ///< Array containing the state of all joysticks buttons
+    float myJoystickAxis[Joy::Count][Joy::AxisCount];      ///< Joysticks position on each axis
 };
 
 } // namespace sf
