@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -42,14 +42,14 @@ namespace sf
     // Windows defines a void* handle (HWND)
     typedef void* WindowHandle;
 
-#elif defined(SFML_SYSTEM_LINUX)
+#elif defined(SFML_SYSTEM_LINUX) || defined(SFML_SYSTEM_FREEBSD)
 
     // Unix - X11 defines an unsigned integer handle (Window)
     typedef unsigned long WindowHandle;
 
 #elif defined(SFML_SYSTEM_MACOS)
 
-    // OSX - A GrafPort struct defined as WindowRef
+    // Mac OS X defines a void* handle (NSWindow)
 	typedef void* WindowHandle;
 
 #endif
