@@ -31,11 +31,11 @@ void PlaySound()
     // Loop while the sound is playing
     while (Sound.GetStatus() == sf::Sound::Playing)
     {
-        // Display the playing position
-        std::cout << "\rPlaying... " << std::fixed << std::setprecision(2) << Sound.GetPlayingOffset() << " sec   ";
-
         // Leave some CPU time for other processes
         sf::Sleep(0.1f);
+
+        // Display the playing position
+        std::cout << "\rPlaying... " << std::fixed << std::setprecision(2) << Sound.GetPlayingOffset() << " sec   ";
     }
     std::cout << std::endl << std::endl;
 }
@@ -60,13 +60,15 @@ void PlayMusic()
 
     // Play it
     Music.Play();
-    std::cout << "Playing... " << std::endl;
 
     // Loop while the music is playing
     while (Music.GetStatus() == sf::Music::Playing)
     {
         // Leave some CPU time for other processes
         sf::Sleep(0.1f);
+
+        // Display the playing position
+        std::cout << "\rPlaying... " << std::fixed << std::setprecision(2) << Music.GetPlayingOffset() << " sec   ";
     }
     std::cout << std::endl;
 }
