@@ -20,9 +20,6 @@ public :
     MyCanvas(wxWindow* Parent, wxWindowID Id, const wxPoint& Position, const wxSize& Size, long Style = 0) :
     wxSFMLCanvas(Parent, Id, Position, Size, Style)
     {
-        // Change background color
-        SetBackgroundColor(sf::Color(0, 128, 128));
-
         // Load an image and assign it to our sprite
         myImage.LoadFromFile("datas/wxwidgets/sfml.png");
         mySprite.SetImage(myImage);
@@ -44,6 +41,9 @@ private :
         if (GetInput().IsMouseButtonDown(sf::Mouse::Left))  mySprite.Rotate( GetFrameTime() * 50);
         if (GetInput().IsMouseButtonDown(sf::Mouse::Right)) mySprite.Rotate(-GetFrameTime() * 50);
 
+        // Clear the view
+        Clear(sf::Color(0, 128, 128));
+
         // Display the sprite in the view
         Draw(mySprite);
     }
@@ -79,13 +79,8 @@ public :
     ///
     ////////////////////////////////////////////////////////////
     MyFrame() :
-    wxFrame(NULL, wxID_ANY, wxT("SFML wxWidgets"), wxDefaultPosition, wxSize(680, 280))
+    wxFrame(NULL, wxID_ANY, wxT("SFML wxWidgets"), wxDefaultPosition, wxSize(440, 280))
     {
-        // Error log text box
-        wxTextCtrl* ErrorLog = new wxTextCtrl(this, wxID_ANY, wxT(""), wxPoint(440, 20), wxSize(200, 200), wxTE_MULTILINE | wxTE_READONLY | wxHSCROLL);
-        std::cerr.rdbuf(ErrorLog);
-        std::cerr << "-- This is the error log --" << std::endl;
-
         // Let's create a SFML view
         new MyCanvas(this, wxID_ANY, wxPoint(20, 20), wxSize(400, 200));
     }
