@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -39,7 +39,6 @@ namespace sf
 ////////////////////////////////////////////////////////////
 Music::Music(std::size_t BufferSize) :
 myFile    (NULL),
-myLoop    (false),
 myDuration(0.f),
 mySamples (BufferSize)
 {
@@ -52,6 +51,9 @@ mySamples (BufferSize)
 ////////////////////////////////////////////////////////////
 Music::~Music()
 {
+    // We must stop before destroying the file :)
+    Stop();
+
     delete myFile;
 }
 
@@ -61,6 +63,9 @@ Music::~Music()
 ////////////////////////////////////////////////////////////
 bool Music::OpenFromFile(const std::string& Filename)
 {
+    // First stop the music if it was already running
+    Stop();
+
     // Create the sound file implementation, and open it in read mode
     delete myFile;
     myFile = priv::SoundFile::CreateRead(Filename);
@@ -85,6 +90,9 @@ bool Music::OpenFromFile(const std::string& Filename)
 ////////////////////////////////////////////////////////////
 bool Music::OpenFromMemory(const char* Data, std::size_t SizeInBytes)
 {
+    // First stop the music if it was already running
+    Stop();
+
     // Create the sound file implementation, and open it in read mode
     delete myFile;
     myFile = priv::SoundFile::CreateRead(Data, SizeInBytes);
@@ -104,30 +112,12 @@ bool Music::OpenFromMemory(const char* Data, std::size_t SizeInBytes)
 }
 
 
-////////////////////////////////////////////////////////////
-/// Set the music loop state
-////////////////////////////////////////////////////////////
-void Music::SetLoop(bool Loop)
-{
-    myLoop = Loop;
-}
-
-
-////////////////////////////////////////////////////////////
-/// Tell whether or not the music is looping
-////////////////////////////////////////////////////////////
-bool Music::GetLoop() const
-{
-    return myLoop;
-}
-
-
 ////////////////////////////////////////////////////////////
 /// /see SoundStream::OnStart
 ////////////////////////////////////////////////////////////
 bool Music::OnStart()
 {
-    return myFile->Restart();
+    return myFile && myFile->Restart();
 }
 
 
@@ -136,30 +126,19 @@ bool Music::OnStart()
 ////////////////////////////////////////////////////////////
 bool Music::OnGetData(SoundStream::Chunk& Data)
 {
-    // Fill the chunk parameters
-    Data.Samples   = &mySamples[0];
-    Data.NbSamples = myFile->Read(&mySamples[0], mySamples.size());
-
-    // Check if we have reached the end of the audio file
-    if (Data.NbSamples < mySamples.size())
+    if (myFile)
     {
-        // Check if we must loop
-        if (myLoop)
-        {
-            if (OnStart())
-            {
-                // We succeeded to restart the audio playback
-                Data.NbSamples += myFile->Read(&mySamples[Data.NbSamples], mySamples.size() - Data.NbSamples);
-                return true;
-            }
-        }
+        // Fill the chunk parameters
+        Data.Samples   = &mySamples[0];
+        Data.NbSamples = myFile->Read(&mySamples[0], mySamples.size());
 
-        // No more audio samples to read : we stop the playback
+        // Check if we have reached the end of the audio file
+        return Data.NbSamples == mySamples.size();
+    }
+    else
+    {
         return false;
     }
-
-    // End of audio file has not been reached, continue playback
-    return true;
 }
 
 
