@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -29,8 +29,15 @@
 // Headers
 ////////////////////////////////////////////////////////////
 #include <SFML/Config.hpp>
+
+#if defined(SFML_SYSTEM_MACOS)
+#include <OpenAL/al.h>
+#include <OpenAL/alc.h>
+#else
 #include <AL/al.h>
 #include <AL/alc.h>
+#endif
+
 #include <iostream>
 #include <string>
 
@@ -61,7 +68,7 @@ namespace priv
 /// Check last OpenAL error
 ///
 ////////////////////////////////////////////////////////////
-inline void ALCheckError(const std::string File, unsigned int Line)
+inline void ALCheckError(const std::string& File, unsigned int Line)
 {
     // Get the last error
     ALenum ErrorCode = alGetError();
