@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -35,11 +35,9 @@ namespace sf
 ////////////////////////////////////////////////////////////
 /// Default constructor
 ////////////////////////////////////////////////////////////
-Sound::Sound() :
-myBuffer(NULL)
+Sound::Sound()
 {
     ALCheck(alGenSources(1, &mySource));
-
     ALCheck(alSourcei(mySource, AL_BUFFER, 0));
 }
 
@@ -48,15 +46,15 @@ myBuffer(NULL)
 /// Construct the sound from its parameters
 ////////////////////////////////////////////////////////////
 Sound::Sound(const SoundBuffer& Buffer, bool Loop, float Pitch, float Volume, const Vector3f& Position) :
-myBuffer(&Buffer)
+myBuffer(NULL)
 {
     ALCheck(alGenSources(1, &mySource));
 
-    ALCheck(alSourcei (mySource, AL_BUFFER,   Buffer.myBuffer));
-    ALCheck(alSourcei (mySource, AL_LOOPING,  Loop));
-    ALCheck(alSourcef (mySource, AL_PITCH,    Pitch));
-    ALCheck(alSourcef (mySource, AL_GAIN,     Volume * 0.01f));
-    ALCheck(alSource3f(mySource, AL_POSITION, Position.x, Position.y, Position.z));
+    SetBuffer(Buffer);
+    SetLoop(Loop);
+    SetPitch(Pitch);
+    SetVolume(Volume);
+    SetPosition(Position);
 }
 
 
@@ -65,15 +63,19 @@ myBuffer(&Buffer)
 ////////////////////////////////////////////////////////////
 Sound::Sound(const Sound& Copy) :
 AudioResource(Copy),
-myBuffer     (Copy.myBuffer)
+myBuffer(NULL)
 {
     ALCheck(alGenSources(1, &mySource));
 
-    ALCheck(alSourcei (mySource, AL_BUFFER,   myBuffer ? myBuffer->myBuffer : 0));
-    ALCheck(alSourcei (mySource, AL_LOOPING,  Copy.GetLoop()));
-    ALCheck(alSourcef (mySource, AL_PITCH,    Copy.GetPitch()));
-    ALCheck(alSourcef (mySource, AL_GAIN,     Copy.GetVolume() * 0.01f));
-    ALCheck(alSource3f(mySource, AL_POSITION, Copy.GetPosition().x, Copy.GetPosition().y, Copy.GetPosition().z));
+    if (Copy.myBuffer)
+        SetBuffer(*Copy.myBuffer);
+    SetLoop(Copy.GetLoop());
+    SetPitch(Copy.GetPitch());
+    SetVolume(Copy.GetVolume());
+    SetPosition(Copy.GetPosition());
+    SetRelativeToListener(Copy.IsRelativeToListener());
+    SetMinDistance(Copy.GetMinDistance());
+    SetAttenuation(Copy.GetAttenuation());
 }
 
 
@@ -88,6 +90,7 @@ Sound::~Sound()
         {
             Stop();
             ALCheck(alSourcei(mySource, AL_BUFFER, 0));
+            myBuffer->DetachSound(this);
         }
         ALCheck(alDeleteSources(1, &mySource));
     }
@@ -126,8 +129,17 @@ void Sound::Stop()
 ////////////////////////////////////////////////////////////
 void Sound::SetBuffer(const SoundBuffer& Buffer)
 {
+    // First detach from the previous buffer
+    if (myBuffer)
+    {
+        Stop();
+        myBuffer->DetachSound(this);
+    }
+
+    // Assign and use the new buffer
     myBuffer = &Buffer;
-    ALCheck(alSourcei(mySource, AL_BUFFER, myBuffer ? myBuffer->myBuffer : 0));
+    myBuffer->AttachSound(this);
+    ALCheck(alSourcei(mySource, AL_BUFFER, myBuffer->myBuffer));
 }
 
 
@@ -177,6 +189,17 @@ void Sound::SetPosition(const Vector3f& Position)
 }
 
 
+////////////////////////////////////////////////////////////
+/// Make the sound's position relative to the listener's
+/// position, or absolute.
+/// The default value is false (absolute)
+////////////////////////////////////////////////////////////
+void Sound::SetRelativeToListener(bool Relative)
+{
+    ALCheck(alSourcei(mySource, AL_SOURCE_RELATIVE, Relative));
+}
+
+
 ////////////////////////////////////////////////////////////
 /// Set the minimum distance - closer than this distance,
 /// the listener will hear the sound at its maximum volume.
@@ -199,6 +222,15 @@ void Sound::SetAttenuation(float Attenuation)
 }
 
 
+////////////////////////////////////////////////////////////
+/// Set the current playing position of the sound
+////////////////////////////////////////////////////////////
+void Sound::SetPlayingOffset(float TimeOffset)
+{
+    ALCheck(alSourcef(mySource, AL_SEC_OFFSET, TimeOffset));
+}
+
+
 ////////////////////////////////////////////////////////////
 /// Get the source buffer
 ////////////////////////////////////////////////////////////
@@ -256,6 +288,19 @@ Vector3f Sound::GetPosition() const
 }
 
 
+////////////////////////////////////////////////////////////
+/// Tell if the sound's position is relative to the listener's
+/// position, or if it's absolute
+////////////////////////////////////////////////////////////
+bool Sound::IsRelativeToListener() const
+{
+    ALint Relative;
+    ALCheck(alGetSourcei(mySource, AL_SOURCE_RELATIVE, &Relative));
+
+    return Relative != 0;
+}
+
+
 ////////////////////////////////////////////////////////////
 /// Get the minimum distance
 ////////////////////////////////////////////////////////////
@@ -280,6 +325,18 @@ float Sound::GetAttenuation() const
 }
 
 
+////////////////////////////////////////////////////////////
+/// Get the current playing position of the sound
+////////////////////////////////////////////////////////////
+float Sound::GetPlayingOffset() const
+{
+    ALfloat Seconds = 0.f;
+    ALCheck(alGetSourcef(mySource, AL_SEC_OFFSET, &Seconds));
+
+    return Seconds;
+}
+
+
 ////////////////////////////////////////////////////////////
 /// Get the status of the sound (stopped, paused, playing)
 ////////////////////////////////////////////////////////////
@@ -300,29 +357,48 @@ Sound::Status Sound::GetStatus() const
 }
 
 
-////////////////////////////////////////////////////////////
-/// Get the current playing position of the sound
-////////////////////////////////////////////////////////////
-float Sound::GetPlayingOffset() const
-{
-    ALfloat Seconds = 0.f;
-    ALCheck(alGetSourcef(mySource, AL_SEC_OFFSET, &Seconds));
-
-    return Seconds;
-}
-
-
 ////////////////////////////////////////////////////////////
 /// Assignment operator
 ////////////////////////////////////////////////////////////
 Sound& Sound::operator =(const Sound& Other)
 {
-    Sound Temp(Other);
+    // Here we don't use the copy-and-swap idiom, because it would mess up
+    // the list of sound instances contained in the buffers
 
-    std::swap(mySource, Temp.mySource);
-    std::swap(myBuffer, Temp.myBuffer);
+    // Detach the sound instance from the previous buffer (if any)
+    if (myBuffer)
+    {
+        Stop();
+        myBuffer->DetachSound(this);
+        myBuffer = NULL;
+    }
+
+    // Copy the sound attributes
+    if (Other.myBuffer)
+        SetBuffer(*Other.myBuffer);
+    SetLoop(Other.GetLoop());
+    SetPitch(Other.GetPitch());
+    SetVolume(Other.GetVolume());
+    SetPosition(Other.GetPosition());
+    SetRelativeToListener(Other.IsRelativeToListener());
+    SetMinDistance(Other.GetMinDistance());
+    SetAttenuation(Other.GetAttenuation());
 
     return *this;
 }
 
+
+////////////////////////////////////////////////////////////
+/// Reset the internal buffer
+////////////////////////////////////////////////////////////
+void Sound::ResetBuffer()
+{
+    // First stop the sound in case it is playing
+    Stop();
+
+    // Detach the buffer
+    ALCheck(alSourcei(mySource, AL_BUFFER, 0));
+    myBuffer = NULL;
+}
+
 } // namespace sf
