@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -27,6 +27,7 @@
 ////////////////////////////////////////////////////////////
 #include <SFML/Audio/SoundBuffer.hpp>
 #include <SFML/Audio/SoundFile.hpp>
+#include <SFML/Audio/Sound.hpp>
 #include <SFML/Audio/AudioDevice.hpp>
 #include <SFML/Audio/OpenAL.hpp>
 #include <iostream>
@@ -51,10 +52,12 @@ myDuration(0.f)
 /// Copy constructor
 ////////////////////////////////////////////////////////////
 SoundBuffer::SoundBuffer(const SoundBuffer& Copy) :
-AudioResource(Copy),
-myBuffer     (0),
-mySamples    (Copy.mySamples),
-myDuration   (Copy.myDuration)
+AudioResource        (Copy),
+Resource<SoundBuffer>(Copy),
+myBuffer             (0),
+mySamples            (Copy.mySamples),
+myDuration           (Copy.myDuration),
+mySounds             () // don't copy the attached sounds
 {
     // Create the buffer
     ALCheck(alGenBuffers(1, &myBuffer));
@@ -69,6 +72,11 @@ myDuration   (Copy.myDuration)
 ////////////////////////////////////////////////////////////
 SoundBuffer::~SoundBuffer()
 {
+    // First detach the buffer from the sounds that use it (to avoid OpenAL errors)
+    for (SoundList::const_iterator it = mySounds.begin(); it != mySounds.end(); ++it)
+        (*it)->ResetBuffer();
+
+    // Destroy the buffer
     if (myBuffer)
         ALCheck(alDeleteBuffers(1, &myBuffer));
 }
@@ -267,9 +275,10 @@ SoundBuffer& SoundBuffer::operator =(const SoundBuffer& Other)
 {
     SoundBuffer Temp(Other);
 
-    mySamples.swap(Temp.mySamples);
+    std::swap(mySamples,  Temp.mySamples);
     std::swap(myBuffer,   Temp.myBuffer);
     std::swap(myDuration, Temp.myDuration);
+    std::swap(mySounds,   Temp.mySounds); // swap sounds too, so that they are detached when Temp is destroyed
 
     return *this;
 }
@@ -304,4 +313,22 @@ bool SoundBuffer::Update(unsigned int ChannelsCount, unsigned int SampleRate)
     return true;
 }
 
+
+////////////////////////////////////////////////////////////
+/// Add a sound to the list of sounds that use this buffer
+////////////////////////////////////////////////////////////
+void SoundBuffer::AttachSound(Sound* Instance) const
+{
+    mySounds.insert(Instance);
+}
+
+
+////////////////////////////////////////////////////////////
+/// Remove a sound from the list of sounds that use this buffer
+////////////////////////////////////////////////////////////
+void SoundBuffer::DetachSound(Sound* Instance) const
+{
+    mySounds.erase(Instance);
+}
+
 } // namespace sf
