@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -33,9 +33,20 @@
 namespace sf
 {
 ////////////////////////////////////////////////////////////
-/// /see SoundBuffer::ProcessSamples
+/// /see SoundBuffer::OnStart
 ////////////////////////////////////////////////////////////
-bool SoundBufferRecorder::ProcessSamples(const Int16* Samples, std::size_t SamplesCount)
+bool SoundBufferRecorder::OnStart()
+{
+    mySamples.clear();
+
+    return true;
+}
+
+
+////////////////////////////////////////////////////////////
+/// /see SoundBuffer::OnProcessSamples
+////////////////////////////////////////////////////////////
+bool SoundBufferRecorder::OnProcessSamples(const Int16* Samples, std::size_t SamplesCount)
 {
     std::copy(Samples, Samples + SamplesCount, std::back_inserter(mySamples));
 
@@ -43,15 +54,21 @@ bool SoundBufferRecorder::ProcessSamples(const Int16* Samples, std::size_t Sampl
 }
 
 
+////////////////////////////////////////////////////////////
+/// /see SoundBuffer::OnStop
+////////////////////////////////////////////////////////////
+void SoundBufferRecorder::OnStop()
+{
+    if (!mySamples.empty())
+        myBuffer.LoadFromSamples(&mySamples[0], mySamples.size(), 1, GetSampleRate());
+}
+
+
 ////////////////////////////////////////////////////////////
 /// Get the sound buffer containing the captured audio data
 ////////////////////////////////////////////////////////////
-const SoundBuffer& SoundBufferRecorder::GetBuffer()
+const SoundBuffer& SoundBufferRecorder::GetBuffer() const
 {
-    // Fill the buffer if it's empty
-    if (myBuffer.GetSamplesCount() == 0)
-        myBuffer.LoadFromSamples(&mySamples[0], mySamples.size(), 1, GetSampleRate());
-
     return myBuffer;
 }
 
