@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -112,10 +112,7 @@ bool SoundFileOgg::OpenRead(const std::string& Filename, std::size_t& NbSamples,
 {
     // Close the file if already opened
     if (myStream)
-    {
         stb_vorbis_close(myStream);
-        myStream = NULL;
-    }
 
     // Open the vorbis stream
     myStream = stb_vorbis_open_filename(const_cast<char*>(Filename.c_str()), NULL, NULL);
@@ -142,10 +139,7 @@ bool SoundFileOgg::OpenRead(const char* Data, std::size_t SizeInBytes, std::size
 {
     // Close the file if already opened
     if (myStream)
-    {
         stb_vorbis_close(myStream);
-        myStream = NULL;
-    }
 
     // Open the vorbis stream
     unsigned char* Buffer = reinterpret_cast<unsigned char*>(const_cast<char*>(Data));
@@ -161,7 +155,7 @@ bool SoundFileOgg::OpenRead(const char* Data, std::size_t SizeInBytes, std::size
     stb_vorbis_info Infos = stb_vorbis_get_info(myStream);
     ChannelsCount = myChannelsCount = Infos.channels;
     SampleRate    = Infos.sample_rate;
-    NbSamples     = static_cast<std::size_t>(stb_vorbis_stream_length_in_samples(myStream));
+    NbSamples     = static_cast<std::size_t>(stb_vorbis_stream_length_in_samples(myStream) * ChannelsCount);
 
     return true;
 }
@@ -172,8 +166,15 @@ bool SoundFileOgg::OpenRead(const char* Data, std::size_t SizeInBytes, std::size
 ////////////////////////////////////////////////////////////
 std::size_t SoundFileOgg::Read(Int16* Data, std::size_t NbSamples)
 {
-    int Read = stb_vorbis_get_samples_short_interleaved(myStream, myChannelsCount, Data, static_cast<int>(NbSamples));
-    return static_cast<std::size_t>(Read * myChannelsCount);
+    if (myStream && Data && NbSamples)
+    {
+        int Read = stb_vorbis_get_samples_short_interleaved(myStream, myChannelsCount, Data, static_cast<int>(NbSamples));
+        return static_cast<std::size_t>(Read * myChannelsCount);
+    }
+    else
+    {
+        return 0;
+    }
 }
 
 } // namespace priv
