@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -26,7 +26,6 @@
 // Headers
 ////////////////////////////////////////////////////////////
 #include <SFML/Audio/SoundStream.hpp>
-#include <SFML/Audio/SoundBuffer.hpp>
 #include <SFML/Audio/AudioDevice.hpp>
 #include <SFML/Audio/OpenAL.hpp>
 #include <SFML/System/Sleep.hpp>
@@ -38,10 +37,12 @@ namespace sf
 /// Default constructor
 ////////////////////////////////////////////////////////////
 SoundStream::SoundStream() :
-myIsStreaming  (false),
-myChannelsCount(0),
-mySampleRate   (0),
-myFormat       (0)
+myIsStreaming     (false),
+myChannelsCount   (0),
+mySampleRate      (0),
+myFormat          (0),
+myLoop            (false),
+mySamplesProcessed(0)
 {
 
 }
@@ -101,6 +102,7 @@ void SoundStream::Play()
     if (OnStart())
     {
         // Start updating the stream in a separate thread to avoid blocking the application
+        mySamplesProcessed = 0;
         myIsStreaming = true;
         Launch();
     }
@@ -152,37 +154,47 @@ Sound::Status SoundStream::GetStatus() const
 
 
 ////////////////////////////////////////////////////////////
-/// /see sfThread::Run
+/// Get the current playing position of the stream
+///
+/// \return Current playing position, expressed in seconds
+///
+////////////////////////////////////////////////////////////
+float SoundStream::GetPlayingOffset() const
+{
+    return Sound::GetPlayingOffset() + static_cast<float>(mySamplesProcessed) / mySampleRate / myChannelsCount;
+}
+
+
+////////////////////////////////////////////////////////////
+/// Set the music loop state
+////////////////////////////////////////////////////////////
+void SoundStream::SetLoop(bool Loop)
+{
+    myLoop = Loop;
+}
+
+
+////////////////////////////////////////////////////////////
+/// Tell whether or not the music is looping
+////////////////////////////////////////////////////////////
+bool SoundStream::GetLoop() const
+{
+    return myLoop;
+}
+
+
+////////////////////////////////////////////////////////////
+/// /see Thread::Run
 ////////////////////////////////////////////////////////////
 void SoundStream::Run()
 {
-    bool RequestStop = false;
-
-    // Enqueue the buffers
+    // Create the buffers
     ALCheck(alGenBuffers(BuffersCount, myBuffers));
     for (int i = 0; i < BuffersCount; ++i)
-    {
-        // Acquire audio data
-        Chunk Data = {NULL, 0};
-        if (OnGetData(Data))
-        {
-            // Create and fill a buffer, and push it to the queue
-            if (Data.Samples && Data.NbSamples)
-            {
-                // Fill the buffer
-                ALsizei Size = static_cast<ALsizei>(Data.NbSamples) * sizeof(Int16);
-                ALCheck(alBufferData(myBuffers[i], myFormat, Data.Samples, Size, mySampleRate));
+        myEndBuffers[i] = false;
 
-                // Push it into the sound queue
-                ALCheck(alSourceQueueBuffers(Sound::mySource, 1, &myBuffers[i]));
-            }
-        }
-        else
-        {
-            RequestStop = true;
-            break;
-        }
-    }
+    // Fill the queue
+    bool RequestStop = FillQueue();
 
     // Play the sound
     Sound::Play();
@@ -192,16 +204,15 @@ void SoundStream::Run()
         // The stream has been interrupted !
         if (Sound::GetStatus() == Stopped)
         {
-            // User requested to stop : finish the streaming loop
-            if (RequestStop)
+            if (!RequestStop)
             {
-                myIsStreaming = false;
-                break;
+                // Just continue
+                Sound::Play();
             }
             else
             {
-                // Streaming is not completed : restart the sound
-                Sound::Play();
+                // End streaming
+                myIsStreaming = false;
             }
         }
 
@@ -209,59 +220,141 @@ void SoundStream::Run()
         ALint NbProcessed;
         ALCheck(alGetSourcei(Sound::mySource, AL_BUFFERS_PROCESSED, &NbProcessed));
 
-        while (NbProcessed-- && !RequestStop)
+        while (NbProcessed--)
         {
             // Pop the first unused buffer from the queue
             ALuint Buffer;
             ALCheck(alSourceUnqueueBuffers(Sound::mySource, 1, &Buffer));
 
-            // Acquire new data
-            Chunk Data = {NULL, 0};
-            if (!OnGetData(Data))
-                RequestStop = true;
+            // Find its number
+            unsigned int BufferNum = 0;
+            for (int i = 0; i < BuffersCount; ++i)
+                if (myBuffers[i] == Buffer)
+                {
+                    BufferNum = i;
+                    break;
+                }
 
-            // Fill the buffer with the new data, and push it back to the queue
-            if (Data.Samples && Data.NbSamples)
+            // Retrieve its size and add it to the samples count
+            if (myEndBuffers[BufferNum])
             {
-                // Fill the buffer
-                ALsizei Size = static_cast<ALsizei>(Data.NbSamples) * sizeof(Int16);
-                ALCheck(alBufferData(Buffer, myFormat, Data.Samples, Size, mySampleRate));
+                // This was the last buffer: reset the sample count
+                mySamplesProcessed = 0;
+                myEndBuffers[BufferNum] = false;
+            }
+            else
+            {
+                ALint Size;
+                ALCheck(alGetBufferi(Buffer, AL_SIZE, &Size));
+                mySamplesProcessed += Size / sizeof(Int16);
+            }
 
-                // Push it into the sound queue
-                ALCheck(alSourceQueueBuffers(Sound::mySource, 1, &Buffer));
+            // Fill it and push it back into the playing queue
+            if (!RequestStop)
+            {
+                if (FillAndPushBuffer(BufferNum))
+                    RequestStop = true;
             }
         }
 
-        // Leave some time for the other threads
-        if (!RequestStop)
+        // Leave some time for the other threads if the stream is still playing
+        if (Sound::GetStatus() != Stopped)
             Sleep(0.1f);
     }
 
-    // Streaming finished : clean up everything
-    CleanUp();
-}
-
-
-////////////////////////////////////////////////////////////
-/// Clean up stream internal resources
-////////////////////////////////////////////////////////////
-void SoundStream::CleanUp()
-{
     // Stop the playback
     Sound::Stop();
 
     // Unqueue any buffer left in the queue
-    ALint  NbQueued;
-    ALuint Buffer;
-    ALCheck(alGetSourcei(Sound::mySource, AL_BUFFERS_QUEUED, &NbQueued));
-    for (ALint i = 0; i < NbQueued; ++i)
-        ALCheck(alSourceUnqueueBuffers(Sound::mySource, 1, &Buffer));
+    ClearQueue();
 
     // Delete the buffers
+    ALCheck(alSourcei(Sound::mySource, AL_BUFFER, 0));
     ALCheck(alDeleteBuffers(BuffersCount, myBuffers));
 }
 
 
+////////////////////////////////////////////////////////////
+/// Fill a new buffer with audio data, and push it to the
+/// playing queue
+////////////////////////////////////////////////////////////
+bool SoundStream::FillAndPushBuffer(unsigned int BufferNum)
+{
+    bool RequestStop = false;
+
+    // Acquire audio data
+    Chunk Data = {NULL, 0};
+    if (!OnGetData(Data))
+    {
+        // Mark the buffer as the last one (so that we know when to reset the playing position)
+        myEndBuffers[BufferNum] = true;
+
+        // Check if the stream must loop or stop
+        if (myLoop && OnStart())
+        {
+            // If we succeeded to restart and we previously had no data, try to fill the buffer once again
+            if (!Data.Samples || (Data.NbSamples == 0))
+            {
+                return FillAndPushBuffer(BufferNum);
+            }
+        }
+        else
+        {
+            // Not looping or restart failed: request stop
+            RequestStop = true;
+        }
+    }
+
+    // Fill the buffer if some data was returned
+    if (Data.Samples && Data.NbSamples)
+    {
+        unsigned int Buffer = myBuffers[BufferNum];
+
+        // Fill the buffer
+        ALsizei Size = static_cast<ALsizei>(Data.NbSamples) * sizeof(Int16);
+        ALCheck(alBufferData(Buffer, myFormat, Data.Samples, Size, mySampleRate));
+
+        // Push it into the sound queue
+        ALCheck(alSourceQueueBuffers(Sound::mySource, 1, &Buffer));
+    }
+
+    return RequestStop;
+}
+
+
+////////////////////////////////////////////////////////////
+/// Fill the buffers queue with all available buffers
+////////////////////////////////////////////////////////////
+bool SoundStream::FillQueue()
+{
+    // Fill and enqueue all the available buffers
+    bool RequestStop = false;
+    for (int i = 0; (i < BuffersCount) && !RequestStop; ++i)
+    {
+        if (FillAndPushBuffer(i))
+            RequestStop = true;
+    }
+
+    return RequestStop;
+}
+
+
+////////////////////////////////////////////////////////////
+/// Clear the queue of any remaining buffers
+////////////////////////////////////////////////////////////
+void SoundStream::ClearQueue()
+{
+    // Get the number of buffers still in the queue
+    ALint NbQueued;
+    ALCheck(alGetSourcei(Sound::mySource, AL_BUFFERS_QUEUED, &NbQueued));
+
+    // Unqueue them all
+    ALuint Buffer;
+    for (ALint i = 0; i < NbQueued; ++i)
+        ALCheck(alSourceUnqueueBuffers(Sound::mySource, 1, &Buffer));
+}
+
+
 ////////////////////////////////////////////////////////////
 /// Called when the sound restarts
 ////////////////////////////////////////////////////////////
