@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -26,7 +26,7 @@
 // Headers
 ////////////////////////////////////////////////////////////
 #include <SFML/Graphics/Drawable.hpp>
-#include <SFML/Graphics/Renderer.hpp>
+#include <SFML/Graphics/GraphicsContext.hpp>
 #include <SFML/Window/Window.hpp>
 #include <math.h>
 
@@ -37,16 +37,16 @@ namespace sf
 /// Default constructor
 ////////////////////////////////////////////////////////////
 Drawable::Drawable(const Vector2f& Position, const Vector2f& Scale, float Rotation, const Color& Col) :
-myPosition  (Position),
-myScale     (Scale),
-myCenter    (0, 0),
-myRotation  (Rotation),
-myColor     (Col),
-myBlendMode (Blend::Alpha),
-myNeedUpdate(true)
+myPosition     (Position),
+myScale        (Scale),
+myCenter       (0, 0),
+myRotation     (Rotation),
+myColor        (Col),
+myBlendMode    (Blend::Alpha),
+myNeedUpdate   (true),
+myInvNeedUpdate(true)
 {
-    // Make sure the renderer exists
-    GetRenderer();
+
 }
 
 
@@ -84,8 +84,9 @@ void Drawable::SetPosition(const Vector2f& Position)
 ////////////////////////////////////////////////////////////
 void Drawable::SetX(float X)
 {
-    myPosition.x = X;
-    myNeedUpdate = true;
+    myPosition.x    = X;
+    myNeedUpdate    = true;
+    myInvNeedUpdate = true;
 }
 
 
@@ -94,8 +95,9 @@ void Drawable::SetX(float X)
 ////////////////////////////////////////////////////////////
 void Drawable::SetY(float Y)
 {
-    myPosition.y = Y;
-    myNeedUpdate = true;
+    myPosition.y    = Y;
+    myNeedUpdate    = true;
+    myInvNeedUpdate = true;
 }
 
 
@@ -126,8 +128,9 @@ void Drawable::SetScaleX(float FactorX)
 {
     if (FactorX > 0)
     {
-        myScale.x    = FactorX;
-        myNeedUpdate = true;
+        myScale.x       = FactorX;
+        myNeedUpdate    = true;
+        myInvNeedUpdate = true;
     }
 }
 
@@ -139,8 +142,9 @@ void Drawable::SetScaleY(float FactorY)
 {
     if (FactorY > 0)
     {
-        myScale.y    = FactorY;
-        myNeedUpdate = true;
+        myScale.y       = FactorY;
+        myNeedUpdate    = true;
+        myInvNeedUpdate = true;
     }
 }
 
@@ -152,9 +156,10 @@ void Drawable::SetScaleY(float FactorY)
 ////////////////////////////////////////////////////////////
 void Drawable::SetCenter(float CenterX, float CenterY)
 {
-    myCenter.x   = CenterX;
-    myCenter.y   = CenterY;
-    myNeedUpdate = true;
+    myCenter.x      = CenterX;
+    myCenter.y      = CenterY;
+    myNeedUpdate    = true;
+    myInvNeedUpdate = true;
 }
 
 
@@ -177,7 +182,8 @@ void Drawable::SetRotation(float Rotation)
     myRotation = static_cast<float>(fmod(Rotation, 360));
     if (myRotation < 0)
         myRotation += 360.f;
-    myNeedUpdate = true;
+    myNeedUpdate    = true;
+    myInvNeedUpdate = true;
 }
 
 
@@ -303,6 +309,25 @@ void Drawable::Rotate(float Angle)
 }
 
 
+////////////////////////////////////////////////////////////
+/// Transform a point from global coordinates into local coordinates
+/// (ie it applies the inverse of object's center, translation, rotation and scale to the point)
+////////////////////////////////////////////////////////////
+sf::Vector2f Drawable::TransformToLocal(const sf::Vector2f& Point) const
+{
+    return GetInverseMatrix().Transform(Point);
+}
+
+////////////////////////////////////////////////////////////
+/// Transform a point from local coordinates into global coordinates
+/// (ie it applies the object's center, translation, rotation and scale to the point)
+////////////////////////////////////////////////////////////
+sf::Vector2f Drawable::TransformToGlobal(const sf::Vector2f& Point) const
+{
+    return GetMatrix().Transform(Point);
+}
+
+
 ////////////////////////////////////////////////////////////
 /// Get the transform matrix of the drawable
 ////////////////////////////////////////////////////////////
@@ -320,21 +345,30 @@ const Matrix3& Drawable::GetMatrix() const
 
 
 ////////////////////////////////////////////////////////////
-/// Draw the object into the specified window
+/// Get the inverse transform matrix of the drawable
 ////////////////////////////////////////////////////////////
-void Drawable::Draw(const RenderWindow& Window) const
+const Matrix3& Drawable::GetInverseMatrix() const
 {
-    // Recompute the transform matrix if needed
-    if (myNeedUpdate)
+    // First recompute it if needed
+    if (myInvNeedUpdate)
     {
-        myMatrix.SetFromTransformations(myCenter, myPosition, myRotation, myScale);
-        myNeedUpdate = false;
+        myInvMatrix     = GetMatrix().GetInverse();
+        myInvNeedUpdate = false;
     }
 
+    return myInvMatrix;
+}
+
+
+////////////////////////////////////////////////////////////
+/// Draw the object into the specified window
+////////////////////////////////////////////////////////////
+void Drawable::Draw(RenderTarget& Target) const
+{
     // Save the current modelview matrix and set the new one
     GLCheck(glMatrixMode(GL_MODELVIEW));
     GLCheck(glPushMatrix());
-    GLCheck(glMultMatrixf(myMatrix.Get4x4Elements()));
+    GLCheck(glMultMatrixf(GetMatrix().Get4x4Elements()));
 
     // Setup alpha-blending
     if (myBlendMode == Blend::None)
@@ -348,7 +382,7 @@ void Drawable::Draw(const RenderWindow& Window) const
         switch (myBlendMode)
         {
             case Blend::Alpha :    GLCheck(glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)); break;
-            case Blend::Add :      GLCheck(glBlendFunc(GL_ONE,       GL_ONE));                 break;
+            case Blend::Add :      GLCheck(glBlendFunc(GL_SRC_ALPHA, GL_ONE));                 break;
             case Blend::Multiply : GLCheck(glBlendFunc(GL_DST_COLOR, GL_ZERO));                break;
             default :                                                                          break;
         }
@@ -358,7 +392,7 @@ void Drawable::Draw(const RenderWindow& Window) const
     GLCheck(glColor4f(myColor.r / 255.f, myColor.g / 255.f, myColor.b / 255.f, myColor.a / 255.f));
 
     // Let the derived class render the object geometry
-    Render(Window);
+    Render(Target);
 
     // Restore the previous modelview matrix
     GLCheck(glMatrixMode(GL_MODELVIEW));
