@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -29,42 +29,32 @@
 #include <SFML/Graphics/Color.hpp>
 #include <SFML/Graphics/Font.hpp>
 #include <SFML/Graphics/Image.hpp>
-#include <SFML/Graphics/Renderer.hpp>
+#include <SFML/Graphics/GraphicsContext.hpp>
+#include FT_GLYPH_H
 #include <iostream>
+#include <map>
 #include <vector>
 #include <math.h>
 
 
+namespace
+{
+    ////////////////////////////////////////////////////////////
+    // Functor to sort glyphs by size
+    ////////////////////////////////////////////////////////////
+    struct SizeCompare
+    {
+        bool operator ()(FT_BitmapGlyph Glyph1, FT_BitmapGlyph Glyph2) const
+        {
+            return Glyph2->bitmap.rows < Glyph1->bitmap.rows;
+        }
+    };
+}
+
 namespace sf
 {
 namespace priv
 {
-////////////////////////////////////////////////////////////
-// Static member data
-////////////////////////////////////////////////////////////
-wchar_t FontLoader::ourDefaultCharset[] =
-{
-     31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
-     44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,
-     57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,
-     70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,
-     83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,
-     96,  97,  98,  99, 100, 101, 102, 103, 104, 105, 106, 107, 108,
-    109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121,
-    122, 123, 124, 125, 126, 127, 128, 129, 130, 131, 132, 133, 134,
-    135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145, 146, 147,
-    148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159, 160,
-    161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173,
-    174, 175, 176, 177, 178, 179, 180, 181, 182, 183, 184, 185, 186,
-    187, 188, 189, 190, 191, 192, 193, 194, 195, 196, 197, 198, 199,
-    200, 201, 202, 203, 204, 205, 206, 207, 208, 209, 210, 211, 212,
-    213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223, 224, 225,
-    226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238,
-    239, 240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251,
-    252, 253, 254, 255, 0
-};
-
-
 ////////////////////////////////////////////////////////////
 /// Get the unique instance of the class
 ////////////////////////////////////////////////////////////
@@ -105,7 +95,7 @@ FontLoader::~FontLoader()
 ////////////////////////////////////////////////////////////
 /// Load a font from a file
 ////////////////////////////////////////////////////////////
-bool FontLoader::LoadFontFromFile(const std::string& Filename, unsigned int CharSize, std::wstring Charset, Font& LoadedFont)
+bool FontLoader::LoadFontFromFile(const std::string& Filename, unsigned int CharSize, const Unicode::UTF32String& Charset, Font& LoadedFont)
 {
     // Check if Freetype is correctly initialized
     if (!myLibrary)
@@ -138,7 +128,7 @@ bool FontLoader::LoadFontFromFile(const std::string& Filename, unsigned int Char
 ////////////////////////////////////////////////////////////
 /// Load the font from a file in memory
 ////////////////////////////////////////////////////////////
-bool FontLoader::LoadFontFromMemory(const char* Data, std::size_t SizeInBytes, unsigned int CharSize, std::wstring Charset, Font& LoadedFont)
+bool FontLoader::LoadFontFromMemory(const char* Data, std::size_t SizeInBytes, unsigned int CharSize, const Unicode::UTF32String& Charset, Font& LoadedFont)
 {
     // Check if Freetype is correctly initialized
     if (!myLibrary)
@@ -171,17 +161,10 @@ bool FontLoader::LoadFontFromMemory(const char* Data, std::size_t SizeInBytes, u
 ////////////////////////////////////////////////////////////
 /// Create a bitmap font from a font face and a characters set
 ////////////////////////////////////////////////////////////
-FT_Error FontLoader::CreateBitmapFont(FT_Face FontFace, unsigned int CharSize, std::wstring Charset, Font& LoadedFont)
+FT_Error FontLoader::CreateBitmapFont(FT_Face FontFace, unsigned int CharSize, const Unicode::UTF32String& Charset, Font& LoadedFont)
 {
-    // If no characters set has been provided, we take the default one (31 - 255 ASCII codes)
-    if (Charset.empty())
-        Charset = ourDefaultCharset;
-
-    // Always add these special characters
-    Charset += L" \n\v\t";
-
-    // Clear the font's previous character map
-    LoadedFont.myCharacters.clear();
+    // Make sure we have a valid context
+    priv::GraphicsContext Ctx;
 
     // Let's find how many characters to put in each row to make them fit into a squared texture
     GLint MaxSize;
@@ -196,11 +179,11 @@ FT_Error FontLoader::CreateBitmapFont(FT_Face FontFace, unsigned int CharSize, s
     unsigned int Left      = 0;
     unsigned int Top       = 0;
     unsigned int TexWidth  = Image::GetValidTextureSize(CharSize * NbChars);
-    unsigned int TexHeight = Image::GetValidTextureSize(CharSize * NbChars);
+    unsigned int TexHeight = CharSize * NbChars;
     std::vector<unsigned int> Tops(TexWidth, 0);
 
-    // Create the bitmap font (all black for now -- we're going to fill it up)
-    LoadedFont.myTexture.Create(TexWidth, TexHeight, Color(0, 0, 0, 0));
+    // Create a pixel buffer for rendering every glyph
+    std::vector<Uint8> GlyphsBuffer(TexWidth * TexHeight * 4);
 
     // Setup the font size
     FT_Error Error = FT_Set_Pixel_Sizes(FontFace, CharSize, CharSize);
@@ -212,16 +195,13 @@ FT_Error FontLoader::CreateBitmapFont(FT_Face FontFace, unsigned int CharSize, s
     if (Error)
         return Error;
 
-    // Render the characters set to a bitmap
-    std::map<wchar_t, IntRect> Coords;
+    // Render all glyphs and sort them by size to optimize texture space
+    typedef std::multimap<FT_BitmapGlyph, Uint32, SizeCompare> GlyphTable;
+    GlyphTable Glyphs;
     for (std::size_t i = 0; i < Charset.length(); ++i)
     {
-        // Get the current character to generate
-        wchar_t c = Charset[i];
-        Font::Character& CurChar = LoadedFont.myCharacters[c];
-
         // Load the glyph corresponding to the current character
-        Error = FT_Load_Char(FontFace, c, FT_LOAD_DEFAULT);
+        Error = FT_Load_Char(FontFace, Charset[i], FT_LOAD_TARGET_NORMAL);
         if (Error)
             return Error;
 
@@ -230,13 +210,22 @@ FT_Error FontLoader::CreateBitmapFont(FT_Face FontFace, unsigned int CharSize, s
         Error = FT_Get_Glyph(FontFace->glyph, &Glyph);
         if (Error)
             return Error;
-        FT_Glyph_To_Bitmap(&Glyph, ft_render_mode_normal, 0, 1);
+        FT_Glyph_To_Bitmap(&Glyph, FT_RENDER_MODE_NORMAL, 0, 1);
         FT_BitmapGlyph BitmapGlyph = (FT_BitmapGlyph)Glyph;
-        FT_Bitmap& Bitmap = BitmapGlyph->bitmap;
 
-        // Should we handle other pixel modes ?
-        if (Bitmap.pixel_mode != FT_PIXEL_MODE_GRAY)
-            return FT_Err_Cannot_Render_Glyph;
+        // Add it to the sorted table of glyphs
+        Glyphs.insert(std::make_pair(BitmapGlyph, Charset[i]));
+    }
+
+    // Copy the rendered glyphs into the texture
+    unsigned int MaxHeight = 0;
+    std::map<Uint32, IntRect> Coords;
+    for (GlyphTable::const_iterator i = Glyphs.begin(); i != Glyphs.end(); ++i)
+    {
+        // Get the bitmap of the current glyph
+        Glyph&         CurGlyph    = LoadedFont.myGlyphs[i->second];
+        FT_BitmapGlyph BitmapGlyph = i->first;
+        FT_Bitmap&     Bitmap      = BitmapGlyph->bitmap;
 
         // Make sure we don't go over the texture width
         if (Left + Bitmap.width + 1 >= TexWidth)
@@ -252,18 +241,18 @@ FT_Error FontLoader::CreateBitmapFont(FT_Face FontFace, unsigned int CharSize, s
         if (Top + Bitmap.rows + 1 >= TexHeight)
         {
             TexHeight *= 2;
-            LoadedFont.myTexture.Resize(TexWidth, TexHeight, Color(0, 0, 0, 0));
+            GlyphsBuffer.resize(TexWidth * TexHeight * 4);
         }
 
         // Store the character's position and size
-        CurChar.Rect.Left   = BitmapGlyph->left;
-        CurChar.Rect.Top    = -BitmapGlyph->top;
-        CurChar.Rect.Right  = CurChar.Rect.Left + Bitmap.width;
-        CurChar.Rect.Bottom = Bitmap.rows - BitmapGlyph->top;
-        CurChar.Advance     = FontFace->glyph->advance.x / 64;
+        CurGlyph.Rectangle.Left   = BitmapGlyph->left;
+        CurGlyph.Rectangle.Top    = -BitmapGlyph->top;
+        CurGlyph.Rectangle.Right  = CurGlyph.Rectangle.Left + Bitmap.width;
+        CurGlyph.Rectangle.Bottom = Bitmap.rows - BitmapGlyph->top;
+        CurGlyph.Advance          = BitmapGlyph->root.advance.x >> 16;
 
         // Texture size may change, so let the texture coordinates be calculated later
-        Coords[c] = IntRect(Left + 1, Top + 1, Left + Bitmap.width + 1, Top + Bitmap.rows + 1);
+        Coords[i->second] = IntRect(Left + 1, Top + 1, Left + Bitmap.width + 1, Top + Bitmap.rows + 1);
 
         // Draw the glyph into our bitmap font
         const Uint8* Pixels = Bitmap.buffer;
@@ -271,7 +260,11 @@ FT_Error FontLoader::CreateBitmapFont(FT_Face FontFace, unsigned int CharSize, s
         {
             for (int x = 0; x < Bitmap.width; ++x)
             {
-                LoadedFont.myTexture.SetPixel(x + Left + 1, y + Top + 1, Color(255, 255, 255, Pixels[x]));
+                std::size_t Index = x + Left + 1 + (y + Top + 1) * TexWidth;
+                GlyphsBuffer[Index * 4 + 0] = 255;
+                GlyphsBuffer[Index * 4 + 1] = 255;
+                GlyphsBuffer[Index * 4 + 2] = 255;
+                GlyphsBuffer[Index * 4 + 3] = Pixels[x];
             }
             Pixels += Bitmap.pitch;
         }
@@ -280,16 +273,23 @@ FT_Error FontLoader::CreateBitmapFont(FT_Face FontFace, unsigned int CharSize, s
         for (int x = 0; x < Bitmap.width + 1; ++x)
             Tops[Left + x] = Top + Bitmap.rows;
         Left += Bitmap.width + 1;
+        if (Top + Bitmap.rows > MaxHeight)
+            MaxHeight = Top + Bitmap.rows;
 
         // Delete the glyph
-        FT_Done_Glyph(Glyph);
+        FT_Done_Glyph((FT_Glyph)BitmapGlyph);
     }
 
-    // Now that the texture has its final size, we can precompute texture coordinates
+    // Create the font's texture
+    TexHeight = MaxHeight + 1;
+    GlyphsBuffer.resize(TexWidth * TexHeight * 4);
+    LoadedFont.myTexture.LoadFromPixels(TexWidth, TexHeight, &GlyphsBuffer[0]);
+
+    // Now that the texture is created, we can precompute texture coordinates
     for (std::size_t i = 0; i < Charset.size(); ++i)
     {
-        wchar_t c = Charset[i];
-        LoadedFont.myCharacters[c].Coord = LoadedFont.myTexture.GetTexCoords(Coords[c], false);
+        Uint32 CurChar = Charset[i];
+        LoadedFont.myGlyphs[CurChar].TexCoords = LoadedFont.myTexture.GetTexCoords(Coords[CurChar]);
     }
 
     // Update the character size (it may have been changed by the function)
