@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -27,10 +27,12 @@
 ////////////////////////////////////////////////////////////
 #include <SFML/Graphics/Image.hpp>
 #include <SFML/Graphics/ImageLoader.hpp>
-#include <SFML/Graphics/Renderer.hpp>
+#include <SFML/Graphics/RenderWindow.hpp>
+#include <SFML/Graphics/GraphicsContext.hpp>
 #include <algorithm>
 #include <iostream>
 #include <vector>
+#include <string.h>
 
 
 namespace sf
@@ -39,13 +41,14 @@ namespace sf
 /// Default constructor
 ////////////////////////////////////////////////////////////
 Image::Image() :
-myWidth        (0),
-myHeight       (0),
-myTextureWidth (0),
-myTextureHeight(0),
-myTexture      (0),
-myIsSmooth     (true),
-myUpdated      (false)
+myWidth            (0),
+myHeight           (0),
+myTextureWidth     (0),
+myTextureHeight    (0),
+myTexture          (0),
+myIsSmooth         (true),
+myNeedTextureUpdate(false),
+myNeedArrayUpdate  (false)
 {
 
 }
@@ -55,15 +58,16 @@ myUpdated      (false)
 /// Copy constructor
 ////////////////////////////////////////////////////////////
 Image::Image(const Image& Copy) :
-VideoResource  (Copy),
-myWidth        (Copy.myWidth),
-myHeight       (Copy.myHeight),
-myTextureWidth (Copy.myTextureWidth),
-myTextureHeight(Copy.myTextureHeight),
-myPixels       (Copy.myPixels),
-myTexture      (0),
-myIsSmooth     (Copy.myIsSmooth),
-myUpdated      (false)
+Resource<Image>    (Copy),
+myWidth            (Copy.myWidth),
+myHeight           (Copy.myHeight),
+myTextureWidth     (Copy.myTextureWidth),
+myTextureHeight    (Copy.myTextureHeight),
+myTexture          (0),
+myIsSmooth         (Copy.myIsSmooth),
+myPixels           (Copy.myPixels),
+myNeedTextureUpdate(false),
+myNeedArrayUpdate  (false)
 {
     CreateTexture();
 }
@@ -73,13 +77,14 @@ myUpdated      (false)
 /// Construct an empty image
 ////////////////////////////////////////////////////////////
 Image::Image(unsigned int Width, unsigned int Height, const Color& Col) :
-myWidth        (0),
-myHeight       (0),
-myTextureWidth (0),
-myTextureHeight(0),
-myTexture      (0),
-myIsSmooth     (true),
-myUpdated      (false)
+myWidth            (0),
+myHeight           (0),
+myTextureWidth     (0),
+myTextureHeight    (0),
+myTexture          (0),
+myIsSmooth         (true),
+myNeedTextureUpdate(false),
+myNeedArrayUpdate  (false)
 {
     Create(Width, Height, Col);
 }
@@ -89,13 +94,14 @@ myUpdated      (false)
 /// Construct the image from pixels in memory
 ////////////////////////////////////////////////////////////
 Image::Image(unsigned int Width, unsigned int Height, const Uint8* Data) :
-myWidth        (0),
-myHeight       (0),
-myTextureWidth (0),
-myTextureHeight(0),
-myTexture      (0),
-myIsSmooth     (true),
-myUpdated      (false)
+myWidth            (0),
+myHeight           (0),
+myTextureWidth     (0),
+myTextureHeight    (0),
+myTexture          (0),
+myIsSmooth         (true),
+myNeedTextureUpdate(false),
+myNeedArrayUpdate  (false)
 {
     LoadFromPixels(Width, Height, Data);
 }
@@ -106,8 +112,8 @@ myUpdated      (false)
 ////////////////////////////////////////////////////////////
 Image::~Image()
 {
-    // Destroy video resources
-    DestroyVideoResources();
+    // Destroy the OpenGL texture
+    DestroyTexture();
 }
 
 
@@ -202,6 +208,9 @@ bool Image::LoadFromPixels(unsigned int Width, unsigned int Height, const Uint8*
 ////////////////////////////////////////////////////////////
 bool Image::SaveToFile(const std::string& Filename) const
 {
+    // Check if the array of pixels needs to be updated
+    EnsureArrayUpdate();
+
     // Let the image loader save our pixel array into the image
     return priv::ImageLoader::GetInstance().SaveImageToFile(Filename, myPixels, myWidth, myHeight);
 }
@@ -239,6 +248,9 @@ bool Image::Create(unsigned int Width, unsigned int Height, Color Col)
 ////////////////////////////////////////////////////////////
 void Image::CreateMaskFromColor(Color ColorKey, Uint8 Alpha)
 {
+    // Check if the array of pixels needs to be updated
+    EnsureArrayUpdate();
+
     // Calculate the new color (old color with no alpha)
     Color NewColor(ColorKey.r, ColorKey.g, ColorKey.b, Alpha);
 
@@ -246,44 +258,144 @@ void Image::CreateMaskFromColor(Color ColorKey, Uint8 Alpha)
     std::replace(myPixels.begin(), myPixels.end(), ColorKey, NewColor);
 
     // The texture will need to be updated
-    myUpdated = false;
+    myNeedTextureUpdate = true;
 }
 
 
 ////////////////////////////////////////////////////////////
-/// Resize the image - warning : this function does not scale the image,
-/// it just adjusts its size (add padding or remove pixels)
+/// Copy pixels from another image onto this one.
+/// This function does a slow pixel copy and should only
+/// be used at initialization time
 ////////////////////////////////////////////////////////////
-bool Image::Resize(unsigned int Width, unsigned int Height, Color Col)
+void Image::Copy(const Image& Source, unsigned int DestX, unsigned int DestY, const IntRect& SourceRect, bool ApplyAlpha)
 {
-    // Check size
-    if ((Width == 0) || (Height == 0))
+    // Make sure both images are valid
+    if ((Source.myWidth == 0) || (Source.myHeight == 0) || (myWidth == 0) || (myHeight == 0))
+        return;
+
+    // Make sure both images have up-to-date arrays
+    EnsureArrayUpdate();
+    Source.EnsureArrayUpdate();
+
+    // Adjust the source rectangle
+    IntRect SrcRect = SourceRect;
+    if (SrcRect.GetWidth() == 0 || (SrcRect.GetHeight() == 0))
     {
-        std::cerr << "Invalid new size for image (width = " << Width << ", height = " << Height << ")" << std::endl;
-        return false;
+        SrcRect.Left   = 0;
+        SrcRect.Top    = 0;
+        SrcRect.Right  = Source.myWidth;
+        SrcRect.Bottom = Source.myHeight;
+    }
+    else
+    {
+        if (SrcRect.Left   < 0) SrcRect.Left = 0;
+        if (SrcRect.Top    < 0) SrcRect.Top  = 0;
+        if (SrcRect.Right  > static_cast<int>(Source.myWidth))  SrcRect.Right  = Source.myWidth;
+        if (SrcRect.Bottom > static_cast<int>(Source.myHeight)) SrcRect.Bottom = Source.myHeight;
     }
 
-    // Create a new pixel array with the desired size
-    std::vector<Color> Pixels(Width * Height, Col);
+    // Then find the valid bounds of the destination rectangle
+    int Width  = SrcRect.GetWidth();
+    int Height = SrcRect.GetHeight();
+    if (DestX + Width  > myWidth)  Width  = myWidth  - DestX;
+    if (DestY + Height > myHeight) Height = myHeight - DestY;
 
-    // Copy the old pixel buffer into the new one
-    for (unsigned int i = 0; i < std::min(Width, myWidth); ++i)
-        for (unsigned int j = 0; j < std::min(Height, myHeight); ++j)
-            Pixels[i + j * Width] = myPixels[i + j * myWidth];
-    Pixels.swap(myPixels);
+    // Make sure the destination area is valid
+    if ((Width <= 0) || (Height <= 0))
+        return;
 
-    // Store the new texture dimensions
-    myWidth  = Width;
-    myHeight = Height;
+    // Precompute as much as possible
+    int          Pitch     = Width * 4;
+    int          Rows      = Height;
+    int          SrcStride = Source.myWidth * 4;
+    int          DstStride = myWidth * 4;
+    const Uint8* SrcPixels = Source.GetPixelsPtr() + (SrcRect.Left + SrcRect.Top * Source.myWidth) * 4;
+    Uint8*       DstPixels = reinterpret_cast<Uint8*>(&myPixels[0]) + (DestX + DestY * myWidth) * 4;
 
-    // We can create the texture
-    if (CreateTexture())
+    // Copy the pixels
+    if (ApplyAlpha)
     {
+        // Interpolation using alpha values, pixel by pixel (slower)
+        for (int i = 0; i < Rows; ++i)
+        {
+            for (int j = 0; j < Width; ++j)
+            {
+                // Get a direct pointer to the components of the current pixel
+                const Uint8* Src   = SrcPixels + j * 4;
+                Uint8*       Dst   = DstPixels + j * 4;
+
+                // Interpolate RGB components using the alpha value of the source pixel
+                Uint8 Alpha = Src[3];
+                Dst[0] = (Src[0] * Alpha + Dst[0] * (255 - Alpha)) / 255;
+                Dst[1] = (Src[1] * Alpha + Dst[1] * (255 - Alpha)) / 255;
+                Dst[2] = (Src[2] * Alpha + Dst[2] * (255 - Alpha)) / 255;
+            }
+
+            SrcPixels += SrcStride;
+            DstPixels += DstStride;
+        }
+    }
+    else
+    {
+        // Optimized copy ignoring alpha values, row by row (faster)
+        for (int i = 0; i < Rows; ++i)
+        {
+            memcpy(DstPixels, SrcPixels, Pitch);
+            SrcPixels += SrcStride;
+            DstPixels += DstStride;
+        }
+    }
+
+    // The texture will need an update
+    myNeedTextureUpdate = true;
+}
+
+
+////////////////////////////////////////////////////////////
+/// Create the image from the current contents of the
+/// given window
+////////////////////////////////////////////////////////////
+bool Image::CopyScreen(RenderWindow& Window, const IntRect& SourceRect)
+{
+    // Adjust the source rectangle
+    IntRect SrcRect = SourceRect;
+    if (SrcRect.GetWidth() == 0 || (SrcRect.GetHeight() == 0))
+    {
+        SrcRect.Left   = 0;
+        SrcRect.Top    = 0;
+        SrcRect.Right  = Window.GetWidth();
+        SrcRect.Bottom = Window.GetHeight();
+    }
+    else
+    {
+        if (SrcRect.Left   < 0) SrcRect.Left = 0;
+        if (SrcRect.Top    < 0) SrcRect.Top  = 0;
+        if (SrcRect.Right  > static_cast<int>(Window.GetWidth()))  SrcRect.Right  = Window.GetWidth();
+        if (SrcRect.Bottom > static_cast<int>(Window.GetHeight())) SrcRect.Bottom = Window.GetHeight();
+    }
+
+    // Store the texture dimensions
+    myWidth  = SrcRect.GetWidth();
+    myHeight = SrcRect.GetHeight();
+
+    // We can then create the texture
+    if (Window.SetActive() && CreateTexture())
+    {
+        GLint PreviousTexture;
+        GLCheck(glGetIntegerv(GL_TEXTURE_BINDING_2D, &PreviousTexture));
+
+        GLCheck(glBindTexture(GL_TEXTURE_2D, myTexture));
+        GLCheck(glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, SrcRect.Left, SrcRect.Top, myWidth, myHeight));
+
+        GLCheck(glBindTexture(GL_TEXTURE_2D, PreviousTexture));
+
+        myNeedTextureUpdate = false;
+        myNeedArrayUpdate = true;
+
         return true;
     }
     else
     {
-        // Oops... something failed
         Reset();
         return false;
     }
@@ -292,10 +404,12 @@ bool Image::Resize(unsigned int Width, unsigned int Height, Color Col)
 
 ////////////////////////////////////////////////////////////
 /// Change the color of a pixel
-/// Don't forget to call Update when you end modifying pixels
 ////////////////////////////////////////////////////////////
 void Image::SetPixel(unsigned int X, unsigned int Y, const Color& Col)
 {
+    // First check if the array of pixels needs to be updated
+    EnsureArrayUpdate();
+
     // Check if pixel is whithin the image bounds
     if ((X >= myWidth) || (Y >= myHeight))
     {
@@ -307,7 +421,7 @@ void Image::SetPixel(unsigned int X, unsigned int Y, const Color& Col)
     myPixels[X + Y * myWidth] = Col;
 
     // The texture will need to be updated
-    myUpdated = false;
+    myNeedTextureUpdate = true;
 }
 
 
@@ -316,6 +430,9 @@ void Image::SetPixel(unsigned int X, unsigned int Y, const Color& Col)
 ////////////////////////////////////////////////////////////
 const Color& Image::GetPixel(unsigned int X, unsigned int Y) const
 {
+    // First check if the array of pixels needs to be updated
+    EnsureArrayUpdate();
+
     // Check if pixel is whithin the image bounds
     if ((X >= myWidth) || (Y >= myHeight))
     {
@@ -335,6 +452,9 @@ const Color& Image::GetPixel(unsigned int X, unsigned int Y) const
 ////////////////////////////////////////////////////////////
 const Uint8* Image::GetPixelsPtr() const
 {
+    // First check if the array of pixels needs to be updated
+    EnsureArrayUpdate();
+
     if (!myPixels.empty())
     {
         return reinterpret_cast<const Uint8*>(&myPixels[0]);
@@ -353,8 +473,7 @@ const Uint8* Image::GetPixelsPtr() const
 void Image::Bind() const
 {
     // First check if the texture needs to be updated
-    if (!myUpdated)
-        const_cast<Image*>(this)->Update();
+    EnsureTextureUpdate();
 
     // Bind it
     if (myTexture)
@@ -370,18 +489,24 @@ void Image::Bind() const
 ////////////////////////////////////////////////////////////
 void Image::SetSmooth(bool Smooth)
 {
-    myIsSmooth = Smooth;
-
-    if (myTexture)
+    if (Smooth != myIsSmooth)
     {
-        GLint PreviousTexture;
-        GLCheck(glGetIntegerv(GL_TEXTURE_BINDING_2D, &PreviousTexture));
+        myIsSmooth = Smooth;
 
-        GLCheck(glBindTexture(GL_TEXTURE_2D, myTexture));
-        GLCheck(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, myIsSmooth ? GL_LINEAR : GL_NEAREST));
-        GLCheck(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, myIsSmooth ? GL_LINEAR : GL_NEAREST));
+        if (myTexture)
+        {
+            // Make sure we have a valid context
+            priv::GraphicsContext Ctx;
 
-        GLCheck(glBindTexture(GL_TEXTURE_2D, PreviousTexture));
+            GLint PreviousTexture;
+            GLCheck(glGetIntegerv(GL_TEXTURE_BINDING_2D, &PreviousTexture));
+
+            GLCheck(glBindTexture(GL_TEXTURE_2D, myTexture));
+            GLCheck(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, myIsSmooth ? GL_LINEAR : GL_NEAREST));
+            GLCheck(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, myIsSmooth ? GL_LINEAR : GL_NEAREST));
+
+            GLCheck(glBindTexture(GL_TEXTURE_2D, PreviousTexture));
+        }
     }
 }
 
@@ -417,25 +542,15 @@ bool Image::IsSmooth() const
 /// Convert a subrect expressed in pixels, into float
 /// texture coordinates
 ////////////////////////////////////////////////////////////
-FloatRect Image::GetTexCoords(const IntRect& Rect, bool Adjust) const
+FloatRect Image::GetTexCoords(const IntRect& Rect) const
 {
     float Width  = static_cast<float>(myTextureWidth);
     float Height = static_cast<float>(myTextureHeight);
 
-    if (Adjust && myIsSmooth)
-    {
-        return FloatRect((Rect.Left   + 0.5f) / Width,
-                         (Rect.Top    + 0.5f) / Height,
-                         (Rect.Right  - 0.5f) / Width,
-                         (Rect.Bottom - 0.5f) / Height);
-    }
-    else
-    {
-        return FloatRect(Rect.Left   / Width,
-                         Rect.Top    / Height,
-                         Rect.Right  / Width,
-                         Rect.Bottom / Height);
-    }
+    return FloatRect(Rect.Left   / Width,
+                     Rect.Top    / Height,
+                     Rect.Right  / Width,
+                     Rect.Bottom / Height);
 }
 
 
@@ -444,6 +559,9 @@ FloatRect Image::GetTexCoords(const IntRect& Rect, bool Adjust) const
 ////////////////////////////////////////////////////////////
 unsigned int Image::GetValidTextureSize(unsigned int Size)
 {
+    // Make sure we have a valid context
+    priv::GraphicsContext Ctx;
+
     if (glewIsSupported("GL_ARB_texture_non_power_of_two") != 0)
     {
         // If hardware supports NPOT textures, then just return the unmodified size
@@ -468,13 +586,14 @@ Image& Image::operator =(const Image& Other)
 {
     Image Temp(Other);
 
-    std::swap(myWidth,         Temp.myWidth);
-    std::swap(myHeight,        Temp.myHeight);
-    std::swap(myTextureWidth,  Temp.myTextureWidth);
-    std::swap(myTextureHeight, Temp.myTextureHeight);
-    std::swap(myTexture,       Temp.myTexture);
-    std::swap(myIsSmooth,      Temp.myIsSmooth);
-    std::swap(myUpdated,       Temp.myUpdated);
+    std::swap(myWidth,             Temp.myWidth);
+    std::swap(myHeight,            Temp.myHeight);
+    std::swap(myTextureWidth,      Temp.myTextureWidth);
+    std::swap(myTextureHeight,     Temp.myTextureHeight);
+    std::swap(myTexture,           Temp.myTexture);
+    std::swap(myIsSmooth,          Temp.myIsSmooth);
+    std::swap(myNeedArrayUpdate,   Temp.myNeedArrayUpdate);
+    std::swap(myNeedTextureUpdate, Temp.myNeedTextureUpdate);
     myPixels.swap(Temp.myPixels);
 
     return *this;
@@ -487,9 +606,12 @@ Image& Image::operator =(const Image& Other)
 bool Image::CreateTexture()
 {
     // Check if texture parameters are valid before creating it
-    if (!myWidth || !myHeight || myPixels.empty())
+    if (!myWidth || !myHeight)
         return false;
 
+    // Make sure we have a valid context
+    priv::GraphicsContext Ctx;
+
     // Adjust internal texture dimensions depending on NPOT textures support
     unsigned int TextureWidth  = GetValidTextureSize(myWidth);
     unsigned int TextureHeight = GetValidTextureSize(myHeight);
@@ -506,32 +628,17 @@ bool Image::CreateTexture()
     // Destroy the previous OpenGL texture if it already exists with another size
     if ((TextureWidth != myTextureWidth) || (TextureHeight != myTextureHeight))
     {
-        DestroyVideoResources();
+        DestroyTexture();
         myTextureWidth  = TextureWidth;
         myTextureHeight = TextureHeight;
     }
 
-    // We don't actually create the OpenGL texture, to allow loading sf::Image instances
-    // without the need for a valid OpenGL context ()
-    // Creation is done in Image::Update()
-    myUpdated = false;
-
-    return true;
-}
-
-
-////////////////////////////////////////////////////////////
-/// Update the whole image in video memory
-////////////////////////////////////////////////////////////
-void Image::Update()
-{
-    GLint PreviousTexture;
-    GLCheck(glGetIntegerv(GL_TEXTURE_BINDING_2D, &PreviousTexture));
-
-    // Create the OpenGL texture if it doesn't exist
-    // Texture creation is delayed on purpose (see Image::CreateTexture())
+    // Create the OpenGL texture
     if (!myTexture)
     {
+        GLint PreviousTexture;
+        GLCheck(glGetIntegerv(GL_TEXTURE_BINDING_2D, &PreviousTexture));
+
         GLuint Texture = 0;
         GLCheck(glGenTextures(1, &Texture));
         GLCheck(glBindTexture(GL_TEXTURE_2D, Texture));
@@ -541,19 +648,90 @@ void Image::Update()
         GLCheck(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, myIsSmooth ? GL_LINEAR : GL_NEAREST));
         GLCheck(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, myIsSmooth ? GL_LINEAR : GL_NEAREST));
         myTexture = static_cast<unsigned int>(Texture);
+
+        GLCheck(glBindTexture(GL_TEXTURE_2D, PreviousTexture));
     }
 
-    // Copy the pixels
-    if (myTexture)
+    myNeedTextureUpdate = true;
+
+    return true;
+}
+
+
+////////////////////////////////////////////////////////////
+/// Make sure the texture in video memory is updated with the
+/// array of pixels
+////////////////////////////////////////////////////////////
+void Image::EnsureTextureUpdate() const
+{
+    if (myNeedTextureUpdate)
     {
-        const Uint8* Pixels = reinterpret_cast<const Uint8*>(&myPixels[0]);
-        GLCheck(glBindTexture(GL_TEXTURE_2D, myTexture));
-        GLCheck(glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, myWidth, myHeight, GL_RGBA, GL_UNSIGNED_BYTE, Pixels));
+        // Copy the pixels
+        if (myTexture && !myPixels.empty())
+        {
+            GLint PreviousTexture;
+            GLCheck(glGetIntegerv(GL_TEXTURE_BINDING_2D, &PreviousTexture));
+
+            // Update the texture with the pixels array in RAM
+            GLCheck(glBindTexture(GL_TEXTURE_2D, myTexture));
+            GLCheck(glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, myWidth, myHeight, GL_RGBA, GL_UNSIGNED_BYTE, &myPixels[0]));
+
+            GLCheck(glBindTexture(GL_TEXTURE_2D, PreviousTexture));
+        }
+
+        myNeedTextureUpdate = false;
     }
+}
 
-    GLCheck(glBindTexture(GL_TEXTURE_2D, PreviousTexture));
 
-    myUpdated = true;
+////////////////////////////////////////////////////////////
+/// Make sure the array of pixels is updated with the
+/// texture in video memory
+////////////////////////////////////////////////////////////
+void Image::EnsureArrayUpdate() const
+{
+    if (myNeedArrayUpdate)
+    {
+        // Save the previous texture
+        GLint PreviousTexture;
+        GLCheck(glGetIntegerv(GL_TEXTURE_BINDING_2D, &PreviousTexture));
+
+        // Resize the destination array of pixels
+        myPixels.resize(myWidth * myHeight);
+
+        if ((myWidth == myTextureWidth) && (myHeight == myTextureHeight))
+        {
+            // Texture and array have the same size, we can use a direct copy
+
+            // Copy pixels from texture to array
+            GLCheck(glBindTexture(GL_TEXTURE_2D, myTexture));
+            GLCheck(glGetTexImage(GL_TEXTURE_2D, 0, GL_RGBA, GL_UNSIGNED_BYTE, &myPixels[0]));
+        }
+        else
+        {
+            // Texture and array don't have the same size, we have to use a slower algorithm
+
+            // All the pixels will first be copied to a temporary array
+            std::vector<Color> AllPixels(myTextureWidth * myTextureHeight);
+            GLCheck(glBindTexture(GL_TEXTURE_2D, myTexture));
+            GLCheck(glGetTexImage(GL_TEXTURE_2D, 0, GL_RGBA, GL_UNSIGNED_BYTE, &AllPixels[0]));
+
+            // The we copy the useful pixels from the temporary array to the final one
+            const Color* Src = &AllPixels[0];
+            Color* Dst = &myPixels[0];
+            for (unsigned int i = 0; i < myHeight; ++i)
+            {
+                std::copy(Src, Src + myWidth, Dst);
+                Src += myTextureWidth;
+                Dst += myWidth;
+            }
+        }
+
+        // Restore the previous texture
+        GLCheck(glBindTexture(GL_TEXTURE_2D, PreviousTexture));
+
+        myNeedArrayUpdate = false;
+    }
 }
 
 
@@ -562,31 +740,36 @@ void Image::Update()
 ////////////////////////////////////////////////////////////
 void Image::Reset()
 {
-    DestroyVideoResources();
+    DestroyTexture();
 
-    myWidth         = 0;
-    myHeight        = 0;
-    myTextureWidth  = 0;
-    myTextureHeight = 0;
-    myTexture       = 0;
-    myIsSmooth      = true;
-    myUpdated       = false;
+    myWidth             = 0;
+    myHeight            = 0;
+    myTextureWidth      = 0;
+    myTextureHeight     = 0;
+    myTexture           = 0;
+    myIsSmooth          = true;
+    myNeedTextureUpdate = false;
+    myNeedArrayUpdate   = false;
     myPixels.clear();
 }
 
 
 ////////////////////////////////////////////////////////////
-/// Destroy all video resources that need a valid rendering context
+/// Destroy the OpenGL texture
 ////////////////////////////////////////////////////////////
-void Image::DestroyVideoResources()
+void Image::DestroyTexture()
 {
     // Destroy the internal texture
     if (myTexture)
     {
+        // Make sure we have a valid context
+        priv::GraphicsContext Ctx;
+
         GLuint Texture = static_cast<GLuint>(myTexture);
         GLCheck(glDeleteTextures(1, &Texture));
-        myTexture = 0;
-        myUpdated = false;
+        myTexture           = 0;
+        myNeedTextureUpdate = false;
+        myNeedArrayUpdate   = false;
     }
 }
 
