@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -28,7 +28,7 @@
 ////////////////////////////////////////////////////////////
 #include <SFML/Graphics/PostFX.hpp>
 #include <SFML/Graphics/RenderWindow.hpp>
-#include <SFML/Graphics/Renderer.hpp>
+#include <SFML/Graphics/GraphicsContext.hpp>
 #include <fstream>
 #include <iostream>
 #include <set>
@@ -53,9 +53,8 @@ myShaderProgram(0)
 ////////////////////////////////////////////////////////////
 PostFX::PostFX(const PostFX& Copy) :
 Drawable        (Copy),
-VideoResource   (Copy),
 myShaderProgram (0),
-mySamplers      (Copy.mySamplers),
+myTextures      (Copy.myTextures),
 myFragmentShader(Copy.myFragmentShader),
 myFrameBuffer   (Copy.myFrameBuffer)
 {
@@ -73,7 +72,14 @@ myFrameBuffer   (Copy.myFrameBuffer)
 ////////////////////////////////////////////////////////////
 PostFX::~PostFX()
 {
-    DestroyVideoResources();
+    // Destroy effect program
+    if (myShaderProgram)
+    {
+        // Make sure we have a valid context
+        priv::GraphicsContext Ctx;
+
+        GLCheck(glDeleteObjectARB(myShaderProgram));
+    }
 }
 
 
@@ -215,8 +221,25 @@ void PostFX::SetParameter(const std::string& Name, float X, float Y, float Z, fl
 ////////////////////////////////////////////////////////////
 void PostFX::SetTexture(const std::string& Name, Image* Texture)
 {
-    // Just store the texture for later use
-    mySamplers[Name] = Texture ? Texture : &myFrameBuffer;
+    // Check that the current texture unit is available
+    GLint MaxUnits;
+    GLCheck(glGetIntegerv(GL_MAX_TEXTURE_COORDS_ARB, &MaxUnits));
+    if (myTextures.size() >= static_cast<std::size_t>(MaxUnits))
+    {
+        std::cerr << "Impossible to use texture \"" << Name << "\" for post-effect : all available texture units are used" << std::endl;
+        return;
+    }
+
+    // Make sure the given name is a valid variable in the effect
+    int Location = glGetUniformLocationARB(myShaderProgram, Name.c_str());
+    if (Location == -1)
+    {
+        std::cerr << "Texture \"" << Name << "\" not found in post-effect" << std::endl;
+        return;
+    }
+
+    // Store the texture for later use
+    myTextures[Name] = Texture ? Texture : &myFrameBuffer;
 }
 
 
@@ -228,7 +251,7 @@ PostFX& PostFX::operator =(const PostFX& Other)
     PostFX Temp(Other);
 
     std::swap(myShaderProgram,  Temp.myShaderProgram);
-    std::swap(mySamplers,       Temp.mySamplers);
+    std::swap(myTextures,       Temp.myTextures);
     std::swap(myFragmentShader, Temp.myFragmentShader);
     std::swap(myFrameBuffer,    Temp.myFrameBuffer);
 
@@ -241,8 +264,8 @@ PostFX& PostFX::operator =(const PostFX& Other)
 ////////////////////////////////////////////////////////////
 bool PostFX::CanUsePostFX()
 {
-    // Make sure the renderer exists
-    GetRenderer();
+    // Make sure we have a valid context
+    priv::GraphicsContext Ctx;
 
     return glewIsSupported("GL_ARB_shading_language_100") != 0 &&
            glewIsSupported("GL_ARB_shader_objects")       != 0 &&
@@ -252,62 +275,38 @@ bool PostFX::CanUsePostFX()
 
 
 ////////////////////////////////////////////////////////////
-/// /see sfDrawable::Render
+/// /see Drawable::Render
 ////////////////////////////////////////////////////////////
-void PostFX::Render(const RenderWindow& Window) const
+void PostFX::Render(RenderTarget& Target) const
 {
     // Check that we have a valid program
     if (!myShaderProgram)
         return;
 
-    // If window dimensions have changed, recreate the frame buffer texture
-    if ((Window.GetWidth() != myFrameBuffer.GetWidth()) || (Window.GetHeight() != myFrameBuffer.GetHeight()))
-    {
-        myFrameBuffer.Resize(Window.GetWidth(), Window.GetHeight());
-    }
-
     // Copy the current framebuffer pixels to our frame buffer texture
-    myFrameBuffer.Bind();
-    GLCheck(glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, myFrameBuffer.GetWidth(), myFrameBuffer.GetHeight()));
+    // The ugly cast is temporary until PostFx are rewritten :)
+    myFrameBuffer.CopyScreen((RenderWindow&)Target);
 
     // Enable program
     GLCheck(glUseProgramObjectARB(myShaderProgram));
 
     // Bind textures
-    GLint MaxUnits;
-    GLCheck(glGetIntegerv(GL_MAX_TEXTURE_COORDS_ARB, &MaxUnits));
-    unsigned int Unit = 0;
-    for (std::map<std::string, Image*>::const_iterator i = mySamplers.begin(); i != mySamplers.end(); ++i)
+    TextureTable::const_iterator ItTex = myTextures.begin();
+    for (std::size_t i = 0; i < myTextures.size(); ++i)
     {
-        // Check that the current texture unit is available
-        if (Unit >= static_cast<unsigned int>(MaxUnits))
-        {
-            // Error : we have reached the maximum texture units count
-            std::cerr << "Impossible to use texture \"" << i->first << "\" for post-effect : all available texture units are used" << std::endl;
-            break;
-        }
-
-        // Get texture location and assign it the given unit
-        int Location = glGetUniformLocationARB(myShaderProgram, i->first.c_str());
-        if (Location != -1)
-        {
-            GLCheck(glUniform1iARB(Location, Unit));
-            GLCheck(glActiveTextureARB(GL_TEXTURE0_ARB + Unit));
-            i->second->Bind();
-            Unit++;
-        }
-        else
-        {
-            std::cerr << "Texture \"" << i->first << "\" not found in post-effect" << std::endl;
-        }
+        int Location = glGetUniformLocationARB(myShaderProgram, ItTex->first.c_str());
+        GLCheck(glUniform1iARB(Location, static_cast<GLint>(i)));
+        GLCheck(glActiveTextureARB(static_cast<GLenum>(GL_TEXTURE0_ARB + i)));
+        ItTex->second->Bind();
+        ItTex++;
     }
 
-    // Compute the texture coordinates (in case the texture is larger than the screen)
+    // Compute the texture coordinates (in case the texture is larger than the screen, or flipped)
     IntRect FrameBufferRect(0, 0, myFrameBuffer.GetWidth(), myFrameBuffer.GetHeight());
     FloatRect TexCoords = myFrameBuffer.GetTexCoords(FrameBufferRect);
 
     // Render a fullscreen quad using the effect on our framebuffer
-    FloatRect Screen = Window.GetView().GetRect();
+    FloatRect Screen = Target.GetView().GetRect();
     glBegin(GL_QUADS);
          glTexCoord2f(TexCoords.Left,  TexCoords.Top);    glVertex2f(Screen.Left,  Screen.Bottom);
          glTexCoord2f(TexCoords.Right, TexCoords.Top);    glVertex2f(Screen.Right, Screen.Bottom);
@@ -319,9 +318,9 @@ void PostFX::Render(const RenderWindow& Window) const
     GLCheck(glUseProgramObjectARB(0));
 
     // Disable texture units
-    for (unsigned int i = 0; i < Unit; ++i)
+    for (std::size_t i = 0; i < myTextures.size(); ++i)
     {
-        GLCheck(glActiveTextureARB(GL_TEXTURE0_ARB + i));
+        GLCheck(glActiveTextureARB(static_cast<GLenum>(GL_TEXTURE0_ARB + i)));
         GLCheck(glBindTexture(GL_TEXTURE_2D, 0));
     }
     GLCheck(glActiveTextureARB(GL_TEXTURE0_ARB));
@@ -412,6 +411,9 @@ std::string PostFX::PreprocessEffect(std::istream& File)
 ////////////////////////////////////////////////////////////
 void PostFX::CreateProgram()
 {
+    // Make sure we have a valid context
+    priv::GraphicsContext Ctx;
+
     // Check that we can use post-FX !
     if (!CanUsePostFX())
     {
@@ -501,18 +503,4 @@ void PostFX::CreateProgram()
     }
 }
 
-
-////////////////////////////////////////////////////////////
-/// Destroy all video resources that need a valid rendering context
-////////////////////////////////////////////////////////////
-void PostFX::DestroyVideoResources()
-{
-    // Destroy effect program
-    if (myShaderProgram)
-    {
-        GLCheck(glDeleteObjectARB(myShaderProgram));
-        myShaderProgram = 0;
-    }
-}
-
 } // namespace sf
