@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -27,7 +27,7 @@
 ////////////////////////////////////////////////////////////
 #include <SFML/Graphics/Sprite.hpp>
 #include <SFML/Graphics/Image.hpp>
-#include <SFML/Graphics/Renderer.hpp>
+#include <SFML/Graphics/GraphicsContext.hpp>
 
 
 namespace sf
@@ -36,7 +36,6 @@ namespace sf
 /// Default constructor
 ////////////////////////////////////////////////////////////
 Sprite::Sprite() :
-myImage     (NULL),
 mySubRect   (0, 0, 1, 1),
 myIsFlippedX(false),
 myIsFlippedY(false)
@@ -50,12 +49,11 @@ myIsFlippedY(false)
 ////////////////////////////////////////////////////////////
 Sprite::Sprite(const Image& Img, const Vector2f& Position, const Vector2f& Scale, float Rotation, const Color& Col) :
 Drawable    (Position, Scale, Rotation, Col),
-myImage     (&Img),
-mySubRect   (0, 0, Img.GetWidth(), Img.GetHeight()),
+mySubRect   (0, 0, 1, 1),
 myIsFlippedX(false),
 myIsFlippedY(false)
 {
-
+    SetImage(Img);
 }
 
 
@@ -64,9 +62,11 @@ myIsFlippedY(false)
 ////////////////////////////////////////////////////////////
 void Sprite::SetImage(const Image& Img)
 {
-    // If there was no source image before, adjust the rectangle
-    if (!myImage)
+    // If there was no source image before and the new image is valid, adjust the source rectangle
+    if (!myImage && (Img.GetWidth() > 0) && (Img.GetHeight() > 0))
+    {
         SetSubRect(IntRect(0, 0, Img.GetWidth(), Img.GetHeight()));
+    }
 
     // Assign the new image
     myImage = &Img;
@@ -88,10 +88,11 @@ void Sprite::SetSubRect(const IntRect& SubRect)
 ////////////////////////////////////////////////////////////
 void Sprite::Resize(float Width, float Height)
 {
-    if ((mySubRect.GetWidth() > 0) && (mySubRect.GetHeight() > 0))
-    {
-        SetScale(Width / mySubRect.GetWidth(), Height / mySubRect.GetHeight());
-    }
+    int LocalWidth  = mySubRect.GetWidth();
+    int LocalHeight = mySubRect.GetHeight();
+
+    if ((LocalWidth > 0) && (LocalHeight > 0))
+        SetScale(Width / LocalWidth, Height / LocalHeight);
 }
 
 
@@ -152,13 +153,17 @@ Vector2f Sprite::GetSize() const
 
 ////////////////////////////////////////////////////////////
 /// Get the color of a given pixel in the sprite
+/// (point is in local coordinates)
 ////////////////////////////////////////////////////////////
 Color Sprite::GetPixel(unsigned int X, unsigned int Y) const
 {
     if (myImage)
     {
-        unsigned int ImageX = mySubRect.Left + static_cast<unsigned int>(X / GetScale().x);
-        unsigned int ImageY = mySubRect.Top  + static_cast<unsigned int>(Y / GetScale().y);
+        unsigned int ImageX = mySubRect.Left + X;
+        unsigned int ImageY = mySubRect.Top  + Y;
+
+        if (myIsFlippedX) ImageX = mySubRect.GetWidth()  - ImageX - 1;
+        if (myIsFlippedY) ImageY = mySubRect.GetHeight() - ImageY - 1;
 
         return myImage->GetPixel(ImageX, ImageY) * GetColor();
     }
@@ -172,16 +177,20 @@ Color Sprite::GetPixel(unsigned int X, unsigned int Y) const
 ////////////////////////////////////////////////////////////
 /// /see sfDrawable::Render
 ////////////////////////////////////////////////////////////
-void Sprite::Render(const RenderWindow&) const
+void Sprite::Render(RenderTarget&) const
 {
     // Get the sprite size
     float Width  = static_cast<float>(mySubRect.GetWidth());
     float Height = static_cast<float>(mySubRect.GetHeight());
 
     // Check if the image is valid
-    if (myImage)
+    if (myImage && (myImage->GetWidth() > 0) && (myImage->GetHeight() > 0))
     {
-        // Set the texture
+        // Use the "offset trick" to get pixel-perfect rendering
+        // see http://www.opengl.org/resources/faq/technical/transformations.htm#tran0030
+        GLCheck(glTranslatef(0.375f, 0.375f, 0.f));
+
+        // Bind the texture
         myImage->Bind();
 
         // Calculate the texture coordinates
