@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -27,70 +27,42 @@
 ////////////////////////////////////////////////////////////
 #include <SFML/Graphics/String.hpp>
 #include <SFML/Graphics/Image.hpp>
-#include <SFML/Graphics/Renderer.hpp>
+#include <SFML/Graphics/GraphicsContext.hpp>
 #include <locale>
 
 
 namespace sf
 {
 ////////////////////////////////////////////////////////////
-/// Construct the string from a multibyte text
+/// Default constructor
 ////////////////////////////////////////////////////////////
-String::String(const std::string& Text, const Font& CharFont, float Size) :
+String::String() :
+myFont          (&Font::GetDefaultFont()),
+mySize          (30.f),
+myStyle         (Regular),
+myNeedRectUpdate(true)
+{
+
+}
+
+
+////////////////////////////////////////////////////////////
+/// Construct the string from any kind of text
+////////////////////////////////////////////////////////////
+String::String(const Unicode::Text& Text, const Font& CharFont, float Size) :
+myFont          (&CharFont),
 mySize          (Size),
 myStyle         (Regular),
 myNeedRectUpdate(true)
 {
-    SetFont(CharFont);
     SetText(Text);
 }
 
 
 ////////////////////////////////////////////////////////////
-/// Construct the string from a unicode text
+/// Set the text (from any kind of string)
 ////////////////////////////////////////////////////////////
-String::String(const std::wstring& Text, const Font& CharFont, float Size) :
-mySize          (Size),
-myStyle         (Regular),
-myNeedRectUpdate(true)
-{
-    SetFont(CharFont);
-    SetText(Text);
-}
-
-
-////////////////////////////////////////////////////////////
-/// Set the text (from a multibyte string)
-////////////////////////////////////////////////////////////
-void String::SetText(const std::string& Text)
-{
-    myNeedRectUpdate = true;
-
-    if (!Text.empty())
-    {
-        std::vector<wchar_t> Buffer(Text.size());
-
-        #ifdef __MINGW32__
-                // MinGW has a bad support for wstring conversions (and wchar_t based standard classes in general)
-                mbstowcs(&Buffer[0], Text.c_str(), Text.size());
-        #else
-                std::locale Locale("");
-                std::use_facet<std::ctype<wchar_t> >(Locale).widen(Text.data(), Text.data() + Text.size(), &Buffer[0]);
-        #endif
-
-        myText.assign(&Buffer[0], Buffer.size());
-    }
-    else
-    {
-        myText = L"";
-    }
-}
-
-
-////////////////////////////////////////////////////////////
-/// Set the text (from a unicode string)
-////////////////////////////////////////////////////////////
-void String::SetText(const std::wstring& Text)
+void String::SetText(const Unicode::Text& Text)
 {
     myNeedRectUpdate = true;
     myText = Text;
@@ -138,40 +110,14 @@ void String::SetStyle(unsigned long TextStyle)
 
 
 ////////////////////////////////////////////////////////////
-/// Get the text (returns a unicode string)
+/// Get the text (the returned text can be converted implicitely to any kind of string)
 ////////////////////////////////////////////////////////////
-const std::wstring& String::GetUnicodeText() const
+const Unicode::Text& String::GetText() const
 {
     return myText;
 }
 
 
-////////////////////////////////////////////////////////////
-/// Get the text (returns a multibyte string)
-////////////////////////////////////////////////////////////
-std::string String::GetText() const
-{
-    if (!myText.empty())
-    {
-        std::vector<char> Buffer(myText.size());
-
-        #ifdef __MINGW32__
-                // MinGW has a bad support for wstring conversions (and wchar_t based standard classes in general)
-                wcstombs(&Buffer[0], myText.c_str(), myText.size());
-        #else
-                std::locale Locale("");
-                std::use_facet<std::ctype<wchar_t> >(Locale).narrow(myText.data(), myText.data() + myText.size(), '?', &Buffer[0]);
-        #endif
-
-        return std::string(&Buffer[0], Buffer.size());
-    }
-    else
-    {
-        return "";
-    }
-}
-
-
 ////////////////////////////////////////////////////////////
 /// Get the font used by the string
 ////////////////////////////////////////////////////////////
@@ -199,6 +145,50 @@ unsigned long String::GetStyle() const
 }
 
 
+////////////////////////////////////////////////////////////
+/// Return the visual position of the Index-th character of the string,
+/// in coordinates relative to the string
+/// (note : translation, center, rotation and scale are not applied)
+////////////////////////////////////////////////////////////
+sf::Vector2f String::GetCharacterPos(std::size_t Index) const
+{
+    // First get the UTF32 representation of the text
+    const Unicode::UTF32String& Text = myText;
+
+    // Adjust the index if it's out of range
+    if (Index > Text.length())
+        Index = Text.length();
+
+    // The final size is based on the text size
+    float FactorX  = mySize / myFont->GetCharacterSize();
+    float AdvanceY = mySize;
+
+    // Compute the position
+    sf::Vector2f Position;
+    for (std::size_t i = 0; i < Index; ++i)
+    {
+        // Get the current character and its corresponding glyph
+        Uint32       CurChar  = Text[i];
+        const Glyph& CurGlyph = myFont->GetGlyph(CurChar);
+        float        AdvanceX = CurGlyph.Advance * FactorX;
+
+        switch (CurChar)
+        {
+            // Handle special characters
+            case L' ' :  Position.x += AdvanceX;                 break;
+            case L'\t' : Position.x += AdvanceX * 4;             break;
+            case L'\v' : Position.y += AdvanceY * 4;             break;
+            case L'\n' : Position.y += AdvanceY; Position.x = 0; break;
+
+            // Regular character : just add its advance value
+            default : Position.x += AdvanceX; break;
+        }
+    }
+
+    return Position;
+}
+
+
 ////////////////////////////////////////////////////////////
 /// Get the string rectangle on screen
 ////////////////////////////////////////////////////////////
@@ -220,22 +210,26 @@ FloatRect String::GetRect() const
 ////////////////////////////////////////////////////////////
 /// /see sfDrawable::Render
 ////////////////////////////////////////////////////////////
-void String::Render(const RenderWindow&) const
+void String::Render(RenderTarget&) const
 {
+    // First get the internal UTF-32 string of the text
+    const Unicode::UTF32String& Text = myText;
+
     // No text, no rendering :)
-    if (myText.empty())
+    if (Text.empty())
         return;
 
     // Set the scaling factor to get the actual size
-    float Factor = mySize / myFont->myCharSize;
+    float CharSize =  static_cast<float>(myFont->GetCharacterSize());
+    float Factor   = mySize / CharSize;
     GLCheck(glScalef(Factor, Factor, 1.f));
 
     // Bind the font texture
-    myFont->myTexture.Bind();
+    myFont->GetImage().Bind();
 
     // Initialize the rendering coordinates
     float X = 0.f;
-    float Y = static_cast<float>(myFont->myCharSize);
+    float Y = CharSize;
 
     // Holds the lines to draw later, for underlined style
     std::vector<float> UnderlineCoords;
@@ -246,40 +240,30 @@ void String::Render(const RenderWindow&) const
 
     // Draw one quad for each character
     glBegin(GL_QUADS);
-    for (std::size_t i = 0; i < myText.size(); ++i)
+    for (std::size_t i = 0; i < Text.size(); ++i)
     {
-        // Get the current character
-        wchar_t c = myText[i];
-
-        // Check if the character is in the charset
-        std::map<wchar_t, Font::Character>::const_iterator It = myFont->myCharacters.find(c);
-        if (It == myFont->myCharacters.end())
-        {
-            // No : add a space and continue to the next character
-            X += myFont->myCharSize;
-            continue;
-        }
-
-        // Get the dimensions of the current character from the font description
-        const Font::Character& CurChar = It->second;
-        const IntRect&         Rect    = CurChar.Rect;
-        const FloatRect&       Coord   = CurChar.Coord;
+        // Get the current character and its corresponding glyph
+        Uint32           CurChar  = Text[i];
+        const Glyph&     CurGlyph = myFont->GetGlyph(CurChar);
+        int              Advance  = CurGlyph.Advance;
+        const IntRect&   Rect     = CurGlyph.Rectangle;
+        const FloatRect& Coord    = CurGlyph.TexCoords;
 
         // If we're using the underlined style and there's a new line,
         // we keep track of the previous line to draw it later
-        if ((c == L'\n') && (myStyle & Underlined))
+        if ((CurChar == L'\n') && (myStyle & Underlined))
         {
             UnderlineCoords.push_back(X);
             UnderlineCoords.push_back(Y + 2);
         }
 
         // Handle special characters
-        switch (c)
+        switch (CurChar)
         {
-            case L' ' :  X += CurChar.Advance;           continue;
-            case L'\n' : Y += myFont->myCharSize; X = 0; continue;
-            case L'\t' : X += CurChar.Advance    * 4;    continue;
-            case L'\v' : Y += myFont->myCharSize * 4;    continue;
+            case L' ' :  X += Advance;         continue;
+            case L'\n' : Y += CharSize; X = 0; continue;
+            case L'\t' : X += Advance  * 4;    continue;
+            case L'\v' : Y += CharSize * 4;    continue;
         }
 
         // Draw a textured quad for the current character
@@ -305,7 +289,7 @@ void String::Render(const RenderWindow&) const
         }
 
         // Advance to the next character
-        X += CurChar.Advance;
+        X += Advance;
     }
     glEnd();
 
@@ -339,11 +323,14 @@ void String::Render(const RenderWindow&) const
 ////////////////////////////////////////////////////////////
 void String::RecomputeRect()
 {
+    // First get the internal UTF-32 string of the text
+    const Unicode::UTF32String& Text = myText;
+
     // Reset the "need update" state
     myNeedRectUpdate = false;
 
     // No text, empty box :)
-    if (myText.empty())
+    if (Text.empty())
     {
         myBaseRect = FloatRect(0, 0, 0, 0);
         return;
@@ -354,34 +341,23 @@ void String::RecomputeRect()
     float CurHeight = 0;
     float Width     = 0;
     float Height    = 0;
-    float Factor    = mySize / myFont->myCharSize;
+    float Factor    = mySize / myFont->GetCharacterSize();
 
     // Go through each character
-    for (std::size_t i = 0; i < myText.size(); ++i)
+    for (std::size_t i = 0; i < Text.size(); ++i)
     {
-        // Get the current character
-        wchar_t c = myText[i];
+        // Get the current character and its corresponding glyph
+        Uint32         CurChar  = Text[i];
+        const Glyph&   CurGlyph = myFont->GetGlyph(CurChar);
+        float          Advance  = CurGlyph.Advance * Factor;
+        const IntRect& Rect     = CurGlyph.Rectangle;
 
-        // Check if the character is in the charset
-        std::map<wchar_t, Font::Character>::const_iterator It = myFont->myCharacters.find(c);
-        if (It == myFont->myCharacters.end())
+        // Handle special characters
+        switch (CurChar)
         {
-            // No : add a space and continue to the next character
-            CurWidth += mySize;
-            continue;
-        }
-
-        // Get the dimensions of the current character from the font description
-        const Font::Character& CurChar = It->second;
-        const IntRect& Rect = CurChar.Rect;
-        float Advance = CurChar.Advance * Factor;
-
-        // Handle special characters...
-        switch (c)
-        {
-            case L' ' :  CurWidth += Advance;                   continue;
-            case L'\t' : CurWidth += mySize * 4;                continue;
-            case L'\v' : Height   += mySize * 4; CurHeight = 0; continue;
+            case L' ' :  CurWidth += Advance;                    continue;
+            case L'\t' : CurWidth += Advance * 4;                continue;
+            case L'\v' : Height   += mySize  * 4; CurHeight = 0; continue;
 
             case L'\n' :
                 Height += mySize;
@@ -396,7 +372,7 @@ void String::RecomputeRect()
         CurWidth += Advance;
 
         // Update the maximum height
-        float CharHeight = (myFont->myCharSize + Rect.Bottom) * Factor;
+        float CharHeight = (myFont->GetCharacterSize() + Rect.Bottom) * Factor;
         if (CharHeight > CurHeight)
             CurHeight = CharHeight;
     }
