@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -26,7 +26,9 @@
 // Headers
 ////////////////////////////////////////////////////////////
 #include <SFML/Network/IPAddress.hpp>
+#include <SFML/Network/Http.hpp>
 #include <SFML/Network/SocketHelper.hpp>
+#include <string.h>
 
 
 namespace sf
@@ -109,6 +111,15 @@ IPAddress::IPAddress(Uint8 Byte0, Uint8 Byte1, Uint8 Byte2, Uint8 Byte3)
 }
 
 
+////////////////////////////////////////////////////////////
+/// Construct the address from a 32-bits integer
+////////////////////////////////////////////////////////////
+IPAddress::IPAddress(Uint32 Address)
+{
+    myAddress = htonl(Address);
+}
+
+
 ////////////////////////////////////////////////////////////
 /// Tell if the address is a valid one
 ////////////////////////////////////////////////////////////
@@ -130,6 +141,15 @@ std::string IPAddress::ToString() const
 }
 
 
+////////////////////////////////////////////////////////////
+/// Get an integer representation of the address
+////////////////////////////////////////////////////////////
+Uint32 IPAddress::ToInteger() const
+{
+    return ntohl(myAddress);
+}
+
+
 ////////////////////////////////////////////////////////////
 /// Get the computer's local IP address (from the LAN point of view)
 ////////////////////////////////////////////////////////////
@@ -137,7 +157,7 @@ IPAddress IPAddress::GetLocalAddress()
 {
     // The method here is to connect a UDP socket to anyone (here to localhost),
     // and get the local socket address with the getsockname function.
-    // UDP connection will not send anything to the network, so this function won't cause any overhead
+    // UDP connection will not send anything to the network, so this function won't cause any overhead.
 
     IPAddress LocalAddress;
 
@@ -181,82 +201,22 @@ IPAddress IPAddress::GetLocalAddress()
 ////////////////////////////////////////////////////////////
 /// Get the computer's public IP address (from the web point of view)
 ////////////////////////////////////////////////////////////
-IPAddress IPAddress::GetPublicAddress()
+IPAddress IPAddress::GetPublicAddress(float Timeout)
 {
     // The trick here is more complicated, because the only way
     // to get our public IP address is to get it from a distant computer.
-    // Here we get the web page from http://www.whatismyip.org
+    // Here we get the web page from http://www.sfml-dev.org/ip-provider.php
     // and parse the result to extract our IP address
-    // (not very hard : the web page contains only our IP address)
+    // (not very hard : the web page contains only our IP address).
 
-    IPAddress PublicAddress;
+    Http Server("www.sfml-dev.org");
+    Http::Request Request(Http::Request::Get, "/ip-provider.php");
+    Http::Response Page = Server.SendRequest(Request, Timeout);
+    if (Page.GetStatus() == Http::Response::Ok)
+        return IPAddress(Page.GetBody());
 
-    // Create the socket
-    SocketHelper::SocketType Socket = socket(PF_INET, SOCK_STREAM, 0);
-    if (Socket == SocketHelper::InvalidSocket())
-        return PublicAddress;
-
-    // Build the server address (use port 80 for HTTP)
-    IPAddress Server("www.whatismyip.org");
-    sockaddr_in SockAddr;
-    memset(SockAddr.sin_zero, 0, sizeof(SockAddr.sin_zero));
-    SockAddr.sin_addr.s_addr = inet_addr(Server.ToString().c_str());
-    SockAddr.sin_family      = AF_INET;
-    SockAddr.sin_port        = htons(80);
-
-    // Connect the socket
-    if (connect(Socket, reinterpret_cast<sockaddr*>(&SockAddr), sizeof(SockAddr)) == -1)
-    {
-        SocketHelper::Close(Socket);
-        return PublicAddress;
-    }
-
-    // Send a request for getting the index page
-    const char Request[] = "GET / HTTP/1.0\r\n"
-                           "From: camembert@fromage.com\r\n"
-                           "User-Agent: SFML/1.0\r\n"
-                           "\r\n";
-    if (send(Socket, Request, sizeof(Request), 0) <= 0)
-    {
-        SocketHelper::Close(Socket);
-        return PublicAddress;
-    }
-
-    // Get the response (the source code of the web page)
-    char Response[1024] = {0};
-    int Received = 0;
-    do
-    {
-        // Get the next chunk of response from the server
-        char Buffer[1024];
-        Received = recv(Socket, Buffer, sizeof(Buffer), 0);
-
-        if (Received > 0)
-        {
-            // We received something : concatenate it to the response
-            Buffer[Received] = '\0';
-            strcat(Response, Buffer);
-        }
-        else if (Received < 0)
-        {
-            // We received an error : return an invalid address
-            return PublicAddress;
-        }
-    } while (Received > 0);
-
-    // Close the socket
-    SocketHelper::Close(Socket);
-
-    // Extract the address from the source code of the web page
-    // (extract from first \r\n\r\n to the end)
-    std::string Page = Response;
-    std::string::size_type Start = Page.find("\r\n\r\n");
-    if (Start != std::string::npos)
-    {
-        PublicAddress = Page.substr(Start + 4);
-    }
-
-    return PublicAddress;
+    // Something failed: return an invalid address
+    return IPAddress();
 }
 
 
