@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -27,6 +27,7 @@
 ////////////////////////////////////////////////////////////
 #include <SFML/Network/Packet.hpp>
 #include <SFML/Network/SocketHelper.hpp>
+#include <string.h>
 
 
 namespace sf
@@ -56,9 +57,12 @@ Packet::~Packet()
 ////////////////////////////////////////////////////////////
 void Packet::Append(const void* Data, std::size_t SizeInBytes)
 {
-    std::size_t Start = myData.size();
-    myData.resize(Start + SizeInBytes);
-    memcpy(&myData[Start], Data, SizeInBytes);
+    if (Data && (SizeInBytes > 0))
+    {
+        std::size_t Start = myData.size();
+        myData.resize(Start + SizeInBytes);
+        memcpy(&myData[Start], Data, SizeInBytes);
+    }
 }
 
 
@@ -87,9 +91,18 @@ const char* Packet::GetData() const
 ////////////////////////////////////////////////////////////
 /// Get the size of the data contained in the packet
 ////////////////////////////////////////////////////////////
-Uint32 Packet::GetDataSize() const
+std::size_t Packet::GetDataSize() const
 {
-    return static_cast<Uint32>(myData.size());
+    return myData.size();
+}
+
+
+////////////////////////////////////////////////////////////
+/// Tell if the reading position has reached the end of the packet
+////////////////////////////////////////////////////////////
+bool Packet::EndOfPacket() const
+{
+    return myReadPos >= myData.size();
 }
 
 
@@ -105,6 +118,14 @@ Packet::operator bool() const
 ////////////////////////////////////////////////////////////
 /// Operator >> overloads to extract data from the packet
 ////////////////////////////////////////////////////////////
+Packet& Packet::operator >>(bool& Data)
+{
+    Uint8 Value;
+    if (*this >> Value)
+        Data = (Value != 0);
+
+    return *this;
+}
 Packet& Packet::operator >>(Int8& Data)
 {
     if (CheckSize(sizeof(Data)))
@@ -191,7 +212,7 @@ Packet& Packet::operator >>(char* Data)
     Uint32 Length;
     *this >> Length;
 
-    if (CheckSize(Length))
+    if ((Length > 0) && CheckSize(Length))
     {
         // Then extract characters
         memcpy(Data, GetData() + myReadPos, Length);
@@ -209,7 +230,8 @@ Packet& Packet::operator >>(std::string& Data)
     Uint32 Length;
     *this >> Length;
 
-    if (CheckSize(Length))
+    Data.clear();
+    if ((Length > 0) && CheckSize(Length))
     {
         // Then extract characters
         Data.assign(GetData() + myReadPos, Length);
@@ -226,7 +248,7 @@ Packet& Packet::operator >>(wchar_t* Data)
     Uint32 Length;
     *this >> Length;
 
-    if (CheckSize(Length * sizeof(Int32)))
+    if ((Length > 0) && CheckSize(Length * sizeof(Int32)))
     {
         // Then extract characters
         for (Uint32 i = 0; i < Length; ++i)
@@ -246,10 +268,10 @@ Packet& Packet::operator >>(std::wstring& Data)
     Uint32 Length;
     *this >> Length;
 
-    if (CheckSize(Length * sizeof(Int32)))
+    Data.clear();
+    if ((Length > 0) && CheckSize(Length * sizeof(Int32)))
     {
         // Then extract characters
-        Data.clear();
         for (Uint32 i = 0; i < Length; ++i)
         {
             Uint32 c;
@@ -265,6 +287,11 @@ Packet& Packet::operator >>(std::wstring& Data)
 ////////////////////////////////////////////////////////////
 /// Operator << overloads to put data into the packet
 ////////////////////////////////////////////////////////////
+Packet& Packet::operator <<(bool Data)
+{
+    *this << static_cast<Uint8>(Data);
+    return *this;
+}
 Packet& Packet::operator <<(Int8 Data)
 {
     Append(&Data, sizeof(Data));
@@ -329,7 +356,10 @@ Packet& Packet::operator <<(const std::string& Data)
     *this << Length;
 
     // Then insert characters
-    Append(Data.c_str(), Length * sizeof(std::string::value_type));
+    if (Length > 0)
+    {
+        Append(Data.c_str(), Length * sizeof(std::string::value_type));
+    }
 
     return *this;
 }
@@ -354,8 +384,11 @@ Packet& Packet::operator <<(const std::wstring& Data)
     *this << Length;
 
     // Then insert characters
-    for (std::wstring::const_iterator c = Data.begin(); c != Data.end(); ++c)
-        *this << static_cast<Int32>(*c);
+    if (Length > 0)
+    {
+        for (std::wstring::const_iterator c = Data.begin(); c != Data.end(); ++c)
+            *this << static_cast<Int32>(*c);
+    }
 
     return *this;
 }
@@ -366,33 +399,28 @@ Packet& Packet::operator <<(const std::wstring& Data)
 ////////////////////////////////////////////////////////////
 bool Packet::CheckSize(std::size_t Size)
 {
-    if (myIsValid && (myReadPos + Size <= myData.size()))
-    {
-        return true;
-    }
-    else
-    {
-        myIsValid = false;
-        return false;
-    }
+    myIsValid = myIsValid && (myReadPos + Size <= myData.size());
+
+    return myIsValid;
 }
 
 
 ////////////////////////////////////////////////////////////
-/// Called before the packet is send to the network
+/// Called before the packet is sent to the network
 ////////////////////////////////////////////////////////////
-void Packet::OnSend()
+const char* Packet::OnSend(std::size_t& DataSize)
 {
-    // Nothing by default
+    DataSize = GetDataSize();
+    return GetData();
 }
 
 
 ////////////////////////////////////////////////////////////
 /// Called after the packet has been received from the network
 ////////////////////////////////////////////////////////////
-void Packet::OnReceive()
+void Packet::OnReceive(const char* Data, std::size_t DataSize)
 {
-    // Nothing by default
+    Append(Data, DataSize);
 }
 
 } // namespace sf
