@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -30,6 +30,7 @@
 #include <SFML/Network/Packet.hpp>
 #include <algorithm>
 #include <iostream>
+#include <string.h>
 
 
 namespace sf
@@ -37,9 +38,7 @@ namespace sf
 ////////////////////////////////////////////////////////////
 /// Default constructor
 ////////////////////////////////////////////////////////////
-SocketUDP::SocketUDP() :
-mySocket(SocketHelper::InvalidSocket()),
-myPort  (0)
+SocketUDP::SocketUDP()
 {
     Create();
 }
@@ -55,6 +54,7 @@ void SocketUDP::SetBlocking(bool Blocking)
         Create();
 
     SocketHelper::SetBlocking(mySocket, Blocking);
+    myIsBlocking = Blocking;
 }
 
 
@@ -66,12 +66,8 @@ bool SocketUDP::Bind(unsigned short Port)
     // Check if the socket is already bound to the specified port
     if (myPort != Port)
     {
-        // If the socket was previously bound to another port, we need to recreate it
-        if (myPort != 0)
-        {
-            Close();
-            Create();
-        }
+        // If the socket was previously bound to another port, we need to unbind it first
+        Unbind();
 
         if (Port != 0)
         {
@@ -163,7 +159,7 @@ Socket::Status SocketUDP::Send(const char* Data, std::size_t Size, const IPAddre
 /// Receive an array of bytes.
 /// This function will block if the socket is blocking
 ////////////////////////////////////////////////////////////
-Socket::Status SocketUDP::Receive(char* Data, std::size_t MaxSize, std::size_t& SizeReceived, IPAddress& Address)
+Socket::Status SocketUDP::Receive(char* Data, std::size_t MaxSize, std::size_t& SizeReceived, IPAddress& Address, unsigned short& Port)
 {
     // First clear the size received
     SizeReceived = 0;
@@ -185,7 +181,7 @@ Socket::Status SocketUDP::Receive(char* Data, std::size_t MaxSize, std::size_t&
         // Data that will be filled with the other computer's address
         sockaddr_in Sender;
         Sender.sin_family      = AF_INET;
-        Sender.sin_port        = htons(myPort);
+        Sender.sin_port        = 0;
         Sender.sin_addr.s_addr = INADDR_ANY;
         memset(Sender.sin_zero, 0, sizeof(Sender.sin_zero));
         SocketHelper::LengthType SenderSize = sizeof(Sender);
@@ -197,12 +193,14 @@ Socket::Status SocketUDP::Receive(char* Data, std::size_t MaxSize, std::size_t&
         if (Received > 0)
         {
             Address = IPAddress(inet_ntoa(Sender.sin_addr));
+            Port = ntohs(Sender.sin_port);
             SizeReceived = static_cast<std::size_t>(Received);
             return Socket::Done;
         }
         else
         {
             Address = IPAddress();
+            Port = 0;
             return Received == 0 ? Socket::Disconnected : SocketHelper::GetErrorStatus();
         }
     }
@@ -220,17 +218,18 @@ Socket::Status SocketUDP::Receive(char* Data, std::size_t MaxSize, std::size_t&
 ////////////////////////////////////////////////////////////
 Socket::Status SocketUDP::Send(Packet& PacketToSend, const IPAddress& Address, unsigned short Port)
 {
-    // Let the packet do custom stuff before sending it
-    PacketToSend.OnSend();
+    // Get the data to send from the packet
+    std::size_t DataSize = 0;
+    const char* Data = PacketToSend.OnSend(DataSize);
 
-    // First send the packet size
-    Uint32 PacketSize = htonl(PacketToSend.GetDataSize());
+    // Send the packet size
+    Uint32 PacketSize = htonl(static_cast<unsigned long>(DataSize));
     Send(reinterpret_cast<const char*>(&PacketSize), sizeof(PacketSize), Address, Port);
 
     // Send the packet data
     if (PacketSize > 0)
     {
-        return Send(PacketToSend.GetData(), PacketToSend.GetDataSize(), Address, Port);
+        return Send(Data, DataSize, Address, Port);
     }
     else
     {
@@ -243,22 +242,27 @@ Socket::Status SocketUDP::Send(Packet& PacketToSend, const IPAddress& Address, u
 /// Receive a packet.
 /// This function will block if the socket is blocking
 ////////////////////////////////////////////////////////////
-Socket::Status SocketUDP::Receive(Packet& PacketToReceive, IPAddress& Address)
+Socket::Status SocketUDP::Receive(Packet& PacketToReceive, IPAddress& Address, unsigned short& Port)
 {
-    // This is not safe at all, as data can be lost, duplicated, or arrive in a different order.
-    // So if a packet is split into more than one chunk, nobody knows what could happen...
-    // Conclusion : we shouldn't use packets with UDP, unless we build a more complex protocol on top of it.
-
     // We start by getting the size of the incoming packet
     Uint32      PacketSize = 0;
     std::size_t Received   = 0;
     if (myPendingPacketSize < 0)
     {
-        Socket::Status Status = Receive(reinterpret_cast<char*>(&PacketSize), sizeof(PacketSize), Received, Address);
-        if (Status != Socket::Done)
-            return Status;
+        // Loop until we've received the entire size of the packet
+        // (even a 4 bytes variable may be received in more than one call)
+        while (myPendingHeaderSize < sizeof(myPendingHeader))
+        {
+            char* Data = reinterpret_cast<char*>(&myPendingHeader) + myPendingHeaderSize;
+            Socket::Status Status = Receive(Data, sizeof(myPendingHeader) - myPendingHeaderSize, Received, Address, Port);
+            myPendingHeaderSize += Received;
 
-        PacketSize = ntohl(PacketSize);
+            if (Status != Socket::Done)
+                return Status;
+        }
+
+        PacketSize = ntohl(myPendingHeader);
+        myPendingHeaderSize = 0;
     }
     else
     {
@@ -266,20 +270,18 @@ Socket::Status SocketUDP::Receive(Packet& PacketToReceive, IPAddress& Address)
         PacketSize = myPendingPacketSize;
     }
 
-    // Clear the user packet
-    PacketToReceive.Clear();
-
     // Use another address instance for receiving the packet data ;
     // chunks of data coming from a different sender will be discarded (and lost...)
     IPAddress Sender;
+    unsigned short SenderPort;
 
     // Then loop until we receive all the packet data
     char Buffer[1024];
-    while (myPendingPacket.GetDataSize() < PacketSize)
+    while (myPendingPacket.size() < PacketSize)
     {
         // Receive a chunk of data
-        Uint32 SizeToGet = std::min(PacketSize - myPendingPacket.GetDataSize(), static_cast<Uint32>(sizeof(Buffer)));
-        Socket::Status Status = Receive(Buffer, SizeToGet, Received, Sender);
+        std::size_t SizeToGet = std::min(static_cast<std::size_t>(PacketSize - myPendingPacket.size()), sizeof(Buffer));
+        Socket::Status Status = Receive(Buffer, SizeToGet, Received, Sender, SenderPort);
         if (Status != Socket::Done)
         {
             // We must save the size of the pending packet until we can receive its content
@@ -289,18 +291,21 @@ Socket::Status SocketUDP::Receive(Packet& PacketToReceive, IPAddress& Address)
         }
 
         // Append it into the packet
-        if (Sender == Address)
-            myPendingPacket.Append(Buffer, Received);
+        if ((Sender == Address) && (SenderPort == Port) && (Received > 0))
+        {
+            myPendingPacket.resize(myPendingPacket.size() + Received);
+            char* Begin = &myPendingPacket[0] + myPendingPacket.size() - Received;
+            memcpy(Begin, Buffer, Received);
+        }
     }
 
     // We have received all the datas : we can copy it to the user packet, and clear our internal packet
-    PacketToReceive = myPendingPacket;
-    myPendingPacket.Clear();
+    PacketToReceive.Clear();
+    if (!myPendingPacket.empty())
+        PacketToReceive.OnReceive(&myPendingPacket[0], myPendingPacket.size());
+    myPendingPacket.clear();
     myPendingPacketSize = -1;
 
-    // Let the packet do custom stuff after data reception
-    PacketToReceive.OnReceive();
-
     return Socket::Done;
 }
 
@@ -310,14 +315,19 @@ Socket::Status SocketUDP::Receive(Packet& PacketToReceive, IPAddress& Address)
 ////////////////////////////////////////////////////////////
 bool SocketUDP::Close()
 {
-    if (!SocketHelper::Close(mySocket))
+    if (IsValid())
     {
-        std::cerr << "Failed to close socket" << std::endl;
-        return false;
+        if (!SocketHelper::Close(mySocket))
+        {
+            std::cerr << "Failed to close socket" << std::endl;
+            return false;
+        }
+
+        mySocket = SocketHelper::InvalidSocket();
     }
 
-    mySocket = SocketHelper::InvalidSocket();
-    myPort   = 0;
+    myPort       = 0;
+    myIsBlocking = true;
 
     return true;
 }
@@ -375,46 +385,49 @@ bool SocketUDP::operator <(const SocketUDP& Other) const
 /// Construct the socket from a socket descriptor
 /// (for internal use only)
 ////////////////////////////////////////////////////////////
-SocketUDP::SocketUDP(SocketHelper::SocketType Descriptor) :
-mySocket(Descriptor),
-myPort  (0)
+SocketUDP::SocketUDP(SocketHelper::SocketType Descriptor)
 {
-    // Set blocking by default (should always be the case anyway)
-    SetBlocking(true);
+    Create(Descriptor);
 }
 
 
 ////////////////////////////////////////////////////////////
 /// Create the socket
 ////////////////////////////////////////////////////////////
-void SocketUDP::Create()
+void SocketUDP::Create(SocketHelper::SocketType Descriptor)
 {
-    // Get a new socket descriptor
-    mySocket = socket(PF_INET, SOCK_DGRAM, 0);
+    // Use the given socket descriptor, or get a new one
+    mySocket = Descriptor ? Descriptor : socket(PF_INET, SOCK_DGRAM, 0);
+    myIsBlocking = true;
 
     // Clear the last port used
     myPort = 0;
 
-    // To avoid the "Address already in use" error message when trying to bind to the same port
-    char Yes = 1;
-    if (setsockopt(mySocket, SOL_SOCKET, SO_REUSEADDR, &Yes, sizeof(int)) == -1)
-    {
-        std::cerr << "Failed to set socket option \"reuse address\" ; "
-                  << "binding to a same port may fail if too fast" << std::endl;
-    }
-
-    // Enable broadcast by default
-    if (setsockopt(mySocket, SOL_SOCKET, SO_BROADCAST, &Yes, sizeof(int)) == -1)
-    {
-        std::cerr << "Failed to enable broadcast on UDP socket" << std::endl;
-    }
-
-    // Set blocking by default (should always be the case anyway)
-    SetBlocking(true);
-
     // Reset the pending packet
-    myPendingPacket.Clear();
+    myPendingHeaderSize = 0;
+    myPendingPacket.clear();
     myPendingPacketSize = -1;
+
+    // Setup default options
+    if (IsValid())
+    {
+        // To avoid the "Address already in use" error message when trying to bind to the same port
+        int Yes = 1;
+        if (setsockopt(mySocket, SOL_SOCKET, SO_REUSEADDR, reinterpret_cast<char*>(&Yes), sizeof(Yes)) == -1)
+        {
+            std::cerr << "Failed to set socket option \"reuse address\" ; "
+                      << "binding to a same port may fail if too fast" << std::endl;
+        }
+
+        // Enable broadcast by default
+        if (setsockopt(mySocket, SOL_SOCKET, SO_BROADCAST, reinterpret_cast<char*>(&Yes), sizeof(Yes)) == -1)
+        {
+            std::cerr << "Failed to enable broadcast on UDP socket" << std::endl;
+        }
+
+        // Set blocking by default (should always be the case anyway)
+        SetBlocking(true);
+    }
 }
 
 } // namespace sf
