@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -26,8 +26,8 @@
 // Headers
 ////////////////////////////////////////////////////////////
 #include <SFML/System/Randomizer.hpp>
-#include <SFML/System/Platform.hpp>
-#include <cstdlib>
+#include <stdlib.h>
+#include <time.h>
 
 
 namespace
@@ -35,7 +35,7 @@ namespace
     // Set the random numbers sequence seed with the current system time, so that it is always different
     unsigned int SetRandomSeed()
     {
-        unsigned int Seed = static_cast<unsigned int>(sf::priv::Platform::GetSystemTime() * 1000);
+        unsigned int Seed = static_cast<unsigned int>(time(NULL));
         srand(Seed);
         return Seed;
     }
