@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -62,8 +62,7 @@ myUserData(UserData)
 Thread::~Thread()
 {
     // Wait for the thread to finish before destroying the instance
-    if (myHandle)
-        Wait();
+    Wait();
 }
 
 
@@ -72,6 +71,9 @@ Thread::~Thread()
 ////////////////////////////////////////////////////////////
 void Thread::Launch()
 {
+    // Wait for the thread to finish, in case it was already running
+    Wait();
+
     // Create the thread
     myHandle = reinterpret_cast<HANDLE>(_beginthreadex(NULL, 0, &Thread::ThreadFunc, this, 0, NULL));
 
@@ -129,7 +131,7 @@ void Thread::Run()
 ////////////////////////////////////////////////////////////
 unsigned int __stdcall Thread::ThreadFunc(void* UserData)
 {
-    // The sfThread instance is stored in the user data
+    // The Thread instance is stored in the user data
     Thread* ThreadInstance = reinterpret_cast<Thread*>(UserData);
 
     // Forward to the instance
