@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -37,23 +37,7 @@ Input::Input() :
 myMouseX(0),
 myMouseY(0)
 {
-    for (int i = 0; i < Key::Count; ++i)
-        myKeys[i] = false;
-
-    for (int i = 0; i < Mouse::Count; ++i)
-        myMouseButtons[i] = false;
-
-    for (int i = 0; i < 16; ++i)
-    {
-        myJoystickButtons[0][i] = false;
-        myJoystickButtons[1][i] = false;
-    }
-
-    for (int i = 0; i < Joy::Count; ++i)
-    {
-        myJoystickAxis[0][i] = 0.f;
-        myJoystickAxis[1][i] = 0.f;
-    }
+    ResetStates();
 }
 
 
@@ -80,7 +64,7 @@ bool Input::IsMouseButtonDown(Mouse::Button Button) const
 ////////////////////////////////////////////////////////////
 bool Input::IsJoystickButtonDown(unsigned int JoyId, unsigned int Button) const
 {
-    if ((JoyId < 2) && (Button < 16))
+    if ((JoyId < Joy::Count) && (Button < Joy::ButtonCount))
         return myJoystickButtons[JoyId][Button];
     else
         return false;
@@ -90,7 +74,7 @@ bool Input::IsJoystickButtonDown(unsigned int JoyId, unsigned int Button) const
 ////////////////////////////////////////////////////////////
 /// Get the mouse left position
 ////////////////////////////////////////////////////////////
-unsigned int Input::GetMouseX() const
+int Input::GetMouseX() const
 {
     return myMouseX;
 }
@@ -99,7 +83,7 @@ unsigned int Input::GetMouseX() const
 ////////////////////////////////////////////////////////////
 /// Get the mouse top position
 ////////////////////////////////////////////////////////////
-unsigned int Input::GetMouseY() const
+int Input::GetMouseY() const
 {
     return myMouseY;
 }
@@ -110,12 +94,15 @@ unsigned int Input::GetMouseY() const
 ////////////////////////////////////////////////////////////
 float Input::GetJoystickAxis(unsigned int JoyId, Joy::Axis Axis) const
 {
-    return myJoystickAxis[JoyId][Axis];
+    if (JoyId < Joy::Count)
+        return myJoystickAxis[JoyId][Axis];
+    else
+        return 0.f;
 }
 
 
 ////////////////////////////////////////////////////////////
-/// /see sfWindowListener::OnEvent
+/// /see WindowListener::OnEvent
 ////////////////////////////////////////////////////////////
 void Input::OnEvent(const Event& EventReceived)
 {
@@ -147,17 +134,7 @@ void Input::OnEvent(const Event& EventReceived)
         // Lost focus event : we must reset all persistent states
         case Event::LostFocus :
         {
-            for (int i = 0; i < Key::Count; ++i)
-                myKeys[i] = false;
-
-            for (int i = 0; i < Mouse::Count; ++i)
-                myMouseButtons[i] = false;
-
-            for (int i = 0; i < 16; ++i)
-            {
-                myJoystickButtons[0][i] = false;
-                myJoystickButtons[1][i] = false;
-            }
+            ResetStates();
             break;
         }
 
@@ -166,4 +143,27 @@ void Input::OnEvent(const Event& EventReceived)
     }
 }
 
+
+////////////////////////////////////////////////////////////
+/// Reset all the states
+////////////////////////////////////////////////////////////
+void Input::ResetStates()
+{
+    for (int i = 0; i < Key::Count; ++i)
+        myKeys[i] = false;
+
+    for (int i = 0; i < Mouse::ButtonCount; ++i)
+        myMouseButtons[i] = false;
+
+    for (int i = 0; i < Joy::Count; ++i)
+    {
+        for (int j = 0; j < Joy::ButtonCount; ++j)
+            myJoystickButtons[i][j] = false;
+
+        for (int j = 0; j < Joy::AxisCount; ++j)
+            myJoystickAxis[i][j] = 0.f;
+        myJoystickAxis[i][Joy::AxisPOV] = -1.f;
+    }
+}
+
 } // namespace sf
