@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -29,13 +29,27 @@
 #include <SFML/Window/Linux/WindowImplX11.hpp>
 #include <SFML/Window/glext/glxext.h>
 #include <SFML/Window/glext/glext.h>
+#include <SFML/System/Unicode.hpp>
 #include <X11/keysym.h>
-#include <X11/XKBlib.h>
 #include <X11/extensions/Xrandr.h>
 #include <iostream>
 #include <sstream>
+#include <vector>
 
 
+namespace
+{
+    ////////////////////////////////////////////////////////////
+    /// Filter the events received by windows
+    /// (only allow those matching a specific window)
+    ////////////////////////////////////////////////////////////
+    Bool CheckEvent(::Display*, XEvent* Event, XPointer UserData)
+    {
+        // Just check if the event matches the window
+        return Event->xany.window == reinterpret_cast< ::Window >(UserData);
+    }
+}
+
 namespace sf
 {
 namespace priv
@@ -47,9 +61,10 @@ Display*       WindowImplX11::ourDisplay          = NULL;
 int            WindowImplX11::ourScreen           = 0;
 WindowImplX11* WindowImplX11::ourFullscreenWindow = NULL;
 unsigned int   WindowImplX11::ourWindowsCount     = 0;
-unsigned long  WindowImplX11::ourEventMask        = FocusChangeMask | ButtonPressMask | ButtonReleaseMask | ButtonMotionMask |
-                                                    PointerMotionMask | KeyPressMask | KeyReleaseMask | StructureNotifyMask;
 XIM            WindowImplX11::ourInputMethod      = NULL;
+unsigned long  WindowImplX11::ourEventMask        = FocusChangeMask | ButtonPressMask | ButtonReleaseMask | ButtonMotionMask |
+                                                    PointerMotionMask | KeyPressMask | KeyReleaseMask | StructureNotifyMask |
+                                                    EnterWindowMask | LeaveWindowMask;
 
 
 ////////////////////////////////////////////////////////////
@@ -63,7 +78,8 @@ myGLContext   (NULL),
 myAtomClose   (0),
 myOldVideoMode(-1),
 myHiddenCursor(0),
-myInputContext(NULL)
+myInputContext(NULL),
+myKeyRepeat   (true)
 {
     // Open the display at first call
     if (!OpenDisplay())
@@ -97,8 +113,7 @@ myInputContext(NULL)
                              Visual.visual,
                              CWColormap, &Attributes);
 
-    // Set our context as the current OpenGL context for rendering
-    SetActive();
+    // Don't activate the dummy context by default
 }
 
 
@@ -112,7 +127,8 @@ myGLContext   (NULL),
 myAtomClose   (0),
 myOldVideoMode(-1),
 myHiddenCursor(0),
-myInputContext(NULL)
+myInputContext(NULL),
+myKeyRepeat   (true)
 {
     // Open the display at first call
     if (!OpenDisplay())
@@ -168,7 +184,8 @@ myGLContext   (NULL),
 myAtomClose   (0),
 myOldVideoMode(-1),
 myHiddenCursor(0),
-myInputContext(NULL)
+myInputContext(NULL),
+myKeyRepeat   (true)
 {
     // Open the display at first call
     if (!OpenDisplay())
@@ -334,6 +351,15 @@ WindowImplX11::~WindowImplX11()
 }
 
 
+////////////////////////////////////////////////////////////
+/// Check if there's an active context on the current thread
+////////////////////////////////////////////////////////////
+bool WindowImplX11::IsContextActive()
+{
+    return glXGetCurrentContext() != NULL;
+}
+
+
 ////////////////////////////////////////////////////////////
 /// /see WindowImpl::Display
 ////////////////////////////////////////////////////////////
@@ -349,26 +375,70 @@ void WindowImplX11::Display()
 ////////////////////////////////////////////////////////////
 void WindowImplX11::ProcessEvents()
 {
+    // This function implements a workaround to properly discard
+    // repeated key events when necessary. The problem is that the
+    // system's key events policy doesn't match SFML's one: X server will generate
+    // both repeated KeyPress and KeyRelease events when maintaining a key down, while
+    // SFML only wants repeated KeyPress events. Thus, we have to:
+    // - Discard duplicated KeyRelease events when EnableKeyRepeat is true
+    // - Discard both duplicated KeyPress and KeyRelease events when EnableKeyRepeat is false
+
+
     // Process any event in the queue matching our window
     XEvent Event;
-    while (XCheckIfEvent(ourDisplay, &Event, &WindowImplX11::CheckEvent, reinterpret_cast<XPointer>(myWindow)))
+    while (XCheckIfEvent(ourDisplay, &Event, &CheckEvent, reinterpret_cast<XPointer>(myWindow)))
+    {
+        // Detect repeated key events
+        if ((Event.type == KeyPress) || (Event.type == KeyRelease))
+        {
+            if (Event.xkey.keycode < 256)
+            {
+                // To detect if it is a repeated key event, we check the current state of the key.
+                // - If the state is "down", KeyReleased events must obviously be discarded.
+                // - KeyPress events are a little bit harder to handle: they depend on the EnableKeyRepeat state,
+                //   and we need to properly forward the first one.
+                char Keys[32];
+                XQueryKeymap(ourDisplay, Keys);
+                if (Keys[Event.xkey.keycode >> 3] & (1 << (Event.xkey.keycode % 8)))
+                {
+                    // KeyRelease event + key down = repeated event --> discard
+                    if (Event.type == KeyRelease)
+                    {
+                        myLastKeyReleaseEvent = Event;
+                        continue;
+                    }
+
+                    // KeyPress event + key repeat disabled + matching KeyRelease event = repeated event --> discard
+                    if ((Event.type == KeyPress) && !myKeyRepeat &&
+                        (myLastKeyReleaseEvent.xkey.keycode == Event.xkey.keycode) &&
+                        (myLastKeyReleaseEvent.xkey.time == Event.xkey.time))
+                    {
+                        continue;
+                    }
+                }
+            }
+        }
+
+        // Process the event
         ProcessEvent(Event);
+   }
 }
 
 
 ////////////////////////////////////////////////////////////
-/// /see WindowImpl::MakeActive
+/// /see WindowImpl::SetActive
 ////////////////////////////////////////////////////////////
-void WindowImplX11::MakeActive(bool Active) const
+void WindowImplX11::SetActive(bool Active) const
 {
     if (Active)
     {
-        if (myWindow && myGLContext)
+        if (myWindow && myGLContext && (glXGetCurrentContext() != myGLContext))
             glXMakeCurrent(ourDisplay, myWindow, myGLContext);
     }
     else
     {
-        glXMakeCurrent(ourDisplay, None, NULL);
+        if (glXGetCurrentContext() == myGLContext)
+            glXMakeCurrent(ourDisplay, None, NULL);
     }
 }
 
@@ -415,6 +485,16 @@ void WindowImplX11::SetPosition(int Left, int Top)
 }
 
 
+////////////////////////////////////////////////////////////
+/// /see WindowImpl::SetSize
+////////////////////////////////////////////////////////////
+void WindowImplX11::SetSize(unsigned int Width, unsigned int Height)
+{
+    XResizeWindow(ourDisplay, myWindow, Width, Height);
+    XFlush(ourDisplay);
+}
+
+
 ////////////////////////////////////////////////////////////
 /// /see sfWindowImpl::Show
 ////////////////////////////////////////////////////////////
@@ -434,10 +514,68 @@ void WindowImplX11::Show(bool State)
 ////////////////////////////////////////////////////////////
 void WindowImplX11::EnableKeyRepeat(bool Enabled)
 {
-    if (Enabled)
-        XAutoRepeatOn(ourDisplay);
-    else
-        XAutoRepeatOff(ourDisplay);
+    myKeyRepeat = Enabled;
+}
+
+
+////////////////////////////////////////////////////////////
+/// /see WindowImpl::SetIcon
+////////////////////////////////////////////////////////////
+void WindowImplX11::SetIcon(unsigned int Width, unsigned int Height, const Uint8* Pixels)
+{
+    // X11 wants BGRA pixels : swap red and blue channels
+    // Note : this memory will never be freed, but it seems to cause a bug on exit if I do so
+    Uint8* IconPixels = new Uint8[Width * Height * 4];
+    for (std::size_t i = 0; i < Width * Height; ++i)
+    {
+        IconPixels[i * 4 + 0] = Pixels[i * 4 + 2];
+        IconPixels[i * 4 + 1] = Pixels[i * 4 + 1];
+        IconPixels[i * 4 + 2] = Pixels[i * 4 + 0];
+        IconPixels[i * 4 + 3] = Pixels[i * 4 + 3];
+    }
+
+    // Create the icon pixmap
+    Visual*      DefVisual = DefaultVisual(ourDisplay, ourScreen);
+    unsigned int DefDepth  = DefaultDepth(ourDisplay, ourScreen);
+    XImage* IconImage = XCreateImage(ourDisplay, DefVisual, DefDepth, ZPixmap, 0, (char*)IconPixels, Width, Height, 32, 0);
+    if (!IconImage)
+    {
+        std::cerr << "Failed to set the window's icon" << std::endl;
+        return;
+    }
+    Pixmap IconPixmap = XCreatePixmap(ourDisplay, RootWindow(ourDisplay, ourScreen), Width, Height, DefDepth);
+    XGCValues Values;
+    GC IconGC = XCreateGC(ourDisplay, IconPixmap, 0, &Values);
+    XPutImage(ourDisplay, IconPixmap, IconGC, IconImage, 0, 0, 0, 0, Width, Height);
+    XFreeGC(ourDisplay, IconGC);
+    XDestroyImage(IconImage);
+
+    // Create the mask pixmap (must have 1 bit depth)
+    std::size_t Pitch = (Width + 7) / 8;
+    static std::vector<Uint8> MaskPixels(Pitch * Height, 0);
+    for (std::size_t j = 0; j < Height; ++j)
+    {
+        for (std::size_t i = 0; i < Pitch; ++i)
+        {
+            for (std::size_t k = 0; k < 8; ++k)
+            {
+                if (i * 8 + k < Width)
+                {
+                    Uint8 Opacity = (Pixels[(i * 8 + k + j * Width) * 4 + 3] > 0) ? 1 : 0;
+                    MaskPixels[i + j * Pitch] |= (Opacity << k);                    
+                }
+            }
+        }
+    }
+    Pixmap MaskPixmap = XCreatePixmapFromBitmapData(ourDisplay, myWindow, (char*)&MaskPixels[0], Width, Height, 1, 0, 1);
+
+    // Send our new icon to the window through the WMHints
+    XWMHints* Hints = XAllocWMHints();
+    Hints->flags       = IconPixmapHint | IconMaskHint;
+    Hints->icon_pixmap = IconPixmap;
+    Hints->icon_mask   = MaskPixmap;
+    XSetWMHints(ourDisplay, myWindow, Hints);
+    XFree(Hints);
 
     XFlush(ourDisplay);
 }
@@ -547,7 +685,6 @@ bool WindowImplX11::CreateContext(const VideoMode& Mode, XVisualInfo& ChosenVisu
             {
                 BestScore  = Score;
                 BestVisual = &Visuals[i];
-                break;
             }
         }
 
@@ -592,6 +729,9 @@ bool WindowImplX11::CreateContext(const VideoMode& Mode, XVisualInfo& ChosenVisu
     ChosenVisual = *BestVisual;
     XFree(Visuals);
 
+    // Activate the context
+    SetActive(true);
+
     // Enable multisampling if needed
     if (Params.AntialiasingLevel > 0)
         glEnable(GL_MULTISAMPLE_ARB);
@@ -605,6 +745,9 @@ bool WindowImplX11::CreateContext(const VideoMode& Mode, XVisualInfo& ChosenVisu
 ////////////////////////////////////////////////////////////
 void WindowImplX11::Initialize()
 {
+    // Make sure the "last key release" is initialized with invalid values
+    myLastKeyReleaseEvent.type = -1;
+
     // Get the atom defining the close event
     myAtomClose = XInternAtom(ourDisplay, "WM_DELETE_WINDOW", false);
     XSetWMProtocols(ourDisplay, myWindow, &myAtomClose, 1);
@@ -616,8 +759,6 @@ void WindowImplX11::Initialize()
                                    XNClientWindow,  myWindow,
                                    XNFocusWindow,   myWindow,
                                    XNInputStyle,    XIMPreeditNothing  | XIMStatusNothing,
-                                   //XNResourceName,  "SFML",
-                                   //XNResourceClass, "SFML",
                                    NULL);
         
         if (!myInputContext)
@@ -666,7 +807,7 @@ void WindowImplX11::CreateHiddenCursor()
 ////////////////////////////////////////////////////////////
 void WindowImplX11::CleanUp()
 {
-    // Restore previous video mode (in case we were running in fullscreen)
+    // Restore the previous video mode (in case we were running in fullscreen)
     if (ourFullscreenWindow == this)
     {
         // Get current screen info
@@ -700,17 +841,6 @@ void WindowImplX11::CleanUp()
 }
 
 
-////////////////////////////////////////////////////////////
-/// Filter the received events
-/// (only allow those matching a specific window)
-////////////////////////////////////////////////////////////
-Bool WindowImplX11::CheckEvent(::Display*, XEvent* Event, XPointer UserData)
-{
-    // Just check if the event matches our window
-    return Event->xany.window == reinterpret_cast< ::Window >(UserData);
-}
-
-
 ////////////////////////////////////////////////////////////
 /// Process an incoming event from the window
 ////////////////////////////////////////////////////////////
@@ -807,12 +937,19 @@ void WindowImplX11::ProcessEvent(XEvent WinEvent)
                 {
                     Status ReturnedStatus;
                     Uint8  KeyBuffer[16];
-                    if (Xutf8LookupString(myInputContext, &WinEvent.xkey, reinterpret_cast<char*>(KeyBuffer), sizeof(KeyBuffer), NULL, &ReturnedStatus))
+                    int Length = Xutf8LookupString(myInputContext, &WinEvent.xkey, reinterpret_cast<char*>(KeyBuffer), sizeof(KeyBuffer), NULL, &ReturnedStatus);
+                    if (Length > 0)
                     {
-                        Event TextEvt;
-                        TextEvt.Type         = Event::TextEntered;
-                        TextEvt.Text.Unicode = UTF8toUTF16(KeyBuffer);
-                        SendEvent(TextEvt);
+                        Uint32 Unicode[2]; // just in case, but 1 character should be enough
+                        const Uint32* End = Unicode::UTF8ToUTF32(KeyBuffer, KeyBuffer + Length, Unicode);
+
+                        if (End > Unicode)
+                        {
+                            Event TextEvt;
+                            TextEvt.Type         = Event::TextEntered;
+                            TextEvt.Text.Unicode = Unicode[0];
+                            SendEvent(TextEvt);
+                        }
                     }
                 }
                 else
@@ -824,7 +961,7 @@ void WindowImplX11::ProcessEvent(XEvent WinEvent)
                     {
                         Event TextEvt;
                         TextEvt.Type         = Event::TextEntered;
-                        TextEvt.Text.Unicode = KeyBuffer[0];
+                        TextEvt.Text.Unicode = static_cast<Uint32>(KeyBuffer[0]);
                         SendEvent(TextEvt);
                     }
                 }
@@ -860,7 +997,9 @@ void WindowImplX11::ProcessEvent(XEvent WinEvent)
             if ((Button == Button1) || (Button == Button2) || (Button == Button3) || (Button == 8) || (Button == 9))
             {
                 Event Evt;
-                Evt.Type = Event::MouseButtonPressed;
+                Evt.Type          = Event::MouseButtonPressed;
+                Evt.MouseButton.X = WinEvent.xbutton.x;
+                Evt.MouseButton.Y = WinEvent.xbutton.y;
                 switch (Button)
                 {
                     case Button1 : Evt.MouseButton.Button = Mouse::Left;     break;
@@ -881,7 +1020,9 @@ void WindowImplX11::ProcessEvent(XEvent WinEvent)
             if ((Button == Button1) || (Button == Button2) || (Button == Button3) || (Button == 8) || (Button == 9))
             {
                 Event Evt;
-                Evt.Type = Event::MouseButtonReleased;
+                Evt.Type          = Event::MouseButtonReleased;
+                Evt.MouseButton.X = WinEvent.xbutton.x;
+                Evt.MouseButton.Y = WinEvent.xbutton.y;
                 switch (Button)
                 {
                     case Button1 : Evt.MouseButton.Button = Mouse::Left;     break;
@@ -912,6 +1053,24 @@ void WindowImplX11::ProcessEvent(XEvent WinEvent)
             SendEvent(Evt);
             break;
         }
+
+        // Mouse entered
+        case EnterNotify :
+        {
+            Event Evt;
+            Evt.Type = Event::MouseEntered;
+            SendEvent(Evt);
+            break;
+        }
+
+        // Mouse left
+        case LeaveNotify :
+        {
+            Event Evt;
+            Evt.Type = Event::MouseLeft;
+            SendEvent(Evt);
+            break;
+        }
     }
 }
 
@@ -1048,11 +1207,6 @@ bool WindowImplX11::OpenDisplay(bool AddWindow)
         {
             ourScreen = DefaultScreen(ourDisplay);
 
-            // Disable repetition of KeyRelease events when a key is held down
-            Bool Supported;
-            XkbSetDetectableAutoRepeat(ourDisplay, True, &Supported);
-            XFlush(ourDisplay);
-
             // Get the input method (XIM) object
             ourInputMethod = XOpenIM(ourDisplay, NULL, NULL, NULL);
         }
