@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -82,6 +82,14 @@ public :
     ////////////////////////////////////////////////////////////
     ~WindowImplX11();
 
+    ////////////////////////////////////////////////////////////
+    /// Check if there's an active context on the current thread
+    ///
+    /// \return True if there's a context bound to the current thread
+    ///
+    ////////////////////////////////////////////////////////////
+    static bool IsContextActive();
+
 private :
 
     friend class VideoModeSupport;
@@ -99,10 +107,10 @@ private :
     virtual void ProcessEvents();
 
     ////////////////////////////////////////////////////////////
-    /// /see WindowImpl::MakeActive
+    /// /see WindowImpl::SetActive
     ///
     ////////////////////////////////////////////////////////////
-    virtual void MakeActive(bool Active = true) const;
+    virtual void SetActive(bool Active = true) const;
 
     ////////////////////////////////////////////////////////////
     /// /see WindowImpl::UseVerticalSync
@@ -128,6 +136,12 @@ private :
     ////////////////////////////////////////////////////////////
     virtual void SetPosition(int Left, int Top);
 
+    ////////////////////////////////////////////////////////////
+    /// /see WindowImpl::SetSize
+    ///
+    ////////////////////////////////////////////////////////////
+    virtual void SetSize(unsigned int Width, unsigned int Height);
+
     ////////////////////////////////////////////////////////////
     /// /see sfWindowImpl::Show
     ///
@@ -140,6 +154,12 @@ private :
     ////////////////////////////////////////////////////////////
     virtual void EnableKeyRepeat(bool Enabled);
 
+    ////////////////////////////////////////////////////////////
+    /// /see WindowImpl::SetIcon
+    ///
+    ////////////////////////////////////////////////////////////
+    virtual void SetIcon(unsigned int Width, unsigned int Height, const Uint8* Pixels);
+
     ////////////////////////////////////////////////////////////
     /// Switch to fullscreen mode
     ///
@@ -180,18 +200,6 @@ private :
     ////////////////////////////////////////////////////////////
     void CleanUp();
 
-    ////////////////////////////////////////////////////////////
-    /// Filter the received events
-    /// (only allow those matching a specific window)
-    ///
-    /// \param Event :    Event to filter
-    /// \param UserData : Data passed to the function (here : the window to compare)
-    ///
-    /// \return True if the event belongs to the specified window
-    ///
-    ////////////////////////////////////////////////////////////
-    static Bool CheckEvent(::Display*, XEvent* Event, XPointer UserData);
-
     ////////////////////////////////////////////////////////////
     /// Process an incoming event from the window
     ///
@@ -239,13 +247,15 @@ private :
     ////////////////////////////////////////////////////////////
     // Member data
     ////////////////////////////////////////////////////////////
-    ::Window   myWindow;       ///< X11 structure defining our window
-    bool       myIsExternal;   ///< Tell whether the window has been created externally or by SFML
-    GLXContext myGLContext;    ///< OpenGL context attached to the window
-    Atom       myAtomClose;    ///< Atom used to identify the close event
-    int        myOldVideoMode; ///< Video mode in use before we switch to fullscreen
-    Cursor     myHiddenCursor; ///< As X11 doesn't provide cursor hidding, we must create a transparent one
-    XIC        myInputContext; ///< Input context used to get unicode input in our window
+    ::Window   myWindow;              ///< X11 structure defining our window
+    bool       myIsExternal;          ///< Tell whether the window has been created externally or by SFML
+    GLXContext myGLContext;           ///< OpenGL context attached to the window
+    Atom       myAtomClose;           ///< Atom used to identify the close event
+    int        myOldVideoMode;        ///< Video mode in use before we switch to fullscreen
+    Cursor     myHiddenCursor;        ///< As X11 doesn't provide cursor hidding, we must create a transparent one
+    XIC        myInputContext;        ///< Input context used to get unicode input in our window
+    bool       myKeyRepeat;           ///< Is the KeyRepeat feature enabled ?
+    XEvent     myLastKeyReleaseEvent; ///< Last key release event we received (needed for discarding repeated key events)
 };
 
 } // namespace priv
