@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -35,13 +35,13 @@
 
     #include <SFML/Window/Win32/VideoModeSupport.hpp>
 
-#elif defined(SFML_SYSTEM_LINUX)
+#elif defined(SFML_SYSTEM_LINUX) || defined(SFML_SYSTEM_FREEBSD)
 
     #include <SFML/Window/Linux/VideoModeSupport.hpp>
 
 #elif defined(SFML_SYSTEM_MACOS)
 
-    #include <SFML/Window/OSXCarbon/VideoModeSupport.hpp>
+    #include <SFML/Window/Cocoa/VideoModeSupport.hpp>
 
 #endif
 
