@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -42,13 +42,18 @@ namespace priv
 void Joystick::Initialize(unsigned int Index)
 {
     // Reset state
-    myIndex     = JOYSTICKID1;
-    myNbAxes    = 0;
-    myNbButtons = 0;
+    myIndex            = JOYSTICKID1;
+    myNbButtons        = 0;
+    myIsConnected      = false;
+    myHasContinuousPOV = false;
+    for (int i = 0; i < Joy::AxisCount; ++i)
+        myAxes[i] = false;
 
     // Get the Index-th connected joystick
     MMRESULT Error;
     JOYINFOEX JoyInfo;
+    JoyInfo.dwSize = sizeof(JoyInfo);
+    JoyInfo.dwFlags = JOY_RETURNALL;
     for (unsigned int NbFound = 0; (Error = joyGetPosEx(myIndex, &JoyInfo)) != JOYERR_PARMS; myIndex++)
     {
         // Check if the current joystick is connected
@@ -58,10 +63,21 @@ void Joystick::Initialize(unsigned int Index)
             if (NbFound == Index)
             {
                 // Ok : store its parameters and return
+                myIsConnected = true;
                 JOYCAPS Caps;
                 joyGetDevCaps(myIndex, &Caps, sizeof(Caps));
-                myNbAxes    = Caps.wNumAxes;
                 myNbButtons = Caps.wNumButtons;
+                if (myNbButtons > Joy::ButtonCount)
+                    myNbButtons = Joy::ButtonCount;
+
+                myAxes[Joy::AxisX]   = true;
+                myAxes[Joy::AxisY]   = true;
+                myAxes[Joy::AxisZ]   = (Caps.wCaps & JOYCAPS_HASZ) != 0;
+                myAxes[Joy::AxisR]   = (Caps.wCaps & JOYCAPS_HASR) != 0;
+                myAxes[Joy::AxisU]   = (Caps.wCaps & JOYCAPS_HASU) != 0;
+                myAxes[Joy::AxisV]   = (Caps.wCaps & JOYCAPS_HASV) != 0;
+                myAxes[Joy::AxisPOV] = (Caps.wCaps & JOYCAPS_HASPOV) != 0;
+                myHasContinuousPOV   = (Caps.wCaps & JOYCAPS_POVCTS) != 0;
 
                 return;
             }
@@ -78,32 +94,39 @@ void Joystick::Initialize(unsigned int Index)
 ////////////////////////////////////////////////////////////
 JoystickState Joystick::UpdateState()
 {
-    JoystickState State = {0};
+    JoystickState State;
 
-    // Get the joystick caps (for range conversions)
-    JOYCAPS Caps;
-    if (joyGetDevCaps(myIndex, &Caps, sizeof(Caps)) == JOYERR_NOERROR)
+    if (myIsConnected)
     {
-        // Get the current joystick state
-        JOYINFOEX Pos;
-        Pos.dwFlags = JOY_RETURNALL;
-        Pos.dwSize  = sizeof(JOYINFOEX);
-        if (joyGetPosEx(myIndex, &Pos) == JOYERR_NOERROR)
+        // Get the joystick caps (for range conversions)
+        JOYCAPS Caps;
+        if (joyGetDevCaps(myIndex, &Caps, sizeof(Caps)) == JOYERR_NOERROR)
         {
-            // Axes
-            State.Axis[Joy::AxisX] = (Pos.dwXpos - (Caps.wXmax + Caps.wXmin) / 2.f) * 200.f / (Caps.wXmax - Caps.wXmin);
-            State.Axis[Joy::AxisY] = (Pos.dwYpos - (Caps.wYmax + Caps.wYmin) / 2.f) * 200.f / (Caps.wYmax - Caps.wYmin);
-            State.Axis[Joy::AxisZ] = (Pos.dwZpos - (Caps.wZmax + Caps.wZmin) / 2.f) * 200.f / (Caps.wZmax - Caps.wZmin);
-            State.Axis[Joy::AxisR] = (Pos.dwRpos - (Caps.wRmax + Caps.wRmin) / 2.f) * 200.f / (Caps.wRmax - Caps.wRmin);
-            State.Axis[Joy::AxisU] = (Pos.dwUpos - (Caps.wUmax + Caps.wUmin) / 2.f) * 200.f / (Caps.wUmax - Caps.wUmin);
-            State.Axis[Joy::AxisV] = (Pos.dwVpos - (Caps.wVmax + Caps.wVmin) / 2.f) * 200.f / (Caps.wVmax - Caps.wVmin);
+            // Get the current joystick state
+            JOYINFOEX Pos;
+            Pos.dwFlags  = JOY_RETURNX | JOY_RETURNY | JOY_RETURNZ | JOY_RETURNR | JOY_RETURNU | JOY_RETURNV | JOY_RETURNBUTTONS;
+            Pos.dwFlags |= myHasContinuousPOV ? JOY_RETURNPOVCTS : JOY_RETURNPOV;
+            Pos.dwSize   = sizeof(JOYINFOEX);
+            if (joyGetPosEx(myIndex, &Pos) == JOYERR_NOERROR)
+            {
+                // Axes
+                State.Axis[Joy::AxisX] = (Pos.dwXpos - (Caps.wXmax + Caps.wXmin) / 2.f) * 200.f / (Caps.wXmax - Caps.wXmin);
+                State.Axis[Joy::AxisY] = (Pos.dwYpos - (Caps.wYmax + Caps.wYmin) / 2.f) * 200.f / (Caps.wYmax - Caps.wYmin);
+                State.Axis[Joy::AxisZ] = (Pos.dwZpos - (Caps.wZmax + Caps.wZmin) / 2.f) * 200.f / (Caps.wZmax - Caps.wZmin);
+                State.Axis[Joy::AxisR] = (Pos.dwRpos - (Caps.wRmax + Caps.wRmin) / 2.f) * 200.f / (Caps.wRmax - Caps.wRmin);
+                State.Axis[Joy::AxisU] = (Pos.dwUpos - (Caps.wUmax + Caps.wUmin) / 2.f) * 200.f / (Caps.wUmax - Caps.wUmin);
+                State.Axis[Joy::AxisV] = (Pos.dwVpos - (Caps.wVmax + Caps.wVmin) / 2.f) * 200.f / (Caps.wVmax - Caps.wVmin);
 
-            // POV
-            State.Axis[Joy::AxisPOV] = Pos.dwPOV / 100.f;
+                // POV
+                if (Pos.dwPOV != 0xFFFF)
+                    State.Axis[Joy::AxisPOV] = Pos.dwPOV / 100.f;
+                else
+                    State.Axis[Joy::AxisPOV] = -1.f;
 
-            // Buttons
-            for (unsigned int i = 0; i < GetButtonsCount(); ++i)
-                State.Buttons[i] = (Pos.dwButtons & (1 << i)) != 0;
+                // Buttons
+                for (unsigned int i = 0; i < GetButtonsCount(); ++i)
+                    State.Buttons[i] = (Pos.dwButtons & (1 << i)) != 0;
+            }
         }
     }
 
@@ -112,11 +135,11 @@ JoystickState Joystick::UpdateState()
 
 
 ////////////////////////////////////////////////////////////
-/// Get the number of axes supported by the joystick
+/// Check if the joystick supports the given axis
 ////////////////////////////////////////////////////////////
-unsigned int Joystick::GetAxesCount() const
+bool Joystick::HasAxis(Joy::Axis Axis) const
 {
-    return myNbAxes;
+    return myAxes[Axis];
 }
 
 
