@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -34,14 +34,18 @@
 #include <SFML/Window/glext/wglext.h>
 #include <SFML/Window/glext/glext.h>
 #include <iostream>
+#include <vector>
 
-// Old versions of MinGW lack the definition of XBUTTON1 and XBUTTON2
+// MinGW lacks the definition of some Win32 constants
 #ifndef XBUTTON1
     #define XBUTTON1 0x0001
 #endif
 #ifndef XBUTTON2
     #define XBUTTON2 0x0002
 #endif
+#ifndef MAPVK_VK_TO_VSC
+    #define MAPVK_VK_TO_VSC (0)
+#endif
 
 
 namespace sf
@@ -65,7 +69,9 @@ WindowImplWin32::WindowImplWin32() :
 myHandle          (NULL),
 myCallback        (0),
 myCursor          (NULL),
-myKeyRepeatEnabled(true)
+myIcon            (NULL),
+myKeyRepeatEnabled(true),
+myIsCursorIn      (false)
 {
     // Register the window class at first call
     if (ourWindowCount == 0)
@@ -91,6 +97,9 @@ myKeyRepeatEnabled(true)
     {
         WindowSettings Params(0, 0, 0);
         CreateContext(VideoMode(myWidth, myHeight, 32), Params);
+
+        // Don't activate by default
+        SetActive(false);
     }
 }
 
@@ -102,7 +111,9 @@ WindowImplWin32::WindowImplWin32(WindowHandle Handle, WindowSettings& Params) :
 myHandle          (NULL),
 myCallback        (0),
 myCursor          (NULL),
-myKeyRepeatEnabled(true)
+myIcon            (NULL),
+myKeyRepeatEnabled(true),
+myIsCursorIn      (false)
 {
     // Save window handle
     myHandle = static_cast<HWND>(Handle);
@@ -133,17 +144,21 @@ WindowImplWin32::WindowImplWin32(VideoMode Mode, const std::string& Title, unsig
 myHandle          (NULL),
 myCallback        (0),
 myCursor          (NULL),
-myKeyRepeatEnabled(true)
+myIcon            (NULL),
+myKeyRepeatEnabled(true),
+myIsCursorIn      (false)
 {
     // Register the window class at first call
     if (ourWindowCount == 0)
         RegisterWindowClass();
 
     // Compute position and size
-    int Left   = (GetDeviceCaps(GetDC(NULL), HORZRES) - Mode.Width)  / 2;
-    int Top    = (GetDeviceCaps(GetDC(NULL), VERTRES) - Mode.Height) / 2;
+    HDC ScreenDC = GetDC(NULL);
+    int Left   = (GetDeviceCaps(ScreenDC, HORZRES) - Mode.Width)  / 2;
+    int Top    = (GetDeviceCaps(ScreenDC, VERTRES) - Mode.Height) / 2;
     int Width  = myWidth  = Mode.Width;
     int Height = myHeight = Mode.Height;
+    ReleaseDC(NULL, ScreenDC);
 
     // Choose the window style according to the Style parameter
     DWORD Win32Style = WS_VISIBLE;
@@ -191,6 +206,13 @@ myKeyRepeatEnabled(true)
 
     // Increment window count
     ourWindowCount++;
+
+    // Get the actual size of the window, which can be smaller even after the call to AdjustWindowRect
+    // This happens when the window is bigger than the desktop
+    RECT ActualRect;
+    GetClientRect(myHandle, &ActualRect);
+    myWidth  = ActualRect.right - ActualRect.left;
+    myHeight = ActualRect.bottom - ActualRect.top;
 }
 
 
@@ -199,6 +221,10 @@ myKeyRepeatEnabled(true)
 ////////////////////////////////////////////////////////////
 WindowImplWin32::~WindowImplWin32()
 {
+    // Destroy the custom icon, if any
+    if (myIcon)
+        DestroyIcon(myIcon);
+
     if (!myCallback)
     {
         // Destroy the window
@@ -230,7 +256,16 @@ WindowImplWin32::~WindowImplWin32()
 
 
 ////////////////////////////////////////////////////////////
-/// /see sfWindowImpl::ProcessEvents
+/// Check if there's an active context on the current thread
+////////////////////////////////////////////////////////////
+bool WindowImplWin32::IsContextActive()
+{
+    return wglGetCurrentContext() != NULL;
+}
+
+
+////////////////////////////////////////////////////////////
+/// /see WindowImpl::ProcessEvents
 ////////////////////////////////////////////////////////////
 void WindowImplWin32::ProcessEvents()
 {
@@ -238,7 +273,7 @@ void WindowImplWin32::ProcessEvents()
     if (!myCallback)
     {
         MSG Message;
-        while (PeekMessage(&Message, myHandle, 0, 0, PM_REMOVE))
+        while (PeekMessage(&Message, NULL, 0, 0, PM_REMOVE))
         {
             TranslateMessage(&Message);
             DispatchMessage(&Message);
@@ -248,7 +283,7 @@ void WindowImplWin32::ProcessEvents()
 
 
 ////////////////////////////////////////////////////////////
-/// /see sfWindowImpl::Display
+/// /see WindowImpl::Display
 ////////////////////////////////////////////////////////////
 void WindowImplWin32::Display()
 {
@@ -258,24 +293,25 @@ void WindowImplWin32::Display()
 
 
 ////////////////////////////////////////////////////////////
-/// /see sfWindowImpl::MakeActive
+/// /see WindowImpl::SetActive
 ////////////////////////////////////////////////////////////
-void WindowImplWin32::MakeActive(bool Active) const
+void WindowImplWin32::SetActive(bool Active) const
 {
     if (Active)
     {
-        if (myDeviceContext && myGLContext)
+        if (myDeviceContext && myGLContext && (wglGetCurrentContext() != myGLContext))
             wglMakeCurrent(myDeviceContext, myGLContext);
     }
     else
     {
-        wglMakeCurrent(NULL, NULL);
+        if (wglGetCurrentContext() == myGLContext)
+            wglMakeCurrent(NULL, NULL);
     }
 }
 
 
 ////////////////////////////////////////////////////////////
-/// /see sfWindowImpl::UseVerticalSync
+/// /see WindowImpl::UseVerticalSync
 ////////////////////////////////////////////////////////////
 void WindowImplWin32::UseVerticalSync(bool Enabled)
 {
@@ -286,7 +322,7 @@ void WindowImplWin32::UseVerticalSync(bool Enabled)
 
 
 ////////////////////////////////////////////////////////////
-/// /see sfWindowImpl::ShowMouseCursor
+/// /see WindowImpl::ShowMouseCursor
 ////////////////////////////////////////////////////////////
 void WindowImplWin32::ShowMouseCursor(bool Show)
 {
@@ -300,7 +336,7 @@ void WindowImplWin32::ShowMouseCursor(bool Show)
 
 
 ////////////////////////////////////////////////////////////
-/// /see sfWindowImpl::SetCursorPosition
+/// /see WindowImpl::SetCursorPosition
 ////////////////////////////////////////////////////////////
 void WindowImplWin32::SetCursorPosition(unsigned int Left, unsigned int Top)
 {
@@ -311,7 +347,7 @@ void WindowImplWin32::SetCursorPosition(unsigned int Left, unsigned int Top)
 
 
 ////////////////////////////////////////////////////////////
-/// /see sfWindowImpl::SetPosition
+/// /see WindowImpl::SetPosition
 ////////////////////////////////////////////////////////////
 void WindowImplWin32::SetPosition(int Left, int Top)
 {
@@ -320,7 +356,23 @@ void WindowImplWin32::SetPosition(int Left, int Top)
 
 
 ////////////////////////////////////////////////////////////
-/// /see sfWindowImpl::Show
+/// /see WindowImpl::SetSize
+////////////////////////////////////////////////////////////
+void WindowImplWin32::SetSize(unsigned int Width, unsigned int Height)
+{
+    // SetWindowPos wants the total size of the window (including title bar and borders),
+    // so we have to compute it
+    RECT Rect = {0, 0, Width, Height};
+    AdjustWindowRect(&Rect, GetWindowLong(myHandle, GWL_STYLE), false);
+    Width  = Rect.right - Rect.left;
+    Height = Rect.bottom - Rect.top;
+
+    SetWindowPos(myHandle, NULL, 0, 0, Width, Height, SWP_NOMOVE | SWP_NOZORDER);
+}
+
+
+////////////////////////////////////////////////////////////
+/// /see WindowImpl::Show
 ////////////////////////////////////////////////////////////
 void WindowImplWin32::Show(bool State)
 {
@@ -329,7 +381,7 @@ void WindowImplWin32::Show(bool State)
 
 
 ////////////////////////////////////////////////////////////
-/// /see sfWindowImpl::EnableKeyRepeat
+/// /see WindowImpl::EnableKeyRepeat
 ////////////////////////////////////////////////////////////
 void WindowImplWin32::EnableKeyRepeat(bool Enabled)
 {
@@ -337,6 +389,41 @@ void WindowImplWin32::EnableKeyRepeat(bool Enabled)
 }
 
 
+////////////////////////////////////////////////////////////
+/// /see WindowImpl::SetIcon
+////////////////////////////////////////////////////////////
+void WindowImplWin32::SetIcon(unsigned int Width, unsigned int Height, const Uint8* Pixels)
+{
+    // First destroy the previous one
+    if (myIcon)
+        DestroyIcon(myIcon);
+
+    // Windows wants BGRA pixels : swap red and blue channels
+    std::vector<Uint8> IconPixels(Width * Height * 4);
+    for (std::size_t i = 0; i < IconPixels.size() / 4; ++i)
+    {
+        IconPixels[i * 4 + 0] = Pixels[i * 4 + 2];
+        IconPixels[i * 4 + 1] = Pixels[i * 4 + 1];
+        IconPixels[i * 4 + 2] = Pixels[i * 4 + 0];
+        IconPixels[i * 4 + 3] = Pixels[i * 4 + 3];
+    }
+
+    // Create the icon from the pixels array
+    myIcon = CreateIcon(GetModuleHandle(NULL), Width, Height, 1, 32, NULL, &IconPixels[0]);
+
+    // Set it as both big and small icon of the window
+    if (myIcon)
+    {
+        SendMessage(myHandle, WM_SETICON, ICON_BIG,   (LPARAM)myIcon);
+        SendMessage(myHandle, WM_SETICON, ICON_SMALL, (LPARAM)myIcon);
+    }
+    else
+    {
+        std::cerr << "Failed to set the window's icon" << std::endl;
+    }
+}
+
+
 ////////////////////////////////////////////////////////////
 /// Register the window class
 ////////////////////////////////////////////////////////////
@@ -394,20 +481,16 @@ void WindowImplWin32::SwitchToFullscreen(const VideoMode& Mode)
         return;
     }
 
-    // Change window style (no border, no titlebar, ...)
-    SetWindowLong(myHandle, GWL_STYLE,   WS_POPUP);
+    // Make the window flags compatible with fullscreen mode
+    SetWindowLong(myHandle, GWL_STYLE, WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS);
     SetWindowLong(myHandle, GWL_EXSTYLE, WS_EX_APPWINDOW);
 
-    // And resize it so that it fits the entire screen
+    // Resize the window so that it fits the entire screen
     SetWindowPos(myHandle, HWND_TOP, 0, 0, Mode.Width, Mode.Height, SWP_FRAMECHANGED);
     ShowWindow(myHandle, SW_SHOW);
 
     // Set "this" as the current fullscreen window
     ourFullscreenWindow = this;
-
-    // SetPixelFormat can fail (really ?) if window style doesn't contain these flags
-    long Style = GetWindowLong(myHandle, GWL_STYLE);
-    SetWindowLong(myHandle, GWL_STYLE, Style | WS_CLIPCHILDREN | WS_CLIPSIBLINGS);
 }
 
 
@@ -430,67 +513,75 @@ void WindowImplWin32::CreateContext(const VideoMode& Mode, WindowSettings& Param
     {
         // Get the wglChoosePixelFormatARB function (it is an extension)
         PFNWGLCHOOSEPIXELFORMATARBPROC wglChoosePixelFormatARB = reinterpret_cast<PFNWGLCHOOSEPIXELFORMATARBPROC>(wglGetProcAddress("wglChoosePixelFormatARB"));
-
-        // Define the basic attributes we want for our window
-        int IntAttributes[] =
+        if (wglChoosePixelFormatARB)
         {
-            WGL_DRAW_TO_WINDOW_ARB, GL_TRUE,
-		    WGL_SUPPORT_OPENGL_ARB, GL_TRUE,
-		    WGL_ACCELERATION_ARB,   WGL_FULL_ACCELERATION_ARB,
-		    WGL_DOUBLE_BUFFER_ARB,  GL_TRUE,
-            WGL_SAMPLE_BUFFERS_ARB, (Params.AntialiasingLevel ? GL_TRUE : GL_FALSE),
-		    WGL_SAMPLES_ARB,        Params.AntialiasingLevel,
-		    0,                      0
-        };
-
-        // Let's check how many formats are supporting our requirements
-        int   Formats[128];
-	    UINT  NbFormats;
-	    float FloatAttributes[] = {0, 0};
-	    bool  IsValid = wglChoosePixelFormatARB(myDeviceContext, IntAttributes, FloatAttributes, sizeof(Formats) / sizeof(*Formats), Formats, &NbFormats) != 0;
-        if (!IsValid || (NbFormats == 0))
-        {
-            if (Params.AntialiasingLevel > 2)
+            // Define the basic attributes we want for our window
+            int IntAttributes[] =
             {
-                // No format matching our needs : reduce the multisampling level
-                std::cerr << "Failed to find a pixel format supporting "
-                          << Params.AntialiasingLevel << " antialiasing levels ; trying with 2 levels" << std::endl;
-
-                Params.AntialiasingLevel = IntAttributes[1] = 2;
-	            IsValid = wglChoosePixelFormatARB(myDeviceContext, IntAttributes, FloatAttributes, sizeof(Formats) / sizeof(*Formats), Formats, &NbFormats) != 0;
-            }
+                WGL_DRAW_TO_WINDOW_ARB, GL_TRUE,
+		        WGL_SUPPORT_OPENGL_ARB, GL_TRUE,
+		        WGL_ACCELERATION_ARB,   WGL_FULL_ACCELERATION_ARB,
+		        WGL_DOUBLE_BUFFER_ARB,  GL_TRUE,
+                WGL_SAMPLE_BUFFERS_ARB, (Params.AntialiasingLevel ? GL_TRUE : GL_FALSE),
+		        WGL_SAMPLES_ARB,        Params.AntialiasingLevel,
+		        0,                      0
+            };
 
+            // Let's check how many formats are supporting our requirements
+            int   Formats[128];
+	        UINT  NbFormats;
+	        float FloatAttributes[] = {0, 0};
+	        bool  IsValid = wglChoosePixelFormatARB(myDeviceContext, IntAttributes, FloatAttributes, sizeof(Formats) / sizeof(*Formats), Formats, &NbFormats) != 0;
             if (!IsValid || (NbFormats == 0))
             {
-                // Cannot find any pixel format supporting multisampling ; disabling antialiasing
-                std::cerr << "Failed to find a pixel format supporting antialiasing ; antialiasing will be disabled" << std::endl;
-                Params.AntialiasingLevel = 0;
-            }
-        }
-
-        // Get the best format among the returned ones
-        if (IsValid && (NbFormats > 0))
-        {
-            int BestScore = 0xFFFF;
-            for (UINT i = 0; i < NbFormats; ++i)
-            {
-                // Get the current format's attributes
-                PIXELFORMATDESCRIPTOR Attribs;
-                Attribs.nSize    = sizeof(PIXELFORMATDESCRIPTOR);
-                Attribs.nVersion = 1;
-                DescribePixelFormat(myDeviceContext, Formats[i], sizeof(PIXELFORMATDESCRIPTOR), &Attribs);
-
-                // Evaluate the current configuration
-                int Color = Attribs.cRedBits + Attribs.cGreenBits + Attribs.cBlueBits + Attribs.cAlphaBits;
-                int Score = EvaluateConfig(Mode, Params, Color, Attribs.cDepthBits, Attribs.cStencilBits, Params.AntialiasingLevel);
-
-                // Keep it if it's better than the current best
-                if (Score < BestScore)
+                if (Params.AntialiasingLevel > 2)
                 {
-                    BestScore  = Score;
-                    BestFormat = Formats[i];
+                    // No format matching our needs : reduce the multisampling level
+                    std::cerr << "Failed to find a pixel format supporting "
+                              << Params.AntialiasingLevel << " antialiasing levels ; trying with 2 levels" << std::endl;
+
+                    Params.AntialiasingLevel = IntAttributes[11] = 2;
+	                IsValid = wglChoosePixelFormatARB(myDeviceContext, IntAttributes, FloatAttributes, sizeof(Formats) / sizeof(*Formats), Formats, &NbFormats) != 0;
+                }
+
+                if (!IsValid || (NbFormats == 0))
+                {
+                    // Cannot find any pixel format supporting multisampling ; disabling antialiasing
+                    std::cerr << "Failed to find a pixel format supporting antialiasing ; antialiasing will be disabled" << std::endl;
+                    Params.AntialiasingLevel = 0;
                 }
             }
+
+            // Get the best format among the returned ones
+            if (IsValid && (NbFormats > 0))
+            {
+                int BestScore = 0xFFFF;
+                for (UINT i = 0; i < NbFormats; ++i)
+                {
+                    // Get the current format's attributes
+                    PIXELFORMATDESCRIPTOR Attribs;
+                    Attribs.nSize    = sizeof(PIXELFORMATDESCRIPTOR);
+                    Attribs.nVersion = 1;
+                    DescribePixelFormat(myDeviceContext, Formats[i], sizeof(PIXELFORMATDESCRIPTOR), &Attribs);
+
+                    // Evaluate the current configuration
+                    int Color = Attribs.cRedBits + Attribs.cGreenBits + Attribs.cBlueBits + Attribs.cAlphaBits;
+                    int Score = EvaluateConfig(Mode, Params, Color, Attribs.cDepthBits, Attribs.cStencilBits, Params.AntialiasingLevel);
+
+                    // Keep it if it's better than the current best
+                    if (Score < BestScore)
+                    {
+                        BestScore  = Score;
+                        BestFormat = Formats[i];
+                    }
+                }
+            }
+        }
+        else
+        {
+            // wglChoosePixelFormatARB not supported ; disabling antialiasing
+            std::cerr << "Antialiasing is not supported ; it will be disabled" << std::endl;
+            Params.AntialiasingLevel = 0;
         }
     }
 
@@ -508,6 +599,7 @@ void WindowImplWin32::CreateContext(const VideoMode& Mode, WindowSettings& Param
         PixelDescriptor.cColorBits   = static_cast<BYTE>(Mode.BitsPerPixel);
         PixelDescriptor.cDepthBits   = static_cast<BYTE>(Params.DepthBits);
         PixelDescriptor.cStencilBits = static_cast<BYTE>(Params.StencilBits);
+        PixelDescriptor.cAlphaBits   = Mode.BitsPerPixel == 32 ? 8 : 0;
 
         // Get the pixel format that best matches our requirements
         BestFormat = ChoosePixelFormat(myDeviceContext, &PixelDescriptor);
@@ -542,11 +634,12 @@ void WindowImplWin32::CreateContext(const VideoMode& Mode, WindowSettings& Param
     }
 
     // Share display lists with other contexts
-    if (wglGetCurrentContext())
-        wglShareLists(wglGetCurrentContext(), myGLContext);
+    HGLRC CurrentContext = wglGetCurrentContext();
+    if (CurrentContext)
+        wglShareLists(CurrentContext, myGLContext);
 
     // Activate the context
-    MakeActive(true);
+    SetActive(true);
 
     // Enable multisampling
     if (Params.AntialiasingLevel > 0)
@@ -572,6 +665,9 @@ void WindowImplWin32::Cleanup()
     // Destroy the OpenGL context
     if (myGLContext)
     {
+        // Unbind the context before destroying it
+        SetActive(false);
+
         wglDeleteContext(myGLContext);
         myGLContext = NULL;
     }
@@ -659,10 +755,13 @@ void WindowImplWin32::ProcessEvent(UINT Message, WPARAM WParam, LPARAM LParam)
         // Text event
         case WM_CHAR :
         {
-            Event Evt;
-            Evt.Type = Event::TextEntered;
-            Evt.Text.Unicode = static_cast<Uint16>(WParam);
-            SendEvent(Evt);
+            if (myKeyRepeatEnabled || ((LParam & (1 << 30)) == 0))
+            {
+                Event Evt;
+                Evt.Type = Event::TextEntered;
+                Evt.Text.Unicode = static_cast<Uint32>(WParam);
+                SendEvent(Evt);
+            }
             break;
         }
 
@@ -670,14 +769,14 @@ void WindowImplWin32::ProcessEvent(UINT Message, WPARAM WParam, LPARAM LParam)
         case WM_KEYDOWN :
         case WM_SYSKEYDOWN :
         {
-            if (myKeyRepeatEnabled || ((LParam & (1 << 30)) == 0))
+            if (myKeyRepeatEnabled || ((HIWORD(LParam) & KF_REPEAT) == 0))
             {
                 Event Evt;
                 Evt.Type        = Event::KeyPressed;
-                Evt.Key.Code    = (WParam == VK_SHIFT) ? GetShiftState(true) : VirtualKeyCodeToSF(WParam, LParam);
                 Evt.Key.Alt     = HIWORD(GetAsyncKeyState(VK_MENU))    != 0;
                 Evt.Key.Control = HIWORD(GetAsyncKeyState(VK_CONTROL)) != 0;
                 Evt.Key.Shift   = HIWORD(GetAsyncKeyState(VK_SHIFT))   != 0;
+                Evt.Key.Code    = VirtualKeyCodeToSF(WParam, LParam);
                 SendEvent(Evt);
             }
             break;
@@ -689,11 +788,12 @@ void WindowImplWin32::ProcessEvent(UINT Message, WPARAM WParam, LPARAM LParam)
         {
             Event Evt;
             Evt.Type        = Event::KeyReleased;
-            Evt.Key.Code    = (WParam == VK_SHIFT) ? GetShiftState(false) : VirtualKeyCodeToSF(WParam, LParam);
             Evt.Key.Alt     = HIWORD(GetAsyncKeyState(VK_MENU))    != 0;
             Evt.Key.Control = HIWORD(GetAsyncKeyState(VK_CONTROL)) != 0;
             Evt.Key.Shift   = HIWORD(GetAsyncKeyState(VK_SHIFT))   != 0;
+            Evt.Key.Code    = VirtualKeyCodeToSF(WParam, LParam);
             SendEvent(Evt);
+
             break;
         }
 
@@ -713,6 +813,8 @@ void WindowImplWin32::ProcessEvent(UINT Message, WPARAM WParam, LPARAM LParam)
             Event Evt;
             Evt.Type               = Event::MouseButtonPressed;
             Evt.MouseButton.Button = Mouse::Left;
+            Evt.MouseButton.X      = LOWORD(LParam);
+            Evt.MouseButton.Y      = HIWORD(LParam);
             SendEvent(Evt);
             break;
         }
@@ -723,6 +825,8 @@ void WindowImplWin32::ProcessEvent(UINT Message, WPARAM WParam, LPARAM LParam)
             Event Evt;
             Evt.Type               = Event::MouseButtonReleased;
             Evt.MouseButton.Button = Mouse::Left;
+            Evt.MouseButton.X      = LOWORD(LParam);
+            Evt.MouseButton.Y      = HIWORD(LParam);
             SendEvent(Evt);
             break;
         }
@@ -733,6 +837,8 @@ void WindowImplWin32::ProcessEvent(UINT Message, WPARAM WParam, LPARAM LParam)
             Event Evt;
             Evt.Type               = Event::MouseButtonPressed;
             Evt.MouseButton.Button = Mouse::Right;
+            Evt.MouseButton.X      = LOWORD(LParam);
+            Evt.MouseButton.Y      = HIWORD(LParam);
             SendEvent(Evt);
             break;
         }
@@ -743,6 +849,8 @@ void WindowImplWin32::ProcessEvent(UINT Message, WPARAM WParam, LPARAM LParam)
             Event Evt;
             Evt.Type               = Event::MouseButtonReleased;
             Evt.MouseButton.Button = Mouse::Right;
+            Evt.MouseButton.X      = LOWORD(LParam);
+            Evt.MouseButton.Y      = HIWORD(LParam);
             SendEvent(Evt);
             break;
         }
@@ -753,6 +861,8 @@ void WindowImplWin32::ProcessEvent(UINT Message, WPARAM WParam, LPARAM LParam)
             Event Evt;
             Evt.Type               = Event::MouseButtonPressed;
             Evt.MouseButton.Button = Mouse::Middle;
+            Evt.MouseButton.X      = LOWORD(LParam);
+            Evt.MouseButton.Y      = HIWORD(LParam);
             SendEvent(Evt);
             break;
         }
@@ -763,6 +873,8 @@ void WindowImplWin32::ProcessEvent(UINT Message, WPARAM WParam, LPARAM LParam)
             Event Evt;
             Evt.Type               = Event::MouseButtonReleased;
             Evt.MouseButton.Button = Mouse::Middle;
+            Evt.MouseButton.X      = LOWORD(LParam);
+            Evt.MouseButton.Y      = HIWORD(LParam);
             SendEvent(Evt);
             break;
         }
@@ -773,6 +885,8 @@ void WindowImplWin32::ProcessEvent(UINT Message, WPARAM WParam, LPARAM LParam)
             Event Evt;
             Evt.Type               = Event::MouseButtonPressed;
             Evt.MouseButton.Button = HIWORD(WParam) == XBUTTON1 ? Mouse::XButton1 : Mouse::XButton2;
+            Evt.MouseButton.X      = LOWORD(LParam);
+            Evt.MouseButton.Y      = HIWORD(LParam);
             SendEvent(Evt);
             break;
         }
@@ -783,6 +897,8 @@ void WindowImplWin32::ProcessEvent(UINT Message, WPARAM WParam, LPARAM LParam)
             Event Evt;
             Evt.Type               = Event::MouseButtonReleased;
             Evt.MouseButton.Button = HIWORD(WParam) == XBUTTON1 ? Mouse::XButton1 : Mouse::XButton2;
+            Evt.MouseButton.X      = LOWORD(LParam);
+            Evt.MouseButton.Y      = HIWORD(LParam);
             SendEvent(Evt);
             break;
         }
@@ -790,6 +906,22 @@ void WindowImplWin32::ProcessEvent(UINT Message, WPARAM WParam, LPARAM LParam)
         // Mouse move event
         case WM_MOUSEMOVE :
         {
+            // Check if we need to generate a MouseEntered event
+            if (!myIsCursorIn)
+            {
+                TRACKMOUSEEVENT MouseEvent;
+                MouseEvent.cbSize    = sizeof(TRACKMOUSEEVENT);
+                MouseEvent.hwndTrack = myHandle;
+                MouseEvent.dwFlags   = TME_LEAVE;
+                TrackMouseEvent(&MouseEvent);
+
+                myIsCursorIn = true;
+
+                Event Evt;
+                Evt.Type = Event::MouseEntered;
+                SendEvent(Evt);
+            }
+
             Event Evt;
             Evt.Type        = Event::MouseMoved;
             Evt.MouseMove.X = LOWORD(LParam);
@@ -797,41 +929,21 @@ void WindowImplWin32::ProcessEvent(UINT Message, WPARAM WParam, LPARAM LParam)
             SendEvent(Evt);
             break;
         }
+
+        // Mouse leave event
+        case WM_MOUSELEAVE :
+        {
+            myIsCursorIn = false;
+
+            Event Evt;
+            Evt.Type = Event::MouseLeft;
+            SendEvent(Evt);
+            break;
+        }
     }
 }
 
 
-////////////////////////////////////////////////////////////
-/// Check the state of the shift keys on a key event,
-/// and return the corresponding SF key code
-////////////////////////////////////////////////////////////
-Key::Code WindowImplWin32::GetShiftState(bool KeyDown)
-{
-    static bool LShiftPrevDown = false;
-    static bool RShiftPrevDown = false;
-
-    bool LShiftDown = (HIWORD(GetAsyncKeyState(VK_LSHIFT)) != 0);
-    bool RShiftDown = (HIWORD(GetAsyncKeyState(VK_RSHIFT)) != 0);
-
-    Key::Code Code = Key::Code(0);
-    if (KeyDown)
-    {
-        if      (!LShiftPrevDown && LShiftDown) Code = Key::LShift;
-        else if (!RShiftPrevDown && RShiftDown) Code = Key::RShift;
-    }
-    else
-    {
-        if      (LShiftPrevDown && !LShiftDown) Code = Key::LShift;
-        else if (RShiftPrevDown && !RShiftDown) Code = Key::RShift;
-    }
-
-    LShiftPrevDown = LShiftDown;
-    RShiftPrevDown = RShiftDown;
-
-    return Code;
-}
-
-
 ////////////////////////////////////////////////////////////
 /// Convert a Win32 virtual key code to a SFML key code
 ////////////////////////////////////////////////////////////
@@ -839,9 +951,21 @@ Key::Code WindowImplWin32::VirtualKeyCodeToSF(WPARAM VirtualKey, LPARAM Flags)
 {
     switch (VirtualKey)
     {
-        // VK_SHIFT is handled by the GetShiftState function
-        case VK_MENU :       return (Flags & (1 << 24)) ? Key::RAlt     : Key::LAlt;
-        case VK_CONTROL :    return (Flags & (1 << 24)) ? Key::RControl : Key::LControl;
+        // Check the scancode to distinguish between left and right shift
+        case VK_SHIFT :
+        {
+            static UINT LShift = MapVirtualKey(VK_LSHIFT, MAPVK_VK_TO_VSC);
+            UINT scancode = (Flags & (0xFF << 16)) >> 16;
+            return scancode == LShift ? Key::LShift : Key::RShift;
+        }
+
+        // Check the "extended" flag to distinguish between left and right alt
+        case VK_MENU : return (HIWORD(Flags) & KF_EXTENDED) ? Key::RAlt : Key::LAlt;
+
+        // Check the "extended" flag to distinguish between left and right control
+        case VK_CONTROL : return (HIWORD(Flags) & KF_EXTENDED) ? Key::RControl : Key::LControl;
+
+        // Other keys are reported properly
         case VK_LWIN :       return Key::LSystem;
         case VK_RWIN :       return Key::RSystem;
         case VK_APPS :       return Key::Menu;
@@ -973,7 +1097,7 @@ LRESULT CALLBACK WindowImplWin32::GlobalOnEvent(HWND Handle, UINT Message, WPARA
     // Associate handle and Window instance when the creation message is received
     if (Message == WM_CREATE)
     {
-        // Get sfWindowImplWin32 instance (it was passed as the last argument of CreateWindow)
+        // Get WindowImplWin32 instance (it was passed as the last argument of CreateWindow)
         long This = reinterpret_cast<long>(reinterpret_cast<CREATESTRUCT*>(LParam)->lpCreateParams);
 
         // Set as the "user data" parameter of the window
@@ -992,7 +1116,7 @@ LRESULT CALLBACK WindowImplWin32::GlobalOnEvent(HWND Handle, UINT Message, WPARA
             return CallWindowProc(reinterpret_cast<WNDPROC>(Window->myCallback), Handle, Message, WParam, LParam);
     }
 
-    // We don't forward the WM_CLOSE message to prevent the OS to automatically destroy the window
+    // We don't forward the WM_CLOSE message to prevent the OS from automatically destroying the window
     if (Message == WM_CLOSE)
         return 0;
 
