@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -78,62 +78,82 @@ public :
     ////////////////////////////////////////////////////////////
     ~WindowImplWin32();
 
+    ////////////////////////////////////////////////////////////
+    /// Check if there's an active context on the current thread
+    ///
+    /// \return True if there's a context bound to the current thread
+    ///
+    ////////////////////////////////////////////////////////////
+    static bool IsContextActive();
+
 private :
 
     ////////////////////////////////////////////////////////////
-    /// /see sfWindowImpl::ProcessEvents
+    /// /see WindowImpl::ProcessEvents
     ///
     ////////////////////////////////////////////////////////////
     virtual void ProcessEvents();
 
     ////////////////////////////////////////////////////////////
-    /// /see sfWindowImpl::Display
+    /// /see WindowImpl::Display
     ///
     ////////////////////////////////////////////////////////////
     virtual void Display();
 
     ////////////////////////////////////////////////////////////
-    /// /see sfWindowImpl::MakeActive
+    /// /see WindowImpl::SetActive
     ///
     ////////////////////////////////////////////////////////////
-    virtual void MakeActive(bool Active = true) const;
+    virtual void SetActive(bool Active = true) const;
 
     ////////////////////////////////////////////////////////////
-    /// /see sfWindowImpl::UseVerticalSync
+    /// /see WindowImpl::UseVerticalSync
     ///
     ////////////////////////////////////////////////////////////
     virtual void UseVerticalSync(bool Enabled);
 
     ////////////////////////////////////////////////////////////
-    /// /see sfWindowImpl::ShowMouseCursor
+    /// /see WindowImpl::ShowMouseCursor
     ///
     ////////////////////////////////////////////////////////////
     virtual void ShowMouseCursor(bool Show);
 
     ////////////////////////////////////////////////////////////
-    /// /see sfWindowImpl::SetCursorPosition
+    /// /see WindowImpl::SetCursorPosition
     ///
     ////////////////////////////////////////////////////////////
     virtual void SetCursorPosition(unsigned int Left, unsigned int Top);
 
     ////////////////////////////////////////////////////////////
-    /// /see sfWindowImpl::SetPosition
+    /// /see WindowImpl::SetPosition
     ///
     ////////////////////////////////////////////////////////////
     virtual void SetPosition(int Left, int Top);
 
     ////////////////////////////////////////////////////////////
-    /// /see sfWindowImpl::Show
+    /// /see WindowImpl::SetSize
+    ///
+    ////////////////////////////////////////////////////////////
+    virtual void SetSize(unsigned int Width, unsigned int Height);
+
+    ////////////////////////////////////////////////////////////
+    /// /see WindowImpl::Show
     ///
     ////////////////////////////////////////////////////////////
     virtual void Show(bool State);
 
     ////////////////////////////////////////////////////////////
-    /// /see sfWindowImpl::EnableKeyRepeat
+    /// /see WindowImpl::EnableKeyRepeat
     ///
     ////////////////////////////////////////////////////////////
     virtual void EnableKeyRepeat(bool Enabled);
 
+    ////////////////////////////////////////////////////////////
+    /// /see WindowImpl::SetIcon
+    ///
+    ////////////////////////////////////////////////////////////
+    virtual void SetIcon(unsigned int Width, unsigned int Height, const Uint8* Pixels);
+
     ////////////////////////////////////////////////////////////
     /// Register the window class
     ///
@@ -173,17 +193,6 @@ private :
     ////////////////////////////////////////////////////////////
     void ProcessEvent(UINT Message, WPARAM WParam, LPARAM LParam);
 
-    ////////////////////////////////////////////////////////////
-    /// Check the state of the shift keys on a key event,
-    /// and return the corresponding SF key code
-    ///
-    /// \param KeyDown : True for a keydown event, false for a keyup event
-    ///
-    /// \return SFML key code corresponding to the shift key
-    ///
-    ////////////////////////////////////////////////////////////
-    static Key::Code GetShiftState(bool KeyDown);
-
     ////////////////////////////////////////////////////////////
     /// Convert a Win32 virtual key code to a SFML key code
     ///
@@ -232,7 +241,9 @@ private :
     HWND          myHandle;           ///< Win32 handle of the window
     long          myCallback;         ///< Stores the original event callback function of the control
     HCURSOR       myCursor;           ///< The system cursor to display into the window
+    HICON         myIcon;             ///< Custom icon assigned to the window
     bool          myKeyRepeatEnabled; ///< Automatic key-repeat state for keydown events
+    bool          myIsCursorIn;       ///< Is the mouse cursor in the window's area ?
     HDC           myDeviceContext;    ///< HDC associated to the window
     HGLRC         myGLContext;        ///< OpenGL rendering context associated to the HDC
 };
