@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -26,30 +26,33 @@
 // Headers
 ////////////////////////////////////////////////////////////
 #include <SFML/Window/Window.hpp>
+#include <SFML/Window/Context.hpp>
 #include <SFML/Window/WindowImpl.hpp>
 #include <SFML/System/Sleep.hpp>
 #include <iostream>
 
 
+////////////////////////////////////////////////////////////
+// Private data
+////////////////////////////////////////////////////////////
+namespace
+{
+    const sf::Window* FullscreenWindow = NULL;
+}
+
+
 namespace sf
 {
-////////////////////////////////////////////////////////////
-// Static member data
-////////////////////////////////////////////////////////////
-priv::WindowImpl* Window::ourDummyWindow = NULL;
-
-
 ////////////////////////////////////////////////////////////
 /// Default constructor
-///
 ////////////////////////////////////////////////////////////
 Window::Window() :
 myWindow        (NULL),
 myLastFrameTime (0.f),
 myIsExternal    (false),
 myFramerateLimit(0),
-mySetCursorPosX (-1),
-mySetCursorPosY (-1)
+mySetCursorPosX (0xFFFF),
+mySetCursorPosY (0xFFFF)
 {
 
 }
@@ -63,8 +66,8 @@ myWindow        (NULL),
 myLastFrameTime (0.f),
 myIsExternal    (false),
 myFramerateLimit(0),
-mySetCursorPosX (-1),
-mySetCursorPosY (-1)
+mySetCursorPosX (0xFFFF),
+mySetCursorPosY (0xFFFF)
 {
     Create(Mode, Title, WindowStyle, Params);
 }
@@ -78,8 +81,8 @@ myWindow        (NULL),
 myLastFrameTime (0.f),
 myIsExternal    (true),
 myFramerateLimit(0),
-mySetCursorPosX (-1),
-mySetCursorPosY (-1)
+mySetCursorPosX (0xFFFF),
+mySetCursorPosY (0xFFFF)
 {
     Create(Handle, Params);
 }
@@ -100,20 +103,39 @@ Window::~Window()
 ////////////////////////////////////////////////////////////
 void Window::Create(VideoMode Mode, const std::string& Title, unsigned long WindowStyle, const WindowSettings& Params)
 {
-    // Make sure we have a valid OpenGL context
-    ForceContextInit();
+    // Destroy the previous window implementation
+    Close();
 
-    // Check validity of video mode
-    if ((WindowStyle & Style::Fullscreen) && !Mode.IsValid())
+    // Fullscreen style requires some tests
+    if (WindowStyle & Style::Fullscreen)
     {
-        std::cerr << "The requested video mode is not available, switching to a valid mode" << std::endl;
-        Mode = VideoMode::GetMode(0);
+        // Make sure there's not already a fullscreen window (only one is allowed)
+        if (FullscreenWindow)
+        {
+            std::cerr << "Creating two fullscreen windows is not allowed, switching to windowed mode" << std::endl;
+            WindowStyle &= ~Style::Fullscreen;
+        }
+        else
+        {
+            // Make sure the chosen video mode is compatible
+            if (!Mode.IsValid())
+            {
+                std::cerr << "The requested video mode is not available, switching to a valid mode" << std::endl;
+                Mode = VideoMode::GetMode(0);
+            }
+
+            // Update the fullscreen window
+            FullscreenWindow = this;
+        }
     }
 
     // Check validity of style
-    if ((WindowStyle & Style::Close) || (WindowStyle & Style::Close))
+    if ((WindowStyle & Style::Close) || (WindowStyle & Style::Resize))
         WindowStyle |= Style::Titlebar;
 
+    // Activate the global context
+    Context::GetGlobal().SetActive(true);
+
     mySettings = Params;
     Initialize(priv::WindowImpl::New(Mode, Title, WindowStyle, mySettings));
 }
@@ -124,8 +146,11 @@ void Window::Create(VideoMode Mode, const std::string& Title, unsigned long Wind
 ////////////////////////////////////////////////////////////
 void Window::Create(WindowHandle Handle, const WindowSettings& Params)
 {
-    // Make sure we have a valid OpenGL context
-    ForceContextInit();
+    // Destroy the previous window implementation
+    Close();
+
+    // Activate the global context
+    Context::GetGlobal().SetActive(true);
 
     mySettings = Params;
     Initialize(priv::WindowImpl::New(Handle, mySettings));
@@ -139,13 +164,13 @@ void Window::Create(WindowHandle Handle, const WindowSettings& Params)
 ////////////////////////////////////////////////////////////
 void Window::Close()
 {
-    // Make sure we always have a valid OpenGL context
-    if (ourDummyWindow)
-        ourDummyWindow->SetActive(true);
-
     // Delete the window implementation
     delete myWindow;
     myWindow = NULL;
+
+    // Update the fullscreen window
+    if (this == FullscreenWindow)
+        FullscreenWindow = NULL;
 }
 
 
@@ -262,6 +287,16 @@ void Window::SetPosition(int Left, int Top)
 }
 
 
+////////////////////////////////////////////////////////////
+/// Change the size of the rendering region of the window
+////////////////////////////////////////////////////////////
+void Window::SetSize(unsigned int Width, unsigned int Height)
+{
+    if (myWindow)
+        myWindow->SetSize(Width, Height);
+}
+
+
 ////////////////////////////////////////////////////////////
 /// Show or hide the window
 ////////////////////////////////////////////////////////////
@@ -286,6 +321,16 @@ void Window::EnableKeyRepeat(bool Enabled)
 }
 
 
+////////////////////////////////////////////////////////////
+/// Change the window's icon
+////////////////////////////////////////////////////////////
+void Window::SetIcon(unsigned int Width, unsigned int Height, const Uint8* Pixels)
+{
+    if (myWindow)
+        myWindow->SetIcon(Width, Height, Pixels);
+}
+
+
 ////////////////////////////////////////////////////////////
 /// Activate of deactivate the window as the current target
 /// for rendering
@@ -319,14 +364,9 @@ void Window::Display()
     myLastFrameTime = myClock.GetElapsedTime();
     myClock.Reset();
 
+    // Display the backbuffer on screen
     if (SetActive())
-    {
-        // Display the backbuffer on screen
         myWindow->Display();
-
-        // Notify the derived class
-        OnDisplay();
-    }
 }
 
 
@@ -368,19 +408,6 @@ void Window::SetJoystickThreshold(float Threshold)
 }
 
 
-////////////////////////////////////////////////////////////
-/// Force a valid OpenGL context to exist even if
-/// no window has been created
-////////////////////////////////////////////////////////////
-void Window::ForceContextInit()
-{
-    // We force a context to exist by creating a dummy window
-    // It might cause a small memory leak as this window won't be destroyed
-    if (!ourDummyWindow)
-        ourDummyWindow = sf::priv::WindowImpl::New();
-}
-
-
 ////////////////////////////////////////////////////////////
 /// Called after the window has been created
 ////////////////////////////////////////////////////////////
@@ -390,15 +417,6 @@ void Window::OnCreate()
 }
 
 
-////////////////////////////////////////////////////////////
-/// Called when the window displays its content on screen
-////////////////////////////////////////////////////////////
-void Window::OnDisplay()
-{
-    // Nothing by default
-}
-
-
 ////////////////////////////////////////////////////////////
 /// Receive an event from window
 ////////////////////////////////////////////////////////////
@@ -406,11 +424,11 @@ void Window::OnEvent(const Event& EventReceived)
 {
     // Discard MouseMove events generated by SetCursorPosition
     if ((EventReceived.Type        == Event::MouseMoved) &&
-        (EventReceived.MouseMove.X == static_cast<unsigned int>(mySetCursorPosX)) &&
-        (EventReceived.MouseMove.Y == static_cast<unsigned int>(mySetCursorPosY)))
+        (EventReceived.MouseMove.X == mySetCursorPosX)   &&
+        (EventReceived.MouseMove.Y == mySetCursorPosY))
     {
-        mySetCursorPosX = -1;
-        mySetCursorPosY = -1;
+        mySetCursorPosX = 0xFFFF;
+        mySetCursorPosY = 0xFFFF;
         return;
     }
 
@@ -423,14 +441,16 @@ void Window::OnEvent(const Event& EventReceived)
 ////////////////////////////////////////////////////////////
 void Window::Initialize(priv::WindowImpl* Window)
 {
-    // Destroy window if already created
-    delete myWindow;
-
-    // Assign new window and listen to its events
+    // Assign and initialize the new window
     myWindow = Window;
-    myWindow->AddListener(this);
+    myWindow->Initialize();
 
-    // Attach input to the window
+    // Clear the event queue
+    while (!myEvents.empty())
+        myEvents.pop();
+
+    // Listen to events from the new window
+    myWindow->AddListener(this);
     myWindow->AddListener(&myInput);
 
     // Setup default behaviours (to get a consistent behaviour across different implementations)
@@ -443,8 +463,10 @@ void Window::Initialize(priv::WindowImpl* Window)
     myClock.Reset();
     myLastFrameTime = 0.f;
 
+    // Activate the window
+    SetActive(true);
+
     // Notify the derived class
-    SetActive();
     OnCreate();
 }
 
