@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -34,22 +34,17 @@
 #if defined(SFML_SYSTEM_WINDOWS)
 
     #include <SFML/Window/Win32/WindowImplWin32.hpp>
+    typedef sf::priv::WindowImplWin32 WindowImplType;
 
-#elif defined(SFML_SYSTEM_LINUX)
+#elif defined(SFML_SYSTEM_LINUX) || defined(SFML_SYSTEM_FREEBSD)
 
     #include <SFML/Window/Linux/WindowImplX11.hpp>
+    typedef sf::priv::WindowImplX11 WindowImplType;
 
 #elif defined(SFML_SYSTEM_MACOS)
 
-	#if defined(SFML_IMPL_CARBON)
-
-		#include <SFML/Window/OSXCarbon/WindowImplCarbon.hpp>
-
-	#elif defined(SFML_IMPL_COCOA)
-
-		#include <SFML/Window/OSXCocoa/WindowImplCocoa.hpp>
-
-	#endif
+	#include <SFML/Window/Cocoa/WindowImplCocoa.hpp>
+	typedef sf::priv::WindowImplCocoa WindowImplType;
 
 #endif
 
@@ -58,46 +53,12 @@ namespace sf
 {
 namespace priv
 {
-////////////////////////////////////////////////////////////
-// Static member data
-////////////////////////////////////////////////////////////
-const WindowImpl* WindowImpl::ourCurrent = NULL;
-
-
 ////////////////////////////////////////////////////////////
 /// Create a new window depending on the current OS
 ////////////////////////////////////////////////////////////
 WindowImpl* WindowImpl::New()
 {
-    WindowImpl* Window = NULL;
-
-    #if defined(SFML_SYSTEM_WINDOWS)
-
-        // Win32 window
-        Window = new WindowImplWin32();
-
-    #elif defined(SFML_SYSTEM_LINUX)
-
-        // Unix window
-        Window = new WindowImplX11();
-
-    #elif defined(SFML_SYSTEM_MACOS)
-
-		#if defined(SFML_IMPL_CARBON)
-		
-			// Carbon window
-			Window = new WindowImplCarbon();
-			
-		#elif defined(SFML_IMPL_COCOA)
-			
-			// Cocoa window
-			Window = new WindowImplCocoa();
-			
-		#endif
-
-    #endif
-
-    return Window;
+    return new WindowImplType();
 }
 
 
@@ -106,31 +67,7 @@ WindowImpl* WindowImpl::New()
 ////////////////////////////////////////////////////////////
 WindowImpl* WindowImpl::New(VideoMode Mode, const std::string& Title, unsigned long WindowStyle, WindowSettings& Params)
 {
-    #if defined(SFML_SYSTEM_WINDOWS)
-
-        // Win32 window
-        return new WindowImplWin32(Mode, Title, WindowStyle, Params);
-
-    #elif defined(SFML_SYSTEM_LINUX)
-
-        // Unix window
-        return new WindowImplX11(Mode, Title, WindowStyle, Params);
-
-    #elif defined(SFML_SYSTEM_MACOS)
-
-		#if defined(SFML_IMPL_CARBON)
-		
-			// Carbon window
-			return new WindowImplCarbon(Mode, Title, WindowStyle, Params);
-		
-		#elif defined(SFML_IMPL_COCOA)
-		
-			// Cocoa window
-			return new WindowImplCocoa(Mode, Title, WindowStyle, Params);
-		
-		#endif
-
-    #endif
+    return new WindowImplType(Mode, Title, WindowStyle, Params);
 }
 
 
@@ -139,30 +76,7 @@ WindowImpl* WindowImpl::New(VideoMode Mode, const std::string& Title, unsigned l
 ////////////////////////////////////////////////////////////
 WindowImpl* WindowImpl::New(WindowHandle Handle, WindowSettings& Params)
 {
-    #if defined(SFML_SYSTEM_WINDOWS)
-
-        // Win32 window
-        return new WindowImplWin32(Handle, Params);
-
-    #elif defined(SFML_SYSTEM_LINUX)
-
-        // Unix window
-        return new WindowImplX11(Handle, Params);
-
-    #elif defined(SFML_SYSTEM_MACOS)
-
-		#if defined(SFML_IMPL_CARBON)
-		
-			// Carbon window
-			return new WindowImplCarbon(Handle, Params);
-			
-		#elif defined(SFML_IMPL_COCOA)
-			
-			// Cocoa window
-			return new WindowImplCocoa(Handle, Params);
-			
-		#endif
-    #endif
+    return new WindowImplType(Handle, Params);
 }
 
 
@@ -174,12 +88,6 @@ myWidth       (0),
 myHeight      (0),
 myJoyThreshold(0.1f)
 {
-    // Initialize the joysticks
-    for (unsigned int i = 0; i < JoysticksCount; ++i)
-    {
-        myJoysticks[i].Initialize(i);
-        myJoyStates[i] = myJoysticks[i].UpdateState();
-    }
 }
 
 
@@ -188,9 +96,7 @@ myJoyThreshold(0.1f)
 ////////////////////////////////////////////////////////////
 WindowImpl::~WindowImpl()
 {
-    // Reset the active window
-    if (ourCurrent == this)
-        ourCurrent = NULL;
+    // Nothing to do
 }
 
 
@@ -213,6 +119,20 @@ void WindowImpl::RemoveListener(WindowListener* Listener)
 }
 
 
+////////////////////////////////////////////////////////////
+/// Initialize window's states that can't be done at construction
+////////////////////////////////////////////////////////////
+void WindowImpl::Initialize()
+{
+    // Initialize the joysticks
+    for (unsigned int i = 0; i < Joy::Count; ++i)
+    {
+        myJoysticks[i].Initialize(i);
+        myJoyStates[i] = myJoysticks[i].UpdateState();
+    }
+}
+
+
 ////////////////////////////////////////////////////////////
 /// Get the client width of the window
 ////////////////////////////////////////////////////////////
@@ -231,25 +151,6 @@ unsigned int WindowImpl::GetHeight() const
 }
 
 
-////////////////////////////////////////////////////////////
-/// Activate of deactivate the window as the current target
-/// for rendering
-////////////////////////////////////////////////////////////
-void WindowImpl::SetActive(bool Active) const
-{
-    if (Active && (ourCurrent != this))
-    {
-        MakeActive(true);
-        ourCurrent = this;
-    }
-    else if (!Active && (ourCurrent == this))
-    {
-        MakeActive(false);
-        ourCurrent = NULL;
-    }
-}
-
-
 ////////////////////////////////////////////////////////////
 /// Change the joystick threshold, ie. the value below which
 /// no move event will be generated
@@ -273,6 +174,15 @@ void WindowImpl::DoEvents()
 }
 
 
+////////////////////////////////////////////////////////////
+/// Check if there's an active context on the current thread
+////////////////////////////////////////////////////////////
+bool WindowImpl::IsContextActive()
+{
+    return WindowImplType::IsContextActive();
+}
+
+
 ////////////////////////////////////////////////////////////
 /// Send an event to listeners
 ////////////////////////////////////////////////////////////
@@ -285,36 +195,6 @@ void WindowImpl::SendEvent(const Event& EventToSend)
 }
 
 
-////////////////////////////////////////////////////////////
-/// Convert an UTF-8 sequence to UTF-16
-////////////////////////////////////////////////////////////
-Uint16 WindowImpl::UTF8toUTF16(const Uint8* Source)
-{
-    // http://www.unicode.org/Public/PROGRAMS/CVTUTF/ConvertUTF.h
-    // http://www.unicode.org/Public/PROGRAMS/CVTUTF/ConvertUTF.c
-
-    Uint8 Byte = *Source;
-
-    int    NbBytes = 0;
-    Uint32 Offset  = 0;
-    if      (Byte < 0xC0) {NbBytes = 0; Offset = 0x00000000;}
-    else if (Byte < 0xE0) {NbBytes = 1; Offset = 0x00003080;}
-    else if (Byte < 0xF0) {NbBytes = 2; Offset = 0x000E2080;}
-    else if (Byte < 0xF8) {NbBytes = 3; Offset = 0x03C82080;}
-    else if (Byte < 0xFC) {NbBytes = 4; Offset = 0xFA082080;}
-    else                  {NbBytes = 5; Offset = 0x82082080;}
-
-    Uint16 Result = 0;
-    for (int i = 0; i < NbBytes; ++i)
-    {
-        Result = static_cast<Uint16>(Result + *Source++);
-        Result <<= 6;
-    }
-
-    return static_cast<Uint16>(Result + *Source - Offset);
-}
-
-
 ////////////////////////////////////////////////////////////
 /// Evaluate a pixel format configuration.
 /// This functions can be used by implementations that have
@@ -334,25 +214,29 @@ int WindowImpl::EvaluateConfig(const VideoMode& Mode, const WindowSettings& Sett
 ////////////////////////////////////////////////////////////
 void WindowImpl::ProcessJoystickEvents()
 {
-    for (unsigned int i = 0; i < JoysticksCount; ++i)
+    for (unsigned int i = 0; i < Joy::Count; ++i)
     {
         // Copy the previous state of the joystick and get the new one
         JoystickState PreviousState = myJoyStates[i];
         myJoyStates[i] = myJoysticks[i].UpdateState();
 
         // Axis
-        for (unsigned int j = 0; j < myJoysticks[i].GetAxesCount(); ++j)
+        for (unsigned int j = 0; j < Joy::AxisCount; ++j)
         {
-            float PrevPos = PreviousState.Axis[j];
-            float CurrPos = myJoyStates[i].Axis[j];
-            if (fabs(CurrPos - PrevPos) >= myJoyThreshold)
+            Joy::Axis Axis = static_cast<Joy::Axis>(j);
+            if (myJoysticks[i].HasAxis(Axis))
             {
-                Event Event;
-                Event.Type               = Event::JoyMoved;
-                Event.JoyMove.JoystickId = i;
-                Event.JoyMove.Axis       = static_cast<Joy::Axis>(j);
-                Event.JoyMove.Position   = CurrPos;
-                SendEvent(Event);
+                float PrevPos = PreviousState.Axis[j];
+                float CurrPos = myJoyStates[i].Axis[j];
+                if (fabs(CurrPos - PrevPos) >= myJoyThreshold)
+                {
+                    Event Event;
+                    Event.Type               = Event::JoyMoved;
+                    Event.JoyMove.JoystickId = i;
+                    Event.JoyMove.Axis       = Axis;
+                    Event.JoyMove.Position   = CurrPos;
+                    SendEvent(Event);
+                }
             }
         }
 
