@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2008 Laurent Gomila (laurent.gom@gmail.com)
+// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -108,6 +108,12 @@ public :
     ////////////////////////////////////////////////////////////
     void RemoveListener(WindowListener* Listener);
 
+    ////////////////////////////////////////////////////////////
+    /// Initialize window's states that can't be done at construction
+    ///
+    ////////////////////////////////////////////////////////////
+    void Initialize();
+
     ////////////////////////////////////////////////////////////
     /// Get the client width of the window
     ///
@@ -131,7 +137,7 @@ public :
     /// \param Active : True to activate, false to deactivate (true by default)
     ///
     ////////////////////////////////////////////////////////////
-    void SetActive(bool Active = true) const;
+    virtual void SetActive(bool Active = true) const = 0;
 
     ////////////////////////////////////////////////////////////
     /// Change the joystick threshold, ie. the value below which
@@ -148,6 +154,14 @@ public :
     ////////////////////////////////////////////////////////////
     void DoEvents();
 
+    ////////////////////////////////////////////////////////////
+    /// Check if there's an active context on the current thread
+    ///
+    /// \return True if there's a context bound to the current thread
+    ///
+    ////////////////////////////////////////////////////////////
+    static bool IsContextActive();
+
     ////////////////////////////////////////////////////////////
     /// Display the window on screen
     ///
@@ -188,6 +202,15 @@ public :
     ////////////////////////////////////////////////////////////
     virtual void SetPosition(int Left, int Top) = 0;
 
+    ////////////////////////////////////////////////////////////
+    /// Change the size of the rendering region of the window
+    ///
+    /// \param Width :  New width
+    /// \param Height : New height
+    ///
+    ////////////////////////////////////////////////////////////
+    virtual void SetSize(unsigned int Width, unsigned int Height) = 0;
+
     ////////////////////////////////////////////////////////////
     /// Show or hide the window
     ///
@@ -204,6 +227,16 @@ public :
     ////////////////////////////////////////////////////////////
     virtual void EnableKeyRepeat(bool Enabled) = 0;
 
+    ////////////////////////////////////////////////////////////
+    /// Change the window's icon
+    ///
+    /// \param Width :  Icon's width, in pixels
+    /// \param Height : Icon's height, in pixels
+    /// \param Pixels : Pointer to the pixels in memory, format must be RGBA 32 bits
+    ///
+    ////////////////////////////////////////////////////////////
+    virtual void SetIcon(unsigned int Width, unsigned int Height, const Uint8* Pixels) = 0;
+
 protected :
 
     ////////////////////////////////////////////////////////////
@@ -220,16 +253,6 @@ protected :
     ////////////////////////////////////////////////////////////
     void SendEvent(const Event& EventToSend);
 
-    ////////////////////////////////////////////////////////////
-    /// Convert an UTF-8 sequence to UTF-16
-    ///
-    /// \param Source : UTF-8 characters sequence to convert
-    ///
-    /// \return UTF-16 character
-    ///
-    ////////////////////////////////////////////////////////////
-    static Uint16 UTF8toUTF16(const Uint8* Source);
-
     ////////////////////////////////////////////////////////////
     /// Evaluate a pixel format configuration.
     /// This functions can be used by implementations that have
@@ -267,32 +290,13 @@ private :
     ////////////////////////////////////////////////////////////
     virtual void ProcessEvents() = 0;
 
-    ////////////////////////////////////////////////////////////
-    /// Activate of deactivate the window as the current target
-    /// for rendering
-    ///
-    /// \param Active : True to activate, false to deactivate (true by default)
-    ///
-    ////////////////////////////////////////////////////////////
-    virtual void MakeActive(bool Active = true) const = 0;
-
-    ////////////////////////////////////////////////////////////
-    // Total number of joysticks supported
-    ////////////////////////////////////////////////////////////
-    enum {JoysticksCount = 2};
-
-    ////////////////////////////////////////////////////////////
-    // Static member data
-    ////////////////////////////////////////////////////////////
-    static const WindowImpl* ourCurrent; ///< Current active window (to avoid setting the same one multiple times)
-
     ////////////////////////////////////////////////////////////
     // Member data
     ////////////////////////////////////////////////////////////
-    std::set<WindowListener*> myListeners;                 ///< Array of listeners connected to the window
-    Joystick                  myJoysticks[JoysticksCount]; ///< Joysticks to observe
-    JoystickState             myJoyStates[JoysticksCount]; ///< Current states of the joysticks
-    float                     myJoyThreshold;              ///< Joystick threshold (minimum motion for MOVE event to be generated)
+    std::set<WindowListener*> myListeners;             ///< Array of listeners connected to the window
+    Joystick                  myJoysticks[Joy::Count]; ///< Joysticks to observe
+    JoystickState             myJoyStates[Joy::Count]; ///< Current states of the joysticks
+    float                     myJoyThreshold;          ///< Joystick threshold (minimum motion for MOVE event to be generated)
 };
 
 } // namespace priv
